@@ -31,7 +31,6 @@
 #include "taler-auditor-httpd.h"
 #include "taler-auditor-httpd_deposit-confirmation.h"
 
-
 GNUNET_NETWORK_STRUCT_BEGIN
 
 /**
@@ -118,11 +117,11 @@ verify_and_execute_deposit_confirmation (
   const struct TALER_CoinSpendSignatureP *coin_sigps[
     GNUNET_NZL (dc->num_coins)];
 
-  for (unsigned int i = 0; i<dc->num_coins; i++)
+  for (unsigned int i = 0; i < dc->num_coins; i++)
     coin_sigps[i] = &dc->coin_sigs[i];
 
   if (GNUNET_TIME_absolute_is_future (es->ep_start.abs_time) ||
-      GNUNET_TIME_absolute_is_past (es->ep_expire.abs_time) )
+      GNUNET_TIME_absolute_is_past (es->ep_expire.abs_time))
   {
     /* Signing key expired */
     TALER_LOG_WARNING ("Expired exchange signing key\n");
@@ -134,7 +133,7 @@ verify_and_execute_deposit_confirmation (
 
   /* check our cache */
   GNUNET_CRYPTO_hash (&skv,
-                      sizeof (skv),
+                      sizeof(skv),
                       &h);
   GNUNET_assert (0 == pthread_mutex_lock (&lock));
   cached = GNUNET_CONTAINER_multihashmap_get (cache,
@@ -158,7 +157,7 @@ verify_and_execute_deposit_confirmation (
           es->ep_start,
           es->ep_expire,
           es->ep_end,
-          &es->master_public_key,
+          &TAH_master_public_key,
           &es->master_sig))
     {
       TALER_LOG_WARNING ("Invalid signature on exchange signing key\n");
@@ -312,8 +311,6 @@ TAH_DEPOSIT_CONFIRMATION_handler (
                                  &dc.exchange_sig),
     GNUNET_JSON_spec_fixed_auto ("exchange_pub",
                                  &dc.exchange_pub),
-    GNUNET_JSON_spec_fixed_auto ("master_pub",
-                                 &es.master_public_key),
     GNUNET_JSON_spec_timestamp ("ep_start",
                                 &es.ep_start),
     GNUNET_JSON_spec_timestamp ("ep_expire",
@@ -341,8 +338,8 @@ TAH_DEPOSIT_CONFIRMATION_handler (
                                      &json);
     if (GNUNET_SYSERR == res)
       return MHD_NO;
-    if ( (GNUNET_NO == res) ||
-         (NULL == json) )
+    if ((GNUNET_NO == res) ||
+        (NULL == json))
       return MHD_YES;
     res = TALER_MHD_parse_json_data (connection,
                                      json,
@@ -350,12 +347,12 @@ TAH_DEPOSIT_CONFIRMATION_handler (
     if (GNUNET_SYSERR == res)
     {
       json_decref (json);
-      return MHD_NO; /* hard failure */
+      return MHD_NO;       /* hard failure */
     }
     if (GNUNET_NO == res)
     {
       json_decref (json);
-      return MHD_YES; /* failure */
+      return MHD_YES;       /* failure */
     }
   }
   num_coins = json_array_size (jcoin_sigs);
@@ -382,7 +379,7 @@ TAH_DEPOSIT_CONFIRMATION_handler (
     struct TALER_CoinSpendSignatureP coin_sigs[num_coins];
     MHD_RESULT res;
 
-    for (unsigned int i = 0; i<num_coins; i++)
+    for (unsigned int i = 0; i < num_coins; i++)
     {
       json_t *jpub = json_array_get (jcoin_pubs,
                                      i);
@@ -391,12 +388,12 @@ TAH_DEPOSIT_CONFIRMATION_handler (
       const char *ps = json_string_value (jpub);
       const char *ss = json_string_value (jsig);
 
-      if ( (NULL == ps) ||
-           (GNUNET_OK !=
-            GNUNET_STRINGS_string_to_data (ps,
-                                           strlen (ps),
-                                           &coin_pubs[i],
-                                           sizeof (coin_pubs[i]))) )
+      if ((NULL == ps) ||
+          (GNUNET_OK !=
+           GNUNET_STRINGS_string_to_data (ps,
+                                          strlen (ps),
+                                          &coin_pubs[i],
+                                          sizeof(coin_pubs[i]))))
       {
         GNUNET_break_op (0);
         json_decref (json);
@@ -405,12 +402,12 @@ TAH_DEPOSIT_CONFIRMATION_handler (
           TALER_EC_GENERIC_PARAMETER_MALFORMED,
           "coin_pub[] malformed");
       }
-      if ( (NULL == ss) ||
-           (GNUNET_OK !=
-            GNUNET_STRINGS_string_to_data (ss,
-                                           strlen (ss),
-                                           &coin_sigs[i],
-                                           sizeof (coin_sigs[i]))) )
+      if ((NULL == ss) ||
+          (GNUNET_OK !=
+           GNUNET_STRINGS_string_to_data (ss,
+                                          strlen (ss),
+                                          &coin_sigs[i],
+                                          sizeof(coin_sigs[i]))))
       {
         GNUNET_break_op (0);
         json_decref (json);
@@ -423,8 +420,7 @@ TAH_DEPOSIT_CONFIRMATION_handler (
     dc.num_coins = num_coins;
     dc.coin_pubs = coin_pubs;
     dc.coin_sigs = coin_sigs;
-    es.exchange_pub = dc.exchange_pub; /* used twice! */
-    dc.master_public_key = es.master_public_key;
+    es.exchange_pub = dc.exchange_pub;     /* used twice! */
     res = verify_and_execute_deposit_confirmation (connection,
                                                    &dc,
                                                    &es);
@@ -454,6 +450,3 @@ TEAH_DEPOSIT_CONFIRMATION_done (void)
     GNUNET_assert (0 == pthread_mutex_destroy (&lock));
   }
 }
-
-
-/* end of taler-auditor-httpd_deposit-confirmation.c */
