@@ -108,7 +108,8 @@ static struct Table tables[] = {
   { .rt = TALER_EXCHANGEDB_RT_REFRESH_COMMITMENTS},
   { .rt = TALER_EXCHANGEDB_RT_REFRESH_REVEALED_COINS},
   { .rt = TALER_EXCHANGEDB_RT_REFRESH_TRANSFER_KEYS},
-  { .rt = TALER_EXCHANGEDB_RT_DEPOSITS},
+  { .rt = TALER_EXCHANGEDB_RT_BATCH_DEPOSITS},
+  { .rt = TALER_EXCHANGEDB_RT_COIN_DEPOSITS},
   { .rt = TALER_EXCHANGEDB_RT_REFUNDS},
   { .rt = TALER_EXCHANGEDB_RT_WIRE_OUT},
   { .rt = TALER_EXCHANGEDB_RT_AGGREGATION_TRACKING},
@@ -605,6 +606,9 @@ main (int argc,
                                    level,
                                    NULL));
   GNUNET_free (level);
+  /* suppress compiler warnings... */
+  GNUNET_assert (NULL != src_cfgfile);
+  GNUNET_assert (NULL != dst_cfgfile);
   if (0 == strcmp (src_cfgfile,
                    dst_cfgfile))
   {
