@@ -46,6 +46,7 @@
 #include "taler-exchange-httpd_kyc-check.h"
 #include "taler-exchange-httpd_kyc-proof.h"
 #include "taler-exchange-httpd_kyc-wallet.h"
+#include "taler-exchange-httpd_kyc-webhook.h"
 #include "taler-exchange-httpd_link.h"
 #include "taler-exchange-httpd_management.h"
 #include "taler-exchange-httpd_melt.h"
@@ -67,9 +68,9 @@
 #include "taler-exchange-httpd_reserves_history.h"
 #include "taler-exchange-httpd_reserves_open.h"
 #include "taler-exchange-httpd_reserves_purse.h"
+#include "taler-exchange-httpd_spa.h"
 #include "taler-exchange-httpd_terms.h"
 #include "taler-exchange-httpd_transfers_get.h"
-#include "taler-exchange-httpd_withdraw.h"
 #include "taler_exchangedb_lib.h"
 #include "taler_exchangedb_plugin.h"
 #include "taler_extensions.h"
@@ -161,13 +162,12 @@ struct TALER_Amount TEH_stefan_log;
 /**
  * Linear STEFAN parameter.
  */
-struct TALER_Amount TEH_stefan_lin;
+float TEH_stefan_lin;
 
 /**
- * Default number of fractional digits to render
- * amounts with.
+ * Where to redirect users from "/"?
  */
-unsigned int TEH_currency_fraction_digits;
+static char *toplevel_redirect_url;
 
 /**
  * Our currency.
@@ -252,6 +252,22 @@ static unsigned long long active_connections;
  */
 static unsigned long long req_max;
 
+/**
+ * Length of the cspecs array.
+ */
+static unsigned int num_cspecs;
+
+/**
+ * Rendering specs for currencies.
+ */
+static struct TALER_CurrencySpecification *cspecs;
+
+/**
+ * Rendering spec for our currency.
+ */
+const struct TALER_CurrencySpecification *TEH_cspec;
+
+
 /**
  * Context for all CURL operations (useful to the event loop)
  */
@@ -373,7 +389,6 @@ handle_post_coins (struct TEH_RequestContext *rc,
  * EdDSA key of the coin and demultiplexes based on $OP.
  *
  * @param rc request context
- * @param root uploaded JSON data
  * @param args array of additional options
  * @return MHD result code
  */
@@ -730,10 +745,6 @@ handle_post_reserves (struct TEH_RequestContext *rc,
       .op = "age-withdraw",
       .handler = &TEH_handler_age_withdraw
     },
-    {
-      .op = "withdraw",
-      .handler = &TEH_handler_withdraw
-    },
     {
       .op = "purse",
       .handler = &TEH_handler_reserves_purse
@@ -1010,6 +1021,11 @@ handle_mhd_completion_callback (void *cls,
   TEH_check_invariants ();
   if (NULL != rc->rh_cleaner)
     rc->rh_cleaner (rc);
+  if (NULL != rc->root)
+  {
+    json_decref (rc->root);
+    rc->root = NULL;
+  }
   TEH_check_invariants ();
   {
 #if MHD_VERSION >= 0x00097304
@@ -1076,7 +1092,6 @@ proceed_with_handler (struct TEH_RequestContext *rc,
   const struct TEH_RequestHandler *rh = rc->rh;
   const char *args[rh->nargs + 2];
   size_t ulen = strlen (url) + 1;
-  json_t *root = NULL;
   MHD_RESULT ret;
 
   /* We do check for "ulen" here, because we'll later stack-allocate a buffer
@@ -1097,8 +1112,9 @@ proceed_with_handler (struct TEH_RequestContext *rc,
 
   /* All POST endpoints come with a body in JSON format. So we parse
      the JSON here. */
-  if (0 == strcasecmp (rh->method,
-                       MHD_HTTP_METHOD_POST))
+  if ( (0 == strcasecmp (rh->method,
+                         MHD_HTTP_METHOD_POST)) &&
+       (NULL == rc->root) )
   {
     enum GNUNET_GenericReturnValue res;
 
@@ -1106,16 +1122,16 @@ proceed_with_handler (struct TEH_RequestContext *rc,
                                      &rc->opaque_post_parsing_context,
                                      upload_data,
                                      upload_data_size,
-                                     &root);
+                                     &rc->root);
     if (GNUNET_SYSERR == res)
     {
-      GNUNET_assert (NULL == root);
+      GNUNET_assert (NULL == rc->root);
       return MHD_NO; /* bad upload, could not even generate error */
     }
     if ( (GNUNET_NO == res) ||
-         (NULL == root) )
+         (NULL == rc->root) )
     {
-      GNUNET_assert (NULL == root);
+      GNUNET_assert (NULL == rc->root);
       return MHD_YES; /* so far incomplete upload or parser error */
     }
   }
@@ -1152,7 +1168,6 @@ proceed_with_handler (struct TEH_RequestContext *rc,
                        rh->url,
                        url);
       GNUNET_break_op (0);
-      json_decref (root);
       return TALER_MHD_reply_with_error (rc->connection,
                                          MHD_HTTP_NOT_FOUND,
                                          TALER_EC_EXCHANGE_GENERIC_WRONG_NUMBER_OF_SEGMENTS,
@@ -1165,7 +1180,7 @@ proceed_with_handler (struct TEH_RequestContext *rc,
     if (0 == strcasecmp (rh->method,
                          MHD_HTTP_METHOD_POST))
       ret = rh->handler.post (rc,
-                              root,
+                              rc->root,
                               args);
     else if (0 == strcasecmp (rh->method,
                               MHD_HTTP_METHOD_DELETE))
@@ -1175,7 +1190,6 @@ proceed_with_handler (struct TEH_RequestContext *rc,
       ret = rh->handler.get (rc,
                              args);
   }
-  json_decref (root);
   return ret;
 }
 
@@ -1492,6 +1506,56 @@ handle_post_auditors (struct TEH_RequestContext *rc,
 }
 
 
+/**
+ * Generates the response for "/", redirecting the
+ * client to the ``toplevel_redirect_url``.
+ *
+ * @param rc request context
+ * @param args remaining arguments (should be empty)
+ * @return MHD result code
+ */
+static MHD_RESULT
+toplevel_redirect (struct TEH_RequestContext *rc,
+                   const char *const args[])
+{
+  const char *text = "Redirecting to /webui/";
+  struct MHD_Response *response;
+
+  response = MHD_create_response_from_buffer (strlen (text),
+                                              (void *) text,
+                                              MHD_RESPMEM_PERSISTENT);
+  if (NULL == response)
+  {
+    GNUNET_break (0);
+    return MHD_NO;
+  }
+  TALER_MHD_add_global_headers (response);
+  GNUNET_break (MHD_YES ==
+                MHD_add_response_header (response,
+                                         MHD_HTTP_HEADER_CONTENT_TYPE,
+                                         "text/plain"));
+  if (MHD_NO ==
+      MHD_add_response_header (response,
+                               MHD_HTTP_HEADER_LOCATION,
+                               toplevel_redirect_url))
+  {
+    GNUNET_break (0);
+    MHD_destroy_response (response);
+    return MHD_NO;
+  }
+
+  {
+    MHD_RESULT ret;
+
+    ret = MHD_queue_response (rc->connection,
+                              MHD_HTTP_FOUND,
+                              response);
+    MHD_destroy_response (response);
+    return ret;
+  }
+}
+
+
 /**
  * Handle incoming HTTP request.
  *
@@ -1525,15 +1589,11 @@ handle_mhd_request (void *cls,
       .data = "User-agent: *\nDisallow: /\n",
       .response_code = MHD_HTTP_OK
     },
-    /* Landing page, tell humans to go away. */
+    /* Landing page, redirect to toplevel_redirect_url */
     {
       .url = "",
       .method = MHD_HTTP_METHOD_GET,
-      .handler.get = TEH_handler_static_response,
-      .mime_type = "text/plain",
-      .data =
-        "Hello, I'm the Taler exchange. This HTTP server is not for humans.\n",
-      .response_code = MHD_HTTP_OK
+      .handler.get = &toplevel_redirect
     },
     /* AGPL licensing page, redirect to source. As per the AGPL-license, every
        deployment is required to offer the user a download of the source of
@@ -1712,6 +1772,20 @@ handle_mhd_request (void *cls,
       .handler.post = &TEH_handler_kyc_wallet,
       .nargs = 0
     },
+    {
+      .url = "kyc-webhook",
+      .method = MHD_HTTP_METHOD_GET,
+      .handler.get = &TEH_handler_kyc_webhook_get,
+      .nargs = 16, /* more is not plausible */
+      .nargs_is_upper_bound = true
+    },
+    {
+      .url = "kyc-webhook",
+      .method = MHD_HTTP_METHOD_POST,
+      .handler.post = &TEH_handler_kyc_webhook_post,
+      .nargs = 16, /* more is not plausible */
+      .nargs_is_upper_bound = true
+    },
     /* POST management endpoints */
     {
       .url = "management",
@@ -1749,7 +1823,13 @@ handle_mhd_request (void *cls,
       .handler.post = &handle_post_aml,
       .nargs = 2
     },
-
+    {
+      .url = "webui",
+      .method = MHD_HTTP_METHOD_GET,
+      .handler.get = &TEH_handler_spa,
+      .nargs = 1,
+      .nargs_is_upper_bound = true
+    },
 
     /* mark end of list */
     {
@@ -1979,6 +2059,11 @@ handle_mhd_request (void *cls,
 static enum GNUNET_GenericReturnValue
 exchange_serve_process_config (void)
 {
+  static struct TALER_CurrencySpecification defspec = {
+    .num_fractional_input_digits = 2,
+    .num_fractional_normal_digits = 2,
+    .num_fractional_trailing_zero_digits = 2
+  };
   if (GNUNET_OK !=
       TALER_KYCLOGIC_kyc_init (TEH_cfg))
   {
@@ -2030,6 +2115,14 @@ exchange_serve_process_config (void)
                                "KYC_AML_TRIGGER");
     return GNUNET_SYSERR;
   }
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_string (TEH_cfg,
+                                             "exchange",
+                                             "TOPLEVEL_REDIRECT_URL",
+                                             &toplevel_redirect_url))
+  {
+    toplevel_redirect_url = GNUNET_strdup ("/terms");
+  }
   if (GNUNET_OK !=
       TALER_config_get_currency (TEH_cfg,
                                  &TEH_currency))
@@ -2039,24 +2132,41 @@ exchange_serve_process_config (void)
                                "CURRENCY");
     return GNUNET_SYSERR;
   }
-  {
-    unsigned long long cfd;
 
-    if (GNUNET_OK !=
-        GNUNET_CONFIGURATION_get_value_number (TEH_cfg,
-                                               "exchange",
-                                               "CURRENCY_FRACTION_DIGITS",
-                                               &cfd))
-      cfd = 0;
-    if (cfd > 8)
+  if (GNUNET_OK !=
+      TALER_CONFIG_parse_currencies (TEH_cfg,
+                                     &num_cspecs,
+                                     &cspecs))
+    return GNUNET_SYSERR;
+  for (unsigned int i = 0; i<num_cspecs; i++)
+  {
+    struct TALER_CurrencySpecification *cspec;
+
+    cspec = &cspecs[i];
+    if (0 == strcmp (TEH_currency,
+                     cspec->currency))
     {
-      GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
-                                 "taler",
-                                 "CURRENCY_FRACTION_DIGITS",
-                                 "Value must be below 8");
-      return GNUNET_SYSERR;
+      TEH_cspec = cspec;
+      break;
     }
-    TEH_currency_fraction_digits = (unsigned int) cfd;
+  }
+  if (NULL == TEH_cspec)
+  {
+    GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_WARNING,
+                               "taler",
+                               "CURRENCY",
+                               "Lacking enabled currency specification for the given currency, using default");
+    defspec.map_alt_unit_names
+      = GNUNET_JSON_PACK (
+          GNUNET_JSON_pack_string ("0",
+                                   TEH_currency)
+          );
+    defspec.name = TEH_currency;
+    GNUNET_assert (strlen (TEH_currency) <
+                   sizeof (defspec.currency));
+    strcpy (defspec.currency,
+            TEH_currency);
+    TEH_cspec = &defspec;
   }
   if (GNUNET_OK !=
       TALER_config_get_amount (TEH_cfg,
@@ -2089,14 +2199,12 @@ exchange_serve_process_config (void)
                                           &TEH_stefan_log));
   }
   if (GNUNET_OK !=
-      TALER_config_get_amount (TEH_cfg,
-                               "exchange",
-                               "STEFAN_LIN",
-                               &TEH_stefan_lin))
+      GNUNET_CONFIGURATION_get_value_float (TEH_cfg,
+                                            "exchange",
+                                            "STEFAN_LIN",
+                                            &TEH_stefan_lin))
   {
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_set_zero (TEH_currency,
-                                          &TEH_stefan_lin));
+    TEH_stefan_lin = 0.0f;
   }
 
   if (0 != strcmp (TEH_currency,
@@ -2152,11 +2260,10 @@ exchange_serve_process_config (void)
       return GNUNET_SYSERR;
     }
     if (GNUNET_OK !=
-        GNUNET_CRYPTO_eddsa_public_key_from_string (master_public_key_str,
-                                                    strlen (
-                                                      master_public_key_str),
-                                                    &TEH_master_public_key.
-                                                    eddsa_pub))
+        GNUNET_CRYPTO_eddsa_public_key_from_string (
+          master_public_key_str,
+          strlen (master_public_key_str),
+          &TEH_master_public_key.eddsa_pub))
     {
       GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
                                  "exchange",
@@ -2349,7 +2456,9 @@ run_single_request (void)
   xfork = fork ();
   if (-1 == xfork)
   {
-    global_ret = EXIT_FAILURE;
+    GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR,
+                         "fork");
+    global_ret = EXIT_NO_RESTART;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -2466,6 +2575,11 @@ do_shutdown (void *cls)
     exchange_curl_rc = NULL;
   }
   TALER_TEMPLATING_done ();
+  TEH_cspec = NULL;
+  TALER_CONFIG_free_currencies (num_cspecs,
+                                cspecs);
+  num_cspecs = 0;
+  cspecs = NULL;
 }
 
 
@@ -2503,38 +2617,48 @@ run (void *cls,
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
+  if (GNUNET_OK !=
+      TEH_spa_init ())
+  {
+    global_ret = EXIT_NOTCONFIGURED;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
   if (GNUNET_OK !=
       TALER_TEMPLATING_init ("exchange"))
   {
-    global_ret = EXIT_FAILURE;
+    global_ret = EXIT_NOTINSTALLED;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
   if (GNUNET_SYSERR ==
       TEH_plugin->preflight (TEH_plugin->cls))
   {
-    global_ret = EXIT_FAILURE;
+    GNUNET_break (0);
+    global_ret = EXIT_NO_RESTART;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
   if (GNUNET_OK !=
       TEH_extensions_init ())
   {
-    global_ret = EXIT_FAILURE;
+    global_ret = EXIT_NOTINSTALLED;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
   if (GNUNET_OK !=
       TEH_keys_init ())
   {
-    global_ret = EXIT_FAILURE;
+    GNUNET_break (0);
+    global_ret = EXIT_NO_RESTART;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
   if (GNUNET_OK !=
       TEH_wire_init ())
   {
-    global_ret = EXIT_FAILURE;
+    GNUNET_break (0);
+    global_ret = EXIT_NO_RESTART;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -2546,7 +2670,7 @@ run (void *cls,
   if (NULL == TEH_curl_ctx)
   {
     GNUNET_break (0);
-    global_ret = EXIT_FAILURE;
+    global_ret = EXIT_NO_RESTART;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -2599,7 +2723,6 @@ run (void *cls,
   global_ret = EXIT_SUCCESS;
   TALER_MHD_daemon_start (mhd);
   atexit (&write_stats);
-
 #if HAVE_DEVELOPER
   if (NULL != input_filename)
     run_single_request ();
