@@ -69,10 +69,9 @@ struct AgeWithdrawContext
   uint32_t num_coins;
 
   /**
-   * kappa * #num_coins hashes of blinded coin planchets.
-   * FIXME[oec]: Make the [][] structure more explicit.
+   * #num_coins * #kappa hashes of blinded coin planchets.
    */
-  struct TALER_BlindedPlanchet *coin_evs;
+  struct TALER_BlindedPlanchet (*coin_evs) [ TALER_CNC_KAPPA];
 
   /**
    * #num_coins hashes of the denominations from which the coins are withdrawn.
@@ -189,13 +188,13 @@ parse_age_withdraw_json (
     }
   };
 
-  /* no overflow because
-   *   num_coins <= TALER_MAX_FRESH_COINS
-   * and
-   *    TALER_MAX_FRESH_COINS * TALER_CNC_KAPPA < INT_MAX
-   */
-  awc->coin_evs = GNUNET_new_array (awc->num_coins * TALER_CNC_KAPPA,
-                                    struct TALER_BlindedPlanchet);
+  {
+    typedef struct TALER_BlindedPlanchet
+      _array_of_kappa_planchets[TALER_CNC_KAPPA];
+
+    awc->coin_evs = GNUNET_new_array (awc->num_coins,
+                                      _array_of_kappa_planchets);
+  }
 
   hash_context = GNUNET_CRYPTO_hash_context_start ();
   GNUNET_assert (NULL != hash_context);
@@ -225,17 +224,20 @@ parse_age_withdraw_json (
     /* Now parse the individual kappa envelopes and calculate the hash of
      * the commitment along the way. */
     {
-      size_t off = idx * TALER_CNC_KAPPA;
       unsigned int kappa = 0;
-      enum GNUNET_GenericReturnValue ret;
 
       json_array_foreach (j_kappa_coin_evs, kappa, value) {
         struct GNUNET_JSON_Specification spec[] = {
-          TALER_JSON_spec_blinded_planchet (NULL, &awc->coin_evs[off + kappa]),
+          TALER_JSON_spec_blinded_planchet (NULL,
+                                            &awc->coin_evs[idx][kappa]),
           GNUNET_JSON_spec_end ()
         };
+
         if (GNUNET_OK !=
-            GNUNET_JSON_parse (value, spec, NULL, NULL))
+            GNUNET_JSON_parse (value,
+                               spec,
+                               NULL,
+                               NULL))
         {
           GNUNET_snprintf (buf,
                            sizeof(buf),
@@ -249,12 +251,10 @@ parse_age_withdraw_json (
         /* Continue to hash of the coin candidates */
         {
           struct TALER_BlindedCoinHashP bch;
-          ret = TALER_coin_ev_hash (&awc->coin_evs[off + kappa],
-                                    &awc->denom_hs[idx],
-                                    &bch);
-
-          GNUNET_assert (GNUNET_OK == ret);
 
+          TALER_coin_ev_hash (&awc->coin_evs[idx][kappa],
+                              &awc->denom_hs[idx],
+                              &bch);
           GNUNET_CRYPTO_hash_context_read (hash_context,
                                            &bch,
                                            sizeof(bch));
@@ -263,17 +263,17 @@ parse_age_withdraw_json (
         /* Check for duplicate planchets. Technically a bug on
          * the client side that is harmless for us, but still
          * not allowed per protocol */
-        for (unsigned int i = 0; i < off + kappa; i++)
+        for (unsigned int i = 0; i < idx; i++)
         {
-          if (0 == TALER_blinded_planchet_cmp (&awc->coin_evs[off + kappa],
-                                               &awc->coin_evs[i]))
+          if (0 == TALER_blinded_planchet_cmp (&awc->coin_evs[idx][kappa],
+                                               &awc->coin_evs[i][kappa]))
           {
+            GNUNET_JSON_parse_free (spec);
             error = "duplicate planchet";
             goto EXIT;
           }
         }
       }
-
     }
   }; /* json_array_foreach over j_blinded_coin_evs */
 
@@ -308,7 +308,7 @@ EXIT:
  * @param ksh The handle to the current state of (denomination) keys in the exchange
  * @param denom_h Hash of the denomination key to check
  * @param[out] pdk On success, will contain the denomination key details
- * @param[out] result On failure, an MHD-response will be qeued and result will be set to accordingly
+ * @param[out] result On failure, an MHD-response will be queued and result will be set to accordingly
  * @return true on success (denomination valid), false otherwise
  */
 static bool
@@ -395,19 +395,19 @@ denomination_is_valid (
  * @param connection The HTTP connection to the client
  * @param len The lengths of the array @a denoms_h
  * @param denom_hs array of hashes of denomination public keys
- * @param coin_evs array of blinded coin planchets
+ * @param coin_evs array of blinded coin planchet candidates
  * @param[out] denom_serials On success, will be filled with the serial-id's of the denomination keys.  Caller must deallocate.
  * @param[out] amount_with_fee On success, will contain the committed amount including fees
  * @param[out] result In the error cases, a response will be queued with MHD and this will be the result.
- * @return GNUNET_OK if the denominations are valid and support age-restriction
- *   GNUNET_SYSERR otherwise
+ * @return #GNUNET_OK if the denominations are valid and support age-restriction
+ *   #GNUNET_SYSERR otherwise
  */
 static enum GNUNET_GenericReturnValue
 are_denominations_valid (
   struct MHD_Connection *connection,
   uint32_t len,
   const struct TALER_DenominationHashP *denom_hs,
-  const struct TALER_BlindedPlanchet *coin_evs,
+  const struct TALER_BlindedPlanchet (*coin_evs) [ TALER_CNC_KAPPA],
   uint64_t **denom_serials,
   struct TALER_Amount *amount_with_fee,
   MHD_RESULT *result)
@@ -448,13 +448,17 @@ are_denominations_valid (
       return GNUNET_SYSERR;
 
     /* Ensure the ciphers from the planchets match the denominations' */
-    if (dk->denom_pub.cipher != coin_evs[i].cipher)
+    for (uint8_t k = 0; k < TALER_CNC_KAPPA; k++)
     {
-      GNUNET_break_op (0);
-      *result = TALER_MHD_reply_with_ec (connection,
-                                         TALER_EC_EXCHANGE_GENERIC_CIPHER_MISMATCH,
-                                         NULL);
-      return GNUNET_SYSERR;
+      if (dk->denom_pub.bsign_pub_key->cipher !=
+          coin_evs[i][k].blinded_message->cipher)
+      {
+        GNUNET_break_op (0);
+        *result = TALER_MHD_reply_with_ec (connection,
+                                           TALER_EC_EXCHANGE_GENERIC_CIPHER_MISMATCH,
+                                           NULL);
+        return GNUNET_SYSERR;
+      }
     }
 
     /* Accumulate the values */
@@ -737,12 +741,14 @@ age_withdraw_transaction (void *cls,
     bool conflict = false;
     uint16_t allowed_maximum_age = 0;
     uint32_t reserve_birthday = 0;
+    struct TALER_Amount reserve_balance;
 
     qs = TEH_plugin->do_age_withdraw (TEH_plugin->cls,
                                       &awc->commitment,
                                       awc->now,
                                       &found,
                                       &balance_ok,
+                                      &reserve_balance,
                                       &age_ok,
                                       &allowed_maximum_age,
                                       &reserve_birthday,
@@ -755,14 +761,14 @@ age_withdraw_transaction (void *cls,
                                             "do_age_withdraw");
       return qs;
     }
-    else if (! found)
+    if (! found)
     {
       *mhd_ret = TALER_MHD_reply_with_ec (connection,
                                           TALER_EC_EXCHANGE_GENERIC_RESERVE_UNKNOWN,
                                           NULL);
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
-    else if (! age_ok)
+    if (! age_ok)
     {
       enum TALER_ErrorCode ec =
         TALER_EC_EXCHANGE_AGE_WITHDRAW_MAXIMUM_AGE_TOO_LARGE;
@@ -779,19 +785,20 @@ age_withdraw_transaction (void *cls,
 
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
-    else if (! balance_ok)
+    if (! balance_ok)
     {
       TEH_plugin->rollback (TEH_plugin->cls);
 
       *mhd_ret = TEH_RESPONSE_reply_reserve_insufficient_balance (
         connection,
         TALER_EC_EXCHANGE_AGE_WITHDRAW_INSUFFICIENT_FUNDS,
+        &reserve_balance,
         &awc->commitment.amount_with_fee,
         &awc->commitment.reserve_pub);
 
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
-    else if (conflict)
+    if (conflict)
     {
       /* do_age_withdraw signaled a conflict, so there MUST be an entry
        * in the DB.  Put that into the response */
@@ -826,7 +833,7 @@ age_withdraw_transaction (void *cls,
  * @param connection HTTP-connection to the client
  * @param awc The context for the current age withdraw request
  * @param[out] result On error, a HTTP-response will be queued and result set accordingly
- * @return GNUNET_OK on success, GNUNET_SYSERR otherwise
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
  */
 static enum GNUNET_GenericReturnValue
 sign_and_do_age_withdraw (
@@ -856,12 +863,12 @@ sign_and_do_age_withdraw (
     /* Pick the chosen blinded coins */
     for (uint32_t i = 0; i<awc->num_coins; i++)
     {
-      csds[i].bp = &awc->coin_evs[TALER_CNC_KAPPA * i + noreveal_index];
+      csds[i].bp = &awc->coin_evs[i][noreveal_index];
       csds[i].h_denom_pub = &awc->denom_hs[i];
     }
 
-    ec = TEH_keys_denomination_batch_sign (csds,
-                                           awc->num_coins,
+    ec = TEH_keys_denomination_batch_sign (awc->num_coins,
+                                           csds,
                                            false,
                                            denom_sigs);
     if (TALER_EC_NONE != ec)
@@ -880,7 +887,7 @@ sign_and_do_age_withdraw (
   /* Prepare the hashes of the coins for insertion */
   for (uint32_t i = 0; i<awc->num_coins; i++)
   {
-    TALER_coin_ev_hash (&awc->coin_evs[TALER_CNC_KAPPA * i + noreveal_index],
+    TALER_coin_ev_hash (&awc->coin_evs[i][noreveal_index],
                         &awc->denom_hs[i],
                         &h_coin_evs[i]);
   }
@@ -1000,7 +1007,7 @@ TEH_handler_age_withdraw (struct TEH_RequestContext *rc,
                                             &awc.commitment.h_commitment,
                                             awc.commitment.noreveal_index);
 
-  } while(0);
+  } while (0);
 
   GNUNET_JSON_parse_free (spec);
   free_age_withdraw_context_resources (&awc);
