@@ -75,7 +75,7 @@ struct BatchWithdrawContext
 {
 
   /**
-   * Public key of the reserv.
+   * Public key of the reserve.
    */
   const struct TALER_ReservePublicKeyP *reserve_pub;
 
@@ -312,6 +312,7 @@ batch_withdraw_transaction (void *cls,
   bool balance_ok = false;
   bool age_ok = false;
   uint16_t allowed_maximum_age = 0;
+  struct TALER_Amount reserve_balance;
   char *kyc_required;
   struct TALER_PaytoHashP reserve_h_payto;
 
@@ -476,6 +477,7 @@ batch_withdraw_transaction (void *cls,
                                       TEH_age_restriction_enabled,
                                       &found,
                                       &balance_ok,
+                                      &reserve_balance,
                                       &age_ok,
                                       &allowed_maximum_age,
                                       &ruuid);
@@ -521,6 +523,7 @@ batch_withdraw_transaction (void *cls,
     *mhd_ret = TEH_RESPONSE_reply_reserve_insufficient_balance (
       connection,
       TALER_EC_EXCHANGE_WITHDRAW_INSUFFICIENT_FUNDS,
+      &reserve_balance,
       &wc->batch_total,
       wc->reserve_pub);
     return GNUNET_DB_STATUS_HARD_ERROR;
@@ -531,14 +534,22 @@ batch_withdraw_transaction (void *cls,
   {
     struct PlanchetContext *pc = &wc->planchets[i];
     const struct TALER_BlindedPlanchet *bp = &pc->blinded_planchet;
-    const struct TALER_CsNonce *nonce;
+    const union GNUNET_CRYPTO_BlindSessionNonce *nonce = NULL;
     bool denom_unknown = true;
     bool conflict = true;
     bool nonce_reuse = true;
 
-    nonce = (TALER_DENOMINATION_CS == bp->cipher)
-            ? &bp->details.cs_blinded_planchet.nonce
-            : NULL;
+    switch (bp->blinded_message->cipher)
+    {
+    case GNUNET_CRYPTO_BSA_INVALID:
+      break;
+    case GNUNET_CRYPTO_BSA_RSA:
+      break;
+    case GNUNET_CRYPTO_BSA_CS:
+      nonce = (const union GNUNET_CRYPTO_BlindSessionNonce *)
+              &bp->blinded_message->details.cs_blinded_message.nonce;
+      break;
+    }
     qs = TEH_plugin->do_batch_withdraw_insert (TEH_plugin->cls,
                                                nonce,
                                                &pc->collectable,
@@ -553,7 +564,7 @@ batch_withdraw_transaction (void *cls,
         *mhd_ret = TALER_MHD_reply_with_error (connection,
                                                MHD_HTTP_INTERNAL_SERVER_ERROR,
                                                TALER_EC_GENERIC_DB_FETCH_FAILED,
-                                               "do_withdraw");
+                                               "do_batch_withdraw_insert");
       return qs;
     }
     if (denom_unknown)
@@ -623,8 +634,8 @@ prepare_transaction (const struct TEH_RequestContext *rc,
     enum TALER_ErrorCode ec;
 
     ec = TEH_keys_denomination_batch_sign (
-      csds,
       wc->planchets_length,
+      csds,
       false,
       bss);
     if (TALER_EC_NONE != ec)
@@ -794,7 +805,8 @@ parse_planchets (const struct TEH_RequestContext *rc,
       }
       return mret;
     }
-    if (dk->denom_pub.cipher != pc->blinded_planchet.cipher)
+    if (dk->denom_pub.bsign_pub_key->cipher !=
+        pc->blinded_planchet.blinded_message->cipher)
     {
       /* denomination cipher and blinded planchet cipher not the same */
       GNUNET_break_op (0);
@@ -826,17 +838,10 @@ parse_planchets (const struct TEH_RequestContext *rc,
                                          NULL);
     }
 
-    if (GNUNET_OK !=
-        TALER_coin_ev_hash (&pc->blinded_planchet,
-                            &pc->collectable.denom_pub_hash,
-                            &pc->collectable.h_coin_envelope))
-    {
-      GNUNET_break (0);
-      return TALER_MHD_reply_with_error (rc->connection,
-                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                         TALER_EC_GENERIC_INTERNAL_INVARIANT_FAILURE,
-                                         NULL);
-    }
+    TALER_coin_ev_hash (&pc->blinded_planchet,
+                        &pc->collectable.denom_pub_hash,
+                        &pc->collectable.h_coin_envelope);
+
     TEH_METRICS_num_verifications[TEH_MT_SIGNATURE_EDDSA]++;
     if (GNUNET_OK !=
         TALER_wallet_withdraw_verify (&pc->collectable.denom_pub_hash,
