@@ -68,7 +68,7 @@ struct ReserveAttestContext
   /**
    * List of requested details.
    */
-  json_t *details;
+  const json_t *details;
 
   /**
    * Client signature approving the request.
@@ -158,8 +158,6 @@ reply_reserve_attest_success (struct MHD_Connection *connection,
  * @param cls our `struct ReserveAttestContext *`
  * @param h_payto account for which the attribute data is stored
  * @param provider_section provider that must be checked
- * @param birthdate birthdate of user, in format YYYY-MM-DD; can be NULL;
- *        digits can be 0 if exact day, month or year are unknown
  * @param collection_time when was the data collected
  * @param expiration_time when does the data expire
  * @param enc_attributes_size number of bytes in @a enc_attributes
@@ -169,7 +167,6 @@ static void
 kyc_process_cb (void *cls,
                 const struct TALER_PaytoHashP *h_payto,
                 const char *provider_section,
-                const char *birthdate,
                 struct GNUNET_TIME_Timestamp collection_time,
                 struct GNUNET_TIME_Timestamp expiration_time,
                 size_t enc_attributes_size,
@@ -290,8 +287,8 @@ TEH_handler_reserves_attest (struct TEH_RequestContext *rc,
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_timestamp ("request_timestamp",
                                 &rsc.timestamp),
-    GNUNET_JSON_spec_json ("details",
-                           &rsc.details),
+    GNUNET_JSON_spec_array_const ("details",
+                                  &rsc.details),
     GNUNET_JSON_spec_fixed_auto ("reserve_sig",
                                  &rsc.reserve_sig),
     GNUNET_JSON_spec_end ()
