@@ -367,8 +367,8 @@ TEH_handler_reserves_close (struct TEH_RequestContext *rc,
     GNUNET_JSON_spec_timestamp ("request_timestamp",
                                 &rcc.timestamp),
     GNUNET_JSON_spec_mark_optional (
-      GNUNET_JSON_spec_string ("payto_uri",
-                               &rcc.payto_uri),
+      TALER_JSON_spec_payto_uri ("payto_uri",
+                                 &rcc.payto_uri),
       NULL),
     GNUNET_JSON_spec_fixed_auto ("reserve_sig",
                                  &rcc.reserve_sig),
