@@ -21,6 +21,7 @@
 #include "platform.h"
 #include "taler_error_codes.h"
 #include "taler_dbevents.h"
+#include "taler_exchangedb_plugin.h"
 #include "taler_pq_lib.h"
 #include "pg_get_coin_transactions.h"
 #include "pg_helper.h"
@@ -698,39 +699,39 @@ handle_history_entry (void *cls,
   struct CoinHistoryContext *chc = cls;
   struct PostgresClosure *pg = chc->pg;
   static const struct Work work[] = {
-    /** #TALER_EXCHANGEDB_TT_DEPOSIT */
+    [TALER_EXCHANGEDB_TT_DEPOSIT] =
     { "coin_deposits",
       "get_deposit_with_coin_pub",
       &add_coin_deposit },
-    /** #TALER_EXCHANGEDB_TT_MELT */
+    [TALER_EXCHANGEDB_TT_MELT] =
     { "refresh_commitments",
       "get_refresh_session_by_coin",
       &add_coin_melt },
-    /** #TALER_EXCHANGEDB_TT_PURSE_DEPOSIT */
+    [TALER_EXCHANGEDB_TT_PURSE_DEPOSIT] =
     { "purse_deposits",
       "get_purse_deposit_by_coin_pub",
       &add_coin_purse_deposit },
-    /** #TALER_EXCHANGEDB_TT_PURSE_REFUND */
+    [TALER_EXCHANGEDB_TT_PURSE_REFUND] =
     { "purse_decision",
       "get_purse_decision_by_coin_pub",
       &add_coin_purse_decision },
-    /** #TALER_EXCHANGEDB_TT_REFUND */
+    [TALER_EXCHANGEDB_TT_REFUND] =
     { "refunds",
       "get_refunds_by_coin",
       &add_coin_refund },
-    /** #TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP */
+    [TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP] =
     { "recoup_refresh::OLD",
       "recoup_by_old_coin",
       &add_old_coin_recoup },
-    /** #TALER_EXCHANGEDB_TT_RECOUP */
+    [TALER_EXCHANGEDB_TT_RECOUP] =
     { "recoup",
       "recoup_by_coin",
       &add_coin_recoup },
-    /** #TALER_EXCHANGEDB_TT_RECOUP_REFRESH */
+    [TALER_EXCHANGEDB_TT_RECOUP_REFRESH] =
     { "recoup_refresh::NEW",
       "recoup_by_refreshed_coin",
       &add_coin_recoup_refresh },
-    /** #TALER_EXCHANGEDB_TT_RESERVE_OPEN */
+    [TALER_EXCHANGEDB_TT_RESERVE_OPEN] =
     { "reserves_open_deposits",
       "reserve_open_by_coin",
       &add_coin_reserve_open },
@@ -810,6 +811,8 @@ TEH_PG_get_coin_transactions (
   uint64_t start_off,
   uint64_t etag_in,
   uint64_t *etag_out,
+  struct TALER_Amount *balance,
+  struct TALER_DenominationHashP *h_denom_pub,
   struct TALER_EXCHANGEDB_TransactionList **tlp)
 {
   struct PostgresClosure *pg = cls;
@@ -833,10 +836,16 @@ TEH_PG_get_coin_transactions (
               "Getting transactions for coin %s\n",
               TALER_B2S (coin_pub));
   PREPARE (pg,
-           "get_coin_history_etag",
+           "get_coin_history_etag_balance",
            "SELECT"
-           " coin_history_serial_id"
-           " FROM coin_history"
+           " ch.coin_history_serial_id"
+           ",kc.remaining"
+           ",denom.denom_pub_hash"
+           " FROM coin_history ch"
+           " JOIN known_coins kc"
+           "   USING (coin_pub)"
+           " JOIN denominations denom"
+           "   USING (denominations_serial)"
            " WHERE coin_pub=$1"
            " ORDER BY coin_history_serial_id DESC"
            " LIMIT 1;");
@@ -917,8 +926,8 @@ TEH_PG_get_coin_transactions (
            "   ON (pd.coin_pub = kc.coin_pub)"
            " JOIN denominations denoms"
            "   USING (denominations_serial)"
-           " WHERE pd.coin_pub=$1"
-           "   AND pd.purse_deposit_serial_id=$2;");
+           " WHERE pd.purse_deposit_serial_id=$2"
+           "   AND pd.coin_pub=$1;");
   PREPARE (pg,
            "get_purse_decision_by_coin_pub",
            "SELECT"
@@ -1045,6 +1054,10 @@ TEH_PG_get_coin_transactions (
     struct GNUNET_PQ_ResultSpec rs[] = {
       GNUNET_PQ_result_spec_uint64 ("coin_history_serial_id",
                                     &end),
+      GNUNET_PQ_result_spec_auto_from_type ("denom_pub_hash",
+                                            h_denom_pub),
+      TALER_PQ_RESULT_SPEC_AMOUNT ("remaining",
+                                   balance),
       GNUNET_PQ_result_spec_end
     };
 
@@ -1059,7 +1072,7 @@ TEH_PG_get_coin_transactions (
        we even need to iterate */
     qs = GNUNET_PQ_eval_prepared_singleton_select (
       pg->conn,
-      "get_coin_history_etag",
+      "get_coin_history_etag_balance",
       params,
       rs);
     switch (qs)
