@@ -51,7 +51,7 @@ compute_notify_on_reserve (const struct TALER_ReservePublicKeyP *reserve_pub)
     .reserve_pub = *reserve_pub
   };
 
-  return GNUNET_PG_get_event_notify_channel (&rep.header);
+  return GNUNET_PQ_get_event_notify_channel (&rep.header);
 }
 
 
@@ -246,6 +246,7 @@ TEH_PG_reserves_in_insert (
                                                params,
                                                &helper_cb,
                                                &ctx);
+    GNUNET_PQ_cleanup_query_params_closures (params);
     if ( (qs < 0) ||
          (GNUNET_OK != ctx.status) )
     {
