@@ -140,8 +140,7 @@ TEH_PG_select_aml_process (
            "SELECT"
            " aml_status_serial_id"
            ",h_payto"
-           ",threshold_val"
-           ",threshold_frac"
+           ",threshold"
            ",status"
            " FROM aml_status"
            " WHERE aml_status_serial_id > $2"
@@ -153,8 +152,7 @@ TEH_PG_select_aml_process (
            "SELECT"
            " aml_status_serial_id"
            ",h_payto"
-           ",threshold_val"
-           ",threshold_frac"
+           ",threshold"
            ",status"
            " FROM aml_status"
            " WHERE aml_status_serial_id < $2"
