@@ -38,7 +38,6 @@
 #include "pg_get_link_data.h"
 #include "pg_helper.h"
 #include "pg_do_reserve_open.h"
-#include "pg_do_withdraw.h"
 #include "pg_get_coin_transactions.h"
 #include "pg_get_expired_reserves.h"
 #include "pg_get_purse_request.h"
@@ -47,6 +46,7 @@
 #include "pg_insert_close_request.h"
 #include "pg_insert_records_by_table.h"
 #include "pg_insert_reserve_open_deposit.h"
+#include "pg_get_pending_kyc_requirement_process.h"
 #include "pg_iterate_kyc_reference.h"
 #include "pg_iterate_reserve_close_info.h"
 #include "pg_lookup_records_by_table.h"
@@ -121,6 +121,7 @@
 #include "pg_get_policy_details.h"
 #include "pg_persist_policy_details.h"
 #include "pg_do_deposit.h"
+#include "pg_get_wire_hash_for_contract.h"
 #include "pg_add_policy_fulfillment_proof.h"
 #include "pg_do_melt.h"
 #include "pg_do_refund.h"
@@ -130,6 +131,7 @@
 #include "pg_count_known_coins.h"
 #include "pg_ensure_coin_known.h"
 #include "pg_get_known_coin.h"
+#include "pg_get_signature_for_known_coin.h"
 #include "pg_get_coin_denomination.h"
 #include "pg_have_deposit2.h"
 #include "pg_aggregate.h"
@@ -222,7 +224,7 @@
  * slow down things a _lot_, but also provide extensive logging
  * in the Postgres database logger for performance analysis.
  */
-#define AUTO_EXPLAIN 1
+#define AUTO_EXPLAIN 0
 
 
 /**
@@ -343,21 +345,29 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     GNUNET_free (pg);
     return NULL;
   }
-  if ( (GNUNET_OK !=
-        GNUNET_CONFIGURATION_get_value_time (cfg,
-                                             "exchangedb",
-                                             "IDLE_RESERVE_EXPIRATION_TIME",
-                                             &pg->idle_reserve_expiration_time))
-       ||
-       (GNUNET_OK !=
-        GNUNET_CONFIGURATION_get_value_time (cfg,
-                                             "exchangedb",
-                                             "LEGAL_RESERVE_EXPIRATION_TIME",
-                                             &pg->legal_reserve_expiration_time)) )
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_time (cfg,
+                                           "exchangedb",
+                                           "IDLE_RESERVE_EXPIRATION_TIME",
+                                           &pg->idle_reserve_expiration_time))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                "exchangedb",
-                               "LEGAL/IDLE_RESERVE_EXPIRATION_TIME");
+                               "IDLE_RESERVE_EXPIRATION_TIME");
+    GNUNET_free (pg->exchange_url);
+    GNUNET_free (pg->sql_dir);
+    GNUNET_free (pg);
+    return NULL;
+  }
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_time (cfg,
+                                           "exchangedb",
+                                           "LEGAL_RESERVE_EXPIRATION_TIME",
+                                           &pg->legal_reserve_expiration_time))
+  {
+    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                               "exchangedb",
+                               "LEGAL_RESERVE_EXPIRATION_TIME");
     GNUNET_free (pg->exchange_url);
     GNUNET_free (pg->sql_dir);
     GNUNET_free (pg);
@@ -413,8 +423,6 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_do_reserve_open;
   plugin->drop_tables
     = &TEH_PG_drop_tables;
-  plugin->do_withdraw
-    = &TEH_PG_do_withdraw;
   plugin->free_coin_transaction_list
     = &TEH_COMMON_free_coin_transaction_list;
   plugin->free_reserve_history
@@ -585,6 +593,8 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_persist_policy_details;
   plugin->do_deposit
     = &TEH_PG_do_deposit;
+  plugin->get_wire_hash_for_contract
+    = &TEH_PG_get_wire_hash_for_contract;
   plugin->add_policy_fulfillment_proof
     = &TEH_PG_add_policy_fulfillment_proof;
   plugin->do_melt
@@ -603,6 +613,8 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_ensure_coin_known;
   plugin->get_known_coin
     = &TEH_PG_get_known_coin;
+  plugin->get_signature_for_known_coin
+    = &TEH_PG_get_signature_for_known_coin;
   plugin->get_coin_denomination
     = &TEH_PG_get_coin_denomination;
   plugin->have_deposit2
@@ -747,6 +759,8 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_select_purse_by_merge_pub;
   plugin->set_purse_balance
     = &TEH_PG_set_purse_balance;
+  plugin->get_pending_kyc_requirement_process
+    = &TEH_PG_get_pending_kyc_requirement_process;
   plugin->insert_kyc_attributes
     = &TEH_PG_insert_kyc_attributes;
   plugin->select_similar_kyc_attributes
