@@ -64,159 +64,6 @@ RETURN;
 END $$;
 COMMENT ON FUNCTION amount_left_minus_right
   IS 'Subtracts the right amount from the left and returns the difference and TRUE, if the left amount is larger than the right, or an invalid amount and FALSE otherwise.';
-CREATE OR REPLACE FUNCTION exchange_do_withdraw(
-  IN cs_nonce BYTEA,
-  IN amount taler_amount,
-  IN h_denom_pub BYTEA,
-  IN rpub BYTEA,
-  IN reserve_sig BYTEA,
-  IN h_coin_envelope BYTEA,
-  IN denom_sig BYTEA,
-  IN now INT8,
-  IN min_reserve_gc INT8,
-  IN do_age_check BOOLEAN,
-  OUT reserve_found BOOLEAN,
-  OUT balance_ok BOOLEAN,
-  OUT nonce_ok BOOLEAN,
-  OUT age_ok BOOLEAN,
-  OUT allowed_maximum_age INT2, 
-  OUT ruuid INT8)
-LANGUAGE plpgsql
-AS $$
-DECLARE
-  reserve RECORD;
-  denom_serial INT8;
-  balance taler_amount;
-  not_before date;
-BEGIN
-SELECT denominations_serial
-  INTO denom_serial
-  FROM exchange.denominations
- WHERE denom_pub_hash=h_denom_pub;
-IF NOT FOUND
-THEN
-  reserve_found=FALSE;
-  balance_ok=FALSE;
-  age_ok=FALSE;
-  allowed_maximum_age=0;
-  ruuid=0;
-  ASSERT false, 'denomination unknown';
-  RETURN;
-END IF;
-SELECT *
-  INTO reserve
-  FROM exchange.reserves
- WHERE reserves.reserve_pub=rpub;
-IF NOT FOUND
-THEN
-  reserve_found=FALSE;
-  balance_ok=FALSE;
-  nonce_ok=TRUE;
-  age_ok=FALSE;
-  allowed_maximum_age=0;
-  ruuid=2;
-  RETURN;
-END IF;
-balance = reserve.current_balance;
-ruuid = reserve.reserve_uuid;
-IF ((NOT do_age_check) OR (reserve.birthday = 0))
-THEN
-  age_ok = TRUE;
-  allowed_maximum_age = -1;
-ELSE
-  not_before=date '1970-01-01' + reserve.birthday;
-  allowed_maximum_age = extract(year from age(current_date, not_before));
-  reserve_found=TRUE;
-  nonce_ok=TRUE; 
-  balance_ok=TRUE;
-  age_ok = FALSE;
-  RETURN;
-END IF;
-INSERT INTO exchange.reserves_out
-  (h_blind_ev
-  ,denominations_serial
-  ,denom_sig
-  ,reserve_uuid
-  ,reserve_sig
-  ,execution_date
-  ,amount_with_fee)
-VALUES
-  (h_coin_envelope
-  ,denom_serial
-  ,denom_sig
-  ,ruuid
-  ,reserve_sig
-  ,now
-  ,amount)
-ON CONFLICT DO NOTHING;
-IF NOT FOUND
-THEN
-  reserve_found=TRUE;
-  balance_ok=TRUE;
-  nonce_ok=TRUE;
-  RETURN;
-END IF;
-IF (balance.val > amount.val)
-THEN
-  IF (balance.frac >= amount.frac)
-  THEN
-    balance.val=balance.val - amount.val;
-    balance.frac=balance.frac - amount.frac;
-  ELSE
-    balance.val=balance.val - amount.val - 1;
-    balance.frac=balance.frac + 100000000 - amount.frac;
-  END IF;
-ELSE
-  IF (balance.val = amount.val) AND (balance.frac >= amount.frac)
-  THEN
-    balance.val=0;
-    balance.frac=balance.frac - amount.frac;
-  ELSE
-    reserve_found=TRUE;
-    nonce_ok=TRUE; 
-    balance_ok=FALSE;
-    RETURN;
-  END IF;
-END IF;
-min_reserve_gc=GREATEST(min_reserve_gc,reserve.gc_date);
-UPDATE reserves SET
-  gc_date=min_reserve_gc
- ,current_balance=balance
-WHERE
-  reserves.reserve_pub=rpub;
-reserve_found=TRUE;
-balance_ok=TRUE;
-IF NOT NULL cs_nonce
-THEN
-  INSERT INTO exchange.cs_nonce_locks
-    (nonce
-    ,max_denomination_serial
-    ,op_hash)
-  VALUES
-    (cs_nonce
-    ,denom_serial
-    ,h_coin_envelope)
-  ON CONFLICT DO NOTHING;
-  IF NOT FOUND
-  THEN
-    SELECT 1
-      FROM exchange.cs_nonce_locks
-     WHERE nonce=cs_nonce
-       AND op_hash=h_coin_envelope;
-    IF NOT FOUND
-    THEN
-      reserve_found=FALSE;
-      balance_ok=FALSE;
-      nonce_ok=FALSE;
-      RETURN;
-    END IF;
-  END IF;
-ELSE
-  nonce_ok=TRUE; 
-END IF;
-END $$;
-COMMENT ON FUNCTION exchange_do_withdraw(BYTEA, taler_amount, BYTEA, BYTEA, BYTEA, BYTEA, BYTEA, INT8, INT8, BOOLEAN)
-  IS 'Checks whether the reserve has sufficient balance for a withdraw operation (or the request is repeated and was previously approved) and if the age requirements are formally met.  If so updates the database with the result';
 CREATE OR REPLACE FUNCTION exchange_do_batch_withdraw(
   IN amount taler_amount,
   IN rpub BYTEA,
@@ -225,6 +72,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch_withdraw(
   IN do_age_check BOOLEAN,
   OUT reserve_found BOOLEAN,
   OUT balance_ok BOOLEAN,
+  OUT reserve_balance taler_amount,
   OUT age_ok BOOLEAN,
   OUT allowed_maximum_age INT2, 
   OUT ruuid INT8)
@@ -235,7 +83,10 @@ DECLARE
   balance taler_amount;
   not_before date;
 BEGIN
-SELECT *
+SELECT current_balance
+      ,reserve_uuid
+      ,birthday
+      ,gc_date
   INTO reserve
   FROM exchange.reserves
  WHERE reserves.reserve_pub=rpub;
@@ -243,12 +94,15 @@ IF NOT FOUND
 THEN
   reserve_found=FALSE;
   balance_ok=FALSE;
+  reserve_balance.frac = 0;
+  reserve_balance.val = 0;
   age_ok=FALSE;
   allowed_maximum_age=0;
   ruuid=2;
   RETURN;
 END IF;
 reserve_found=TRUE;
+reserve_balance = reserve.current_balance;
 ruuid = reserve.reserve_uuid;
 IF ((NOT do_age_check) OR (reserve.birthday = 0))
 THEN
@@ -261,22 +115,21 @@ ELSE
   age_ok = FALSE;
   RETURN;
 END IF;
-balance = reserve.current_balance;
-IF (balance.val > amount.val)
+IF (reserve_balance.val > amount.val)
 THEN
-  IF (balance.frac >= amount.frac)
+  IF (reserve_balance.frac >= amount.frac)
   THEN
-    balance.val=balance.val - amount.val;
-    balance.frac=balance.frac - amount.frac;
+    balance.val=reserve_balance.val - amount.val;
+    balance.frac=reserve_balance.frac - amount.frac;
   ELSE
-    balance.val=balance.val - amount.val - 1;
-    balance.frac=balance.frac + 100000000 - amount.frac;
+    balance.val=reserve_balance.val - amount.val - 1;
+    balance.frac=reserve_balance.frac + 100000000 - amount.frac;
   END IF;
 ELSE
-  IF (balance.val = amount.val) AND (balance.frac >= amount.frac)
+  IF (reserve_balance.val = amount.val) AND (reserve_balance.frac >= amount.frac)
   THEN
     balance.val=0;
-    balance.frac=balance.frac - amount.frac;
+    balance.frac=reserve_balance.frac - amount.frac;
   ELSE
     balance_ok=FALSE;
     RETURN;
@@ -389,6 +242,7 @@ CREATE OR REPLACE FUNCTION exchange_do_age_withdraw(
   IN denom_sigs BYTEA[],
   OUT reserve_found BOOLEAN,
   OUT balance_ok BOOLEAN,
+  OUT reserve_balance taler_amount,
   OUT age_ok BOOLEAN,
   OUT required_age INT2, 
   OUT reserve_birthday INT4,
@@ -402,7 +256,9 @@ DECLARE
   not_before date;
   earliest_date date;
 BEGIN
-SELECT *
+SELECT current_balance
+      ,birthday
+      ,gc_date
   INTO reserve
   FROM exchange.reserves
  WHERE reserves.reserve_pub=rpub;
@@ -412,12 +268,14 @@ THEN
   age_ok = FALSE;
   required_age=-1;
   conflict=FALSE;
+  reserve_balance.val = 0;
+  reserve_balance.frac = 0;
   balance_ok=FALSE;
   RETURN;
 END IF;
 reserve_found = TRUE;
 conflict=FALSE; 
-balance = reserve.current_balance;
+reserve_balance = reserve.current_balance;
 reserve_birthday = reserve.birthday;
 IF (reserve.birthday <> 0)
 THEN
@@ -435,7 +293,7 @@ age_ok = TRUE;
 required_age=0;
 SELECT *
 INTO difference
-FROM amount_left_minus_right(balance
+FROM amount_left_minus_right(reserve_balance
                             ,amount_with_fee);
 balance_ok = difference.ok;
 IF NOT balance_ok
@@ -506,7 +364,6 @@ AS $$
 DECLARE
   wtsi INT8; 
   bdsi INT8; 
-  curs REFCURSOR;
   i INT4;
   ini_amount_with_fee taler_amount;
   ini_coin_pub BYTEA;
@@ -686,13 +543,13 @@ END IF;
 IF in_zombie_required
 THEN
   PERFORM
-    FROM exchange.recoup_refresh
+    FROM recoup_refresh
    WHERE rrc_serial IN
     (SELECT rrc_serial
-       FROM exchange.refresh_revealed_coins
+       FROM refresh_revealed_coins
       WHERE melt_serial_id IN
       (SELECT melt_serial_id
-         FROM exchange.refresh_commitments
+         FROM refresh_commitments
         WHERE old_coin_pub=in_old_coin_pub));
   IF NOT FOUND
   THEN
@@ -1175,7 +1032,7 @@ THEN
       recoup_timestamp
     INTO
       out_recoup_timestamp
-    FROM exchange.recoup_refresh
+    FROM recoup_refresh
     WHERE coin_pub=in_coin_pub;
   out_recoup_ok=FOUND;
   RETURN;
@@ -1207,7 +1064,7 @@ THEN
   out_internal_failure=TRUE;
   RETURN;
 END IF;
-INSERT INTO exchange.recoup_refresh
+INSERT INTO recoup_refresh
   (coin_pub
   ,known_coin_id
   ,coin_sig
@@ -1240,89 +1097,91 @@ DECLARE
   reserve_out_min INT8; 
   denom_min INT8; 
 BEGIN
-DELETE FROM exchange.prewire
+DELETE FROM prewire
   WHERE finished=TRUE;
-DELETE FROM exchange.wire_fee
+DELETE FROM wire_fee
   WHERE end_date < in_ancient_date;
-DELETE FROM exchange.reserves
+DELETE FROM reserves
   WHERE gc_date < in_now
     AND current_balance = (0, 0);
 SELECT
      reserve_out_serial_id
   INTO
      reserve_out_min
-  FROM exchange.reserves_out
+  FROM reserves_out
   ORDER BY reserve_out_serial_id ASC
   LIMIT 1;
-DELETE FROM exchange.recoup
+DELETE FROM recoup
   WHERE reserve_out_serial_id < reserve_out_min;
 SELECT
      reserve_uuid
   INTO
      reserve_uuid_min
-  FROM exchange.reserves
+  FROM reserves
   ORDER BY reserve_uuid ASC
   LIMIT 1;
-DELETE FROM exchange.reserves_out
+DELETE FROM reserves_out
   WHERE reserve_uuid < reserve_uuid_min;
-DELETE FROM exchange.denominations
+DELETE FROM denominations
   WHERE expire_legal < in_now
     AND denominations_serial NOT IN
       (SELECT DISTINCT denominations_serial
-         FROM exchange.reserves_out)
+         FROM reserves_out)
     AND denominations_serial NOT IN
       (SELECT DISTINCT denominations_serial
-         FROM exchange.known_coins
+         FROM known_coins
         WHERE coin_pub IN
           (SELECT DISTINCT coin_pub
-             FROM exchange.recoup))
+             FROM recoup))
     AND denominations_serial NOT IN
       (SELECT DISTINCT denominations_serial
-         FROM exchange.known_coins
+         FROM known_coins
         WHERE coin_pub IN
           (SELECT DISTINCT coin_pub
-             FROM exchange.recoup_refresh));
+             FROM recoup_refresh));
 SELECT
      melt_serial_id
   INTO
      melt_min
-  FROM exchange.refresh_commitments
+  FROM refresh_commitments
   ORDER BY melt_serial_id ASC
   LIMIT 1;
-DELETE FROM exchange.refresh_revealed_coins
+DELETE FROM refresh_revealed_coins
   WHERE melt_serial_id < melt_min;
-DELETE FROM exchange.refresh_transfer_keys
+DELETE FROM refresh_transfer_keys
   WHERE melt_serial_id < melt_min;
 SELECT
      known_coin_id
   INTO
      coin_min
-  FROM exchange.known_coins
+  FROM known_coins
   ORDER BY known_coin_id ASC
   LIMIT 1;
-DELETE FROM exchange.batch_deposits
+DELETE FROM recoup_refresh
+  WHERE known_coin_id < coin_min;
+DELETE FROM batch_deposits
   WHERE wire_deadline < in_ancient_date;
 SELECT
      batch_deposit_serial_id
   INTO
      batch_deposit_min
-  FROM exchange.coin_deposits
+  FROM coin_deposits
   ORDER BY batch_deposit_serial_id ASC
   LIMIT 1;
-DELETE FROM exchange.refunds
+DELETE FROM refunds
   WHERE batch_deposit_serial_id < batch_deposit_min;
-DELETE FROM exchange.aggregation_tracking
+DELETE FROM aggregation_tracking
   WHERE batch_deposit_serial_id < batch_deposit_min;
-DELETE FROM exchange.coin_deposits
+DELETE FROM coin_deposits
   WHERE batch_deposit_serial_id < batch_deposit_min;
 SELECT
      denominations_serial
   INTO
      denom_min
-  FROM exchange.denominations
+  FROM denominations
   ORDER BY denominations_serial ASC
   LIMIT 1;
-DELETE FROM exchange.cs_nonce_locks
+DELETE FROM cs_nonce_locks
   WHERE max_denomination_serial <= denom_min;
 END $$;
 CREATE OR REPLACE FUNCTION exchange_do_purse_delete(
@@ -1440,7 +1299,7 @@ DECLARE
 DECLARE
   rval RECORD;
 BEGIN
-INSERT INTO exchange.purse_deposits
+INSERT INTO purse_deposits
   (partner_serial_id
   ,purse_pub
   ,coin_pub
@@ -1456,9 +1315,9 @@ INSERT INTO exchange.purse_deposits
 IF NOT FOUND
 THEN
   PERFORM
-  FROM exchange.purse_deposits
-  WHERE coin_pub = in_coin_pub
-    AND purse_pub = in_purse_pub
+  FROM purse_deposits
+  WHERE purse_pub = in_purse_pub
+    AND coin_pub = in_coin_pub
     AND coin_sig = in_coin_sig;
   IF NOT FOUND
   THEN
@@ -1529,7 +1388,7 @@ SELECT COALESCE(partner_serial_id,0)
       ,reserve_pub
   INTO psi
       ,my_reserve_pub
-  FROM exchange.purse_merges
+  FROM purse_merges
  WHERE purse_pub=in_purse_pub;
 IF NOT FOUND
 THEN
@@ -1574,7 +1433,7 @@ THEN
     SET purses_active=purses_active-1
   WHERE reserve_pub IN
     (SELECT reserve_pub
-       FROM exchange.purse_merges
+       FROM purse_merges
       WHERE purse_pub=my_purse_pub
      LIMIT 1);
 END IF;
@@ -1695,7 +1554,7 @@ my_amount := rval.amount_with_fee;
 my_purse_fee := rval.purse_fee;
 my_in_reserve_quota := rval.in_reserve_quota;
 out_no_balance=FALSE;
-INSERT INTO exchange.purse_merges
+INSERT INTO purse_merges
     (partner_serial_id
     ,reserve_pub
     ,purse_pub
@@ -1711,7 +1570,7 @@ INSERT INTO exchange.purse_merges
 IF NOT FOUND
 THEN
   PERFORM
-  FROM exchange.purse_merges
+  FROM purse_merges
   WHERE purse_pub=in_purse_pub
      AND merge_sig=in_merge_sig;
   IF NOT FOUND
@@ -1743,11 +1602,11 @@ THEN
     SET purses_active=purses_active-1
   WHERE reserve_pub IN
     (SELECT reserve_pub
-       FROM exchange.purse_merges
+       FROM purse_merges
       WHERE purse_pub=my_purse_pub
      LIMIT 1);
 END IF;
-INSERT INTO exchange.account_merges
+INSERT INTO account_merges
   (reserve_pub
   ,reserve_sig
   ,purse_pub
@@ -1810,7 +1669,7 @@ CREATE OR REPLACE FUNCTION exchange_do_reserve_purse(
 LANGUAGE plpgsql
 AS $$
 BEGIN
-INSERT INTO exchange.purse_merges
+INSERT INTO purse_merges
     (partner_serial_id
     ,reserve_pub
     ,purse_pub
@@ -1826,7 +1685,7 @@ INSERT INTO exchange.purse_merges
 IF NOT FOUND
 THEN
   PERFORM
-  FROM exchange.purse_merges
+  FROM purse_merges
   WHERE purse_pub=in_purse_pub
      AND merge_sig=in_merge_sig;
   IF NOT FOUND
@@ -1906,7 +1765,7 @@ ELSE
   END IF;
 END IF;
 out_no_funds=FALSE;
-INSERT INTO exchange.account_merges
+INSERT INTO account_merges
   (reserve_pub
   ,reserve_sig
   ,purse_pub
@@ -1937,12 +1796,11 @@ SELECT purse_pub
       ,in_reserve_quota
   INTO my_purse_pub
       ,my_in_reserve_quota
-  FROM exchange.purse_requests
+  FROM purse_requests
  WHERE (purse_expiration >= in_start_time) AND
        (purse_expiration < in_end_time) AND
-   purse_pub NOT IN (SELECT purse_pub
-                       FROM purse_decision)
- ORDER BY purse_expiration ASC
+       NOT was_decided
+  ORDER BY purse_expiration ASC
  LIMIT 1;
 out_found = FOUND;
 IF NOT FOUND
@@ -1957,6 +1815,7 @@ VALUES
   (my_purse_pub
   ,in_now
   ,TRUE);
+NOTIFY X8DJSPNYJMNZDAP7GN6YQ4EZVSQXMF3HRP4VAR347WP9SZYP1C200;
 IF (my_in_reserve_quota)
 THEN
   UPDATE reserves
@@ -2060,7 +1919,9 @@ CREATE OR REPLACE FUNCTION exchange_do_reserve_open(
   IN in_open_fee taler_amount,
   OUT out_open_cost taler_amount,
   OUT out_final_expiration INT8,
-  OUT out_no_funds BOOLEAN)
+  OUT out_no_reserve BOOLEAN,
+  OUT out_no_funds BOOLEAN,
+  OUT out_reserve_balance taler_amount)
 LANGUAGE plpgsql
 AS $$
 DECLARE
@@ -2073,16 +1934,26 @@ DECLARE
   my_expiration_date INT8;
   reserve RECORD;
 BEGIN
-SELECT *
+SELECT current_balance
+      ,expiration_date
+      ,purses_allowed
   INTO reserve
   FROM reserves
  WHERE reserve_pub=in_reserve_pub;
 IF NOT FOUND
 THEN
   RAISE NOTICE 'reserve not found';
+  out_no_reserve = TRUE;
+  out_no_funds = TRUE;
+  out_reserve_balance.val = 0;
+  out_reserve_balance.frac = 0;
+  out_open_cost.val = 0;
+  out_open_cost.frac = 0;
+  out_final_expiration = 0;
   RETURN;
 END IF;
-my_balance = reserve.current_balance;
+out_no_reserve = FALSE;
+out_reserve_balance = reserve.current_balance;
 IF (reserve.expiration_date < in_now)
 THEN
   my_expiration_date = in_now;
@@ -2150,21 +2021,21 @@ THEN
   RAISE NOTICE 'amount paid too low';
   RETURN;
 END IF;
-IF (my_balance.val > in_reserve_payment.val)
+IF (out_reserve_balance.val > in_reserve_payment.val)
 THEN
-  IF (my_balance.frac >= in_reserve_payment.frac)
+  IF (out_reserve_balance.frac >= in_reserve_payment.frac)
   THEN
-    my_balance.val=my_balance.val - in_reserve_payment.val;
-    my_balance.frac=my_balance.frac - in_reserve_payment.frac;
+    my_balance.val=out_reserve_balance.val - in_reserve_payment.val;
+    my_balance.frac=out_reserve_balance.frac - in_reserve_payment.frac;
   ELSE
-    my_balance.val=my_balance.val - in_reserve_payment.val - 1;
-    my_balance.frac=my_balance.frac + 100000000 - in_reserve_payment.frac;
+    my_balance.val=out_reserve_balance.val - in_reserve_payment.val - 1;
+    my_balance.frac=out_reserve_balance.frac + 100000000 - in_reserve_payment.frac;
   END IF;
 ELSE
-  IF (my_balance.val = in_reserve_payment.val) AND (my_balance.frac >= in_reserve_payment.frac)
+  IF (out_reserve_balance.val = in_reserve_payment.val) AND (out_reserve_balance.frac >= in_reserve_payment.frac)
   THEN
     my_balance.val=0;
-    my_balance.frac=my_balance.frac - in_reserve_payment.frac;
+    my_balance.frac=out_reserve_balance.frac - in_reserve_payment.frac;
   ELSE
     out_final_expiration = reserve.expiration_date;
     out_open_cost.val = my_cost.val;
@@ -2248,13 +2119,13 @@ BEGIN
        THEN
                RAISE EXCEPTION 'accumulation overflow';
        END IF;
-       IF (out_fullfillment_state = 1) 
+       IF (out_fullfillment_state = 2) 
        THEN
                IF (out_accumulated_total.val >= cur_commitment.val OR
                        (out_accumulated_total.val = cur_commitment.val AND
                                out_accumulated_total.frac >= cur_commitment.frac))
                THEN
-                       out_fulfillment_state = 2; 
+                       out_fulfillment_state = 3; 
                END IF;
        END IF;
        UPDATE exchange.policy_details
@@ -2281,7 +2152,7 @@ LANGUAGE plpgsql
 AS $$
 BEGIN
 PERFORM
-  FROM exchange.aml_staff
+  FROM aml_staff
   WHERE decider_pub=in_decider_pub
     AND is_active
     AND NOT read_only;
@@ -2294,7 +2165,7 @@ END IF;
 out_invalid_officer=FALSE;
 SELECT decision_time
   INTO out_last_date
-  FROM exchange.aml_history
+  FROM aml_history
   WHERE h_payto=in_h_payto
   ORDER BY decision_time DESC;
 IF FOUND
@@ -2303,7 +2174,7 @@ THEN
   THEN
     RETURN;
   END IF;
-  UPDATE exchange.aml_status
+  UPDATE aml_status
     SET threshold=in_new_threshold
        ,status=in_new_status
        ,kyc_requirement=in_requirement_row
@@ -2311,7 +2182,7 @@ THEN
   ASSERT FOUND, 'cannot have AML decision history but no AML status';
 ELSE
   out_last_date = 0;
-  INSERT INTO exchange.aml_status
+  INSERT INTO aml_status
     (h_payto
     ,threshold
     ,status
@@ -2320,9 +2191,13 @@ ELSE
     (in_h_payto
     ,in_new_threshold
     ,in_new_status
-    ,in_requirement_row);
+    ,in_requirement_row)
+    ON CONFLICT (h_payto) DO
+    UPDATE SET
+       threshold=in_new_threshold
+      ,status=in_new_status;
 END IF;
-INSERT INTO exchange.aml_history
+INSERT INTO aml_history
   (h_payto
   ,new_threshold
   ,new_status
@@ -2503,7 +2378,6 @@ RETURNS SETOF exchange_do_array_reserve_insert_return_type
 LANGUAGE plpgsql
 AS $$
 DECLARE
-  curs REFCURSOR;
   conflict BOOL;
   dup BOOL;
   uuid INT8;
