@@ -20,6 +20,8 @@
 /**
  * @file include/taler_testing_lib.h
  * @brief API for writing an interpreter to test Taler components
+ *        This library is not thread-safe, all APIs must only be used from a single thread.
+ *        This library calls abort() if it runs out of memory. Be aware of these limitations.
  * @author Christian Grothoff <christian@grothoff.org>
  * @author Marcello Stanisci
  */
@@ -125,6 +127,12 @@ struct TALER_TESTING_Credentials
    */
   struct TALER_BANK_AuthenticationData ba;
 
+  /**
+   * Bank authentication details for the admin bank
+   * account.
+   */
+  struct TALER_BANK_AuthenticationData ba_admin;
+
   /**
    * Configuration file data.
    */
@@ -591,19 +599,6 @@ TALER_TESTING_parse_coin_reference (
   unsigned int *idx);
 
 
-/**
- * Compare @a h1 and @a h2.
- *
- * @param h1 a history entry
- * @param h2 a history entry
- * @return 0 if @a h1 and @a h2 are equal
- */
-int
-TALER_TESTING_history_entry_cmp (
-  const struct TALER_EXCHANGE_ReserveHistoryEntry *h1,
-  const struct TALER_EXCHANGE_ReserveHistoryEntry *h2);
-
-
 /* ************** Specific interpreter commands ************ */
 
 
@@ -837,44 +832,6 @@ TALER_TESTING_cmd_deposit_confirmation_with_retry (
   struct TALER_TESTING_Command cmd);
 
 
-/**
- * Create a "list exchanges" command.
- *
- * @param label command label.
- * @param expected_response_code expected HTTP response code.
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_exchanges (const char *label,
-                             unsigned int expected_response_code);
-
-
-/**
- * Create a "list exchanges" command and check whether
- * a particular exchange belongs to the returned bundle.
- *
- * @param label command label.
- * @param expected_response_code expected HTTP response code.
- * @param exchange_url URL of the exchange supposed to
- *  be included in the response.
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_exchanges_with_url (const char *label,
-                                      unsigned int expected_response_code,
-                                      const char *exchange_url);
-
-/**
- * Modify an exchanges command to enable retries when we get
- * transient errors from the auditor.
- *
- * @param cmd a deposit confirmation command
- * @return the command with retries enabled
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_exchanges_with_retry (struct TALER_TESTING_Command cmd);
-
-
 /**
  * Create /admin/add-incoming command.
  *
@@ -967,24 +924,6 @@ TALER_TESTING_cmd_exec_wget (const char *label,
                              const char *url);
 
 
-/**
- * Request fetch-transactions via "wget".
- *
- * @param label command label.
- * @param username username to use
- * @param password password to use
- * @param bank_base_url base URL of the nexus
- * @param account_id account to fetch transactions for
- * @return the command.
- */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_nexus_fetch_transactions (const char *label,
-                                            const char *username,
-                                            const char *password,
-                                            const char *bank_base_url,
-                                            const char *account_id);
-
-
 /**
  * Make a "expire" CMD.
  *
@@ -1105,11 +1044,40 @@ TALER_TESTING_cmd_withdraw_amount (const char *label,
                                    unsigned int expected_response_code);
 
 
+/**
+ * Create a batch withdraw command, letting the caller specify the type of
+ * conflict between the coins and the desired amounts as string.
+ *
+ * Takes a variable, non-empty list of the denomination amounts via VARARGS,
+ * similar to #TALER_TESTING_cmd_withdraw_amount(), just using a batch
+ * withdraw.
+ *
+ * @param label command label.
+ * @param reserve_reference command providing us with a reserve to withdraw from
+ * @param conflict if true, enforce a conflict (same priv key, different denom and age commiment)
+ * @param age if > 0, age restriction applies (same for all coins)
+ * @param expected_response_code which HTTP response code
+ *        we expect from the exchange.
+ * @param amount how much we withdraw for the first coin
+ * @param ... NULL-terminated list of additional amounts to withdraw (one per coin)
+ * @return the withdraw command to be executed by the interpreter.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_batch_withdraw_with_conflict (
+  const char *label,
+  const char *reserve_reference,
+  bool conflict,
+  uint8_t age,
+  unsigned int expected_response_code,
+  const char *amount,
+  ...);
+
 /**
  * Create a batch withdraw command, letting the caller specify
  * the desired amounts as string.  Takes a variable, non-empty
  * list of the denomination amounts via VARARGS, similar to
  * #TALER_TESTING_cmd_withdraw_amount(), just using a batch withdraw.
+ * The coins are generated without a conflict (different private keys).
  *
  * @param label command label.
  * @param reserve_reference command providing us with a reserve to withdraw from
@@ -1120,13 +1088,20 @@ TALER_TESTING_cmd_withdraw_amount (const char *label,
  * @param ... NULL-terminated list of additional amounts to withdraw (one per coin)
  * @return the withdraw command to be executed by the interpreter.
  */
-struct TALER_TESTING_Command
-TALER_TESTING_cmd_batch_withdraw (const char *label,
-                                  const char *reserve_reference,
-                                  uint8_t age,
-                                  unsigned int expected_response_code,
-                                  const char *amount,
-                                  ...);
+#define TALER_TESTING_cmd_batch_withdraw(label, \
+                                         reserve_reference, \
+                                         age, \
+                                         expected_response_code, \
+                                         amount, \
+                                         ...) \
+  TALER_TESTING_cmd_batch_withdraw_with_conflict ( \
+    (label), \
+    (reserve_reference), \
+    false, \
+    (age), \
+    (expected_response_code), \
+    (amount), \
+    __VA_ARGS__)
 
 /**
  * Create an age-withdraw command, letting the caller specify
@@ -1176,7 +1151,7 @@ TALER_TESTING_cmd_age_withdraw_reveal (
  * @param amount how much we withdraw.
  * @param age if > 0, age restriction applies.
  * @param coin_ref reference to (withdraw/reveal) command of a coin
- *        from which we should re-use the private key
+ *        from which we should reuse the private key
  * @param expected_response_code which HTTP response code
  *        we expect from the exchange.
  * @return the withdraw command to be executed by the interpreter.
@@ -1272,7 +1247,7 @@ TALER_TESTING_cmd_reserve_poll_finish (const char *label,
 
 
 /**
- * Create a POST "/reserves/$RID/history" command.
+ * Create a GET "/reserves/$RID/history" command.
  *
  * @param label the command label.
  * @param reserve_reference reference to the reserve to check.
@@ -1287,6 +1262,22 @@ TALER_TESTING_cmd_reserve_history (const char *label,
                                    unsigned int expected_response_code);
 
 
+/**
+ * Create a GET "/coins/$COIN_PUB/history" command.
+ *
+ * @param label the command label.
+ * @param coin_reference reference to the coin to check.
+ * @param expected_balance expected balance for the coin.
+ * @param expected_response_code expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_coin_history (const char *label,
+                                const char *coin_reference,
+                                const char *expected_balance,
+                                unsigned int expected_response_code);
+
+
 /**
  * Create a POST "/reserves/$RID/open" command.
  *
@@ -1928,7 +1919,7 @@ struct TALER_TESTING_Timer
   struct GNUNET_TIME_Relative total_duration;
 
   /**
-   * Total time spend waiting for the *successful* exeuction
+   * Total time spend waiting for the *successful* execution
    * in all commands of this type.
    */
   struct GNUNET_TIME_Relative success_latency;
@@ -2723,6 +2714,7 @@ TALER_TESTING_get_trait (const struct TALER_TESTING_Trait *traits,
   op (age_commitment_proof, const struct TALER_AgeCommitmentProof)      \
   op (h_age_commitment, const struct TALER_AgeCommitmentHash)           \
   op (reserve_history, const struct TALER_EXCHANGE_ReserveHistoryEntry) \
+  op (coin_history, const struct TALER_EXCHANGE_CoinHistoryEntry) \
   op (planchet_secrets, const struct TALER_PlanchetMasterSecretP)       \
   op (exchange_wd_value, const struct TALER_ExchangeWithdrawValues)     \
   op (coin_priv, const struct TALER_CoinSpendPrivateKeyP)               \
@@ -2734,7 +2726,7 @@ TALER_TESTING_get_trait (const struct TALER_TESTING_Trait *traits,
   op (refund_deadline, const struct GNUNET_TIME_Timestamp)              \
   op (exchange_pub, const struct TALER_ExchangePublicKeyP)              \
   op (exchange_sig, const struct TALER_ExchangeSignatureP)              \
-  op (blinding_key, const union TALER_DenominationBlindingKeyP)         \
+  op (blinding_key, const union GNUNET_CRYPTO_BlindingSecretP)         \
   op (h_blinded_coin, const struct TALER_BlindedCoinHashP)
 
 TALER_TESTING_SIMPLE_TRAITS (TALER_TESTING_MAKE_DECL_SIMPLE_TRAIT)
