@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2023 Taler Systems SA
+  Copyright (C) 2014-2024 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -16,7 +16,10 @@
 /**
  * @file include/taler_util.h
  * @brief Interface for common utility functions
+ *        This library is not thread-safe, all APIs must only be used from a single thread.
+ *        This library calls abort() if it runs out of memory. Be aware of these limitations.
  * @author Sree Harsha Totakura <sreeharsha@totakura.in>
+ * @author Christian Grothoff
  */
 #ifndef TALER_UTIL_H
 #define TALER_UTIL_H
@@ -33,7 +36,7 @@
  * Version of the Taler API, in hex.
  * Thus 0.8.4-1 = 0x00080401.
  */
-#define TALER_API_VERSION 0x00090200
+#define TALER_API_VERSION 0x00090401
 
 /**
  * Stringify operator.
@@ -209,6 +212,101 @@ TALER_config_get_currency (const struct GNUNET_CONFIGURATION_Handle *cfg,
                            char **currency);
 
 
+/**
+ * Details about how to render a currency.
+ */
+struct TALER_CurrencySpecification
+{
+  /**
+   * Currency code of the currency.
+   */
+  char currency[TALER_CURRENCY_LEN];
+
+  /**
+   * Human-readable long name of the currency, e.g.
+   * "Japanese Yen".
+   */
+  char *name;
+
+  /**
+   * how many digits the user may enter at most after the @e decimal_separator
+   */
+  unsigned int num_fractional_input_digits;
+
+  /**
+   * how many digits we render in normal scale after the @e decimal_separator
+   */
+  unsigned int num_fractional_normal_digits;
+
+  /**
+   * how many digits we render in after the @e decimal_separator even if all
+   * remaining digits are zero.
+   */
+  unsigned int num_fractional_trailing_zero_digits;
+
+  /**
+   * Mapping of powers of 10 to alternative currency names or symbols.
+   * Keys are the decimal powers, values the currency symbol to use.
+   * Map MUST contain an entry for "0" to the default currency symbol.
+   */
+  json_t *map_alt_unit_names;
+
+};
+
+
+/**
+ * Parse information about supported currencies from
+ * our configuration.
+ *
+ * @param cfg configuration to parse
+ * @param[out] num_currencies set to number of enabled currencies, length of @e cspecs
+ * @param[out] cspecs set to currency specification array
+ * @return #GNUNET_OK on success, #GNUNET_NO if zero
+ *  currency specifications were enabled,
+ *  #GNUNET_SYSERR if the configuration was malformed
+ */
+enum GNUNET_GenericReturnValue
+TALER_CONFIG_parse_currencies (const struct GNUNET_CONFIGURATION_Handle *cfg,
+                               unsigned int *num_currencies,
+                               struct TALER_CurrencySpecification **cspecs);
+
+
+/**
+ * Free @a cspecs array.
+ *
+ * @param num_currencies length of @a cspecs array
+ * @param[in] cspecs array to free
+ */
+void
+TALER_CONFIG_free_currencies (
+  unsigned int num_currencies,
+  struct TALER_CurrencySpecification cspecs[static num_currencies]);
+
+
+/**
+ * Convert a currency specification to the
+ * respective JSON object.
+ *
+ * @param cspec currency specification
+ * @return JSON object encoding @a cspec for `/config`.
+ */
+json_t *
+TALER_CONFIG_currency_specs_to_json (
+  const struct TALER_CurrencySpecification *cspec);
+
+
+/**
+ * Check that @a map contains a valid currency scale
+ * map that maps integers from [-12,24] to currency
+ * symbols given as strings.
+ *
+ * @param map map to check
+ * @return #GNUNET_OK if @a map is valid
+ */
+enum GNUNET_GenericReturnValue
+TALER_check_currency_scale_map (const json_t *map);
+
+
 /**
  * Allow user to specify an amount on the command line.
  *
@@ -271,6 +369,17 @@ bool
 TALER_url_valid_charset (const char *url);
 
 
+/**
+ * Test if the URL is a valid "http" (or "https")
+ * URL (includes test for #TALER_url_valid_charset()).
+ *
+ * @param url a string to test if it could be a valid URL
+ * @return true if @a url is well-formed
+ */
+bool
+TALER_is_web_url (const char *url);
+
+
 /**
  * Check if @a lang matches the @a language_pattern, and if so with
  * which preference.
@@ -307,8 +416,9 @@ TALER_mhd_is_https (struct MHD_Connection *connection);
  * that a NULL value does not terminate the list, only a NULL key signals the
  * end of the list of arguments.
  *
- * @param base_url absolute base URL to use
- * @param path path of the url
+ * @param base_url absolute base URL to use, must either
+ *          end with '/' *or* @a path must be the empty string
+ * @param path path of the url to append to the @a base_url
  * @param ... NULL-terminated key-value pairs (char *) for query parameters,
  *        only the value will be url-encoded
  * @returns the URL, must be freed with #GNUNET_free
@@ -390,6 +500,20 @@ char *
 TALER_payto_get_method (const char *payto_uri);
 
 
+/**
+ * Normalize payto://-URI to make "strcmp()" sufficient
+ * to check if two payto-URIs refer to the same bank
+ * account. Removes optional arguments (everything after
+ * "?") and applies method-specific normalizations to
+ * the main part of the URI.
+ *
+ * @param input a payto://-URI
+ * @return normalized URI, or NULL if @a input was not well-formed
+ */
+char *
+TALER_payto_normalize (const char *input);
+
+
 /**
  * Obtain the account name from a payto URL.
  *
@@ -539,7 +663,7 @@ char *strchrnul (const char *s, int c);
  * @param in Input string representation of the date
  * @param mask Age mask
  * @param[out] out Where to write the result
- * @return GNUNET_OK on success, GNUNET_SYSERR otherwise
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
  */
 enum GNUNET_GenericReturnValue
 TALER_parse_coarse_date (
@@ -585,6 +709,7 @@ TALER_parse_age_group_string (
   const char *groups,
   struct TALER_AgeMask *mask);
 
+
 /**
  * @brief Encodes the age mask into a string, like "8:10:12:14:16:18:21"
  *
@@ -599,7 +724,8 @@ const char *
 TALER_age_mask_to_string (
   const struct TALER_AgeMask *mask);
 
-/*
+
+/**
  * @brief returns the age group of a given age for a given age mask
  *
  * @param mask Age mask
@@ -611,6 +737,7 @@ TALER_get_age_group (
   const struct TALER_AgeMask *mask,
   uint8_t age);
 
+
 /**
  * @brief Parses a JSON object { "age_groups": "a:b:...y:z" }.
  *
@@ -623,11 +750,12 @@ TALER_JSON_parse_age_groups (const json_t *root,
                              struct TALER_AgeMask *mask);
 
 
-/* @brief Return the lowest age in the corresponding group for a given age
+/**
+ * @brief Return the lowest age in the corresponding group for a given age
  * according the given age mask.
  *
- * @param[IN] mask age mask
- * @param[IN] age age to check
+ * @param mask age mask
+ * @param age age to check
  * @return lowest age in corresponding age group
  */
 uint8_t
@@ -635,7 +763,9 @@ TALER_get_lowest_age (
   const struct TALER_AgeMask *mask,
   uint8_t age);
 
-/* @brief Get the lowest age for the largest age group
+
+/**
+ * @brief Get the lowest age for the largest age group
  *
  * @param mask the age mask
  * @return lowest age for the largest age group
