@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2022 Taler Systems SA
+  Copyright (C) 2022-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -186,6 +186,7 @@ TALER_KYCLOGIC_kyc_trigger_from_string (const char *trigger_s,
     enum TALER_KYCLOGIC_KycTriggerEvent out;
   } map [] = {
     { "withdraw", TALER_KYCLOGIC_KYC_TRIGGER_WITHDRAW },
+    { "age-withdraw", TALER_KYCLOGIC_KYC_TRIGGER_AGE_WITHDRAW },
     { "deposit", TALER_KYCLOGIC_KYC_TRIGGER_DEPOSIT  },
     { "merge", TALER_KYCLOGIC_KYC_TRIGGER_P2P_RECEIVE },
     { "balance", TALER_KYCLOGIC_KYC_TRIGGER_WALLET_BALANCE },
@@ -214,6 +215,8 @@ TALER_KYCLOGIC_kyc_trigger2s (enum TALER_KYCLOGIC_KycTriggerEvent trigger)
   {
   case TALER_KYCLOGIC_KYC_TRIGGER_WITHDRAW:
     return "withdraw";
+  case TALER_KYCLOGIC_KYC_TRIGGER_AGE_WITHDRAW:
+    return "age-withdraw";
   case TALER_KYCLOGIC_KYC_TRIGGER_DEPOSIT:
     return "deposit";
   case TALER_KYCLOGIC_KYC_TRIGGER_P2P_RECEIVE:
@@ -777,10 +780,11 @@ TALER_KYCLOGIC_kyc_init (const struct GNUNET_CONFIGURATION_Handle *cfg)
       TALER_KYCLOGIC_kyc_done ();
       return GNUNET_SYSERR;
     }
-  qsort (kyc_triggers,
-         num_kyc_triggers,
-         sizeof (struct TALER_KYCLOGIC_KycTrigger *),
-         &sort_by_timeframe);
+  if (0 != num_kyc_triggers)
+    qsort (kyc_triggers,
+           num_kyc_triggers,
+           sizeof (struct TALER_KYCLOGIC_KycTrigger *),
+           &sort_by_timeframe);
   return GNUNET_OK;
 }
 
@@ -1476,4 +1480,33 @@ TALER_KYCLOGIC_kyc_iterate_thresholds (
 }
 
 
+void
+TALER_KYCLOGIC_lookup_checks (const char *section_name,
+                              unsigned int *num_checks,
+                              char ***provided_checks)
+{
+  *num_checks = 0;
+  *provided_checks = NULL;
+  for (unsigned int i = 0; i<num_kyc_providers; i++)
+  {
+    struct TALER_KYCLOGIC_KycProvider *kp = kyc_providers[i];
+
+    if (0 !=
+        strcasecmp (section_name,
+                    kp->provider_section_name))
+      continue;
+    *num_checks = kp->num_checks;
+    if (0 != kp->num_checks)
+    {
+      char **pc = GNUNET_new_array (kp->num_checks,
+                                    char *);
+      for (unsigned int i = 0; i<kp->num_checks; i++)
+        pc[i] = GNUNET_strdup (kp->provided_checks[i]->name);
+      *provided_checks = pc;
+    }
+    return;
+  }
+}
+
+
 /* end of taler-exchange-httpd_kyc.c */
