@@ -113,10 +113,10 @@ struct TALER_KYCLOGIC_ProviderDetails
   char *post_kyc_redirect_url;
 
   /**
-   * Template for converting user-data returned by
-   * the provider into our KYC attribute data.
+   * Name of the program we use to convert outputs
+   * from Persona into our JSON inputs.
    */
-  char *attribute_template;
+  char *conversion_binary;
 
   /**
    * Validity time for a successful KYC process.
@@ -187,6 +187,12 @@ struct TALER_KYCLOGIC_ProofHandle
    */
   struct MHD_Connection *connection;
 
+  /**
+   * Handle to an external process that converts the
+   * Persona response to our internal format.
+   */
+  struct TALER_JSON_ExternalConversion *ec;
+
   /**
    * Hash of the payto URI that this is about.
    */
@@ -301,7 +307,7 @@ oauth2_unload_configuration (struct TALER_KYCLOGIC_ProviderDetails *pd)
   GNUNET_free (pd->client_id);
   GNUNET_free (pd->client_secret);
   GNUNET_free (pd->post_kyc_redirect_url);
-  GNUNET_free (pd->attribute_template);
+  GNUNET_free (pd->conversion_binary);
   GNUNET_free (pd);
 }
 
@@ -336,6 +342,21 @@ oauth2_load_configuration (void *cls,
     oauth2_unload_configuration (pd);
     return NULL;
   }
+
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_string (ps->cfg,
+                                             provider_section_name,
+                                             "KYC_OAUTH2_CLIENT_ID",
+                                             &s))
+  {
+    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                               provider_section_name,
+                               "KYC_OAUTH2_CLIENT_ID");
+    oauth2_unload_configuration (pd);
+    return NULL;
+  }
+  pd->client_id = s;
+
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_string (ps->cfg,
                                              provider_section_name,
@@ -414,9 +435,10 @@ oauth2_load_configuration (void *cls,
     pd->authorize_url = GNUNET_strndup (s,
                                         extra - s);
     GNUNET_asprintf (&pd->setup_url,
-                     "%.*s/setup",
+                     "%.*s/setup/%s",
                      (int) (slash - s),
-                     s);
+                     s,
+                     pd->client_id);
     GNUNET_free (s);
   }
   else
@@ -454,20 +476,6 @@ oauth2_load_configuration (void *cls,
   }
   pd->info_url = s;
 
-  if (GNUNET_OK !=
-      GNUNET_CONFIGURATION_get_value_string (ps->cfg,
-                                             provider_section_name,
-                                             "KYC_OAUTH2_CLIENT_ID",
-                                             &s))
-  {
-    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               provider_section_name,
-                               "KYC_OAUTH2_CLIENT_ID");
-    oauth2_unload_configuration (pd);
-    return NULL;
-  }
-  pd->client_id = s;
-
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_string (ps->cfg,
                                              provider_section_name,
@@ -499,16 +507,14 @@ oauth2_load_configuration (void *cls,
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_string (ps->cfg,
                                              provider_section_name,
-                                             "KYC_OAUTH2_ATTRIBUTE_TEMPLATE",
-                                             &s))
+                                             "KYC_OAUTH2_CONVERTER_HELPER",
+                                             &pd->conversion_binary))
   {
-    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_WARNING,
+    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                provider_section_name,
-                               "KYC_OAUTH2_ATTRIBUTE_TEMPLATE");
-  }
-  else
-  {
-    pd->attribute_template = s;
+                               "KYC_OAUTH2_CONVERTER_HELPER");
+    oauth2_unload_configuration (pd);
+    return NULL;
   }
 
   return pd;
@@ -547,18 +553,18 @@ initiate_with_url (struct TALER_KYCLOGIC_InitiateHandle *ih,
       char *redirect_uri;
 
       GNUNET_asprintf (&redirect_uri,
-                       "%skyc-proof/%s?state=%s",
+                       "%skyc-proof/%s",
                        ps->exchange_base_url,
-                       pd->section,
-                       hps);
+                       pd->section);
       redirect_uri_encoded = TALER_urlencode (redirect_uri);
       GNUNET_free (redirect_uri);
     }
     GNUNET_asprintf (&url,
-                     "%s?response_type=code&client_id=%s&redirect_uri=%s",
+                     "%s?response_type=code&client_id=%s&redirect_uri=%s&state=%s",
                      authorize_url,
                      pd->client_id,
-                     redirect_uri_encoded);
+                     redirect_uri_encoded,
+                     hps);
     GNUNET_free (redirect_uri_encoded);
   }
   ih->cb (ih->cb_cls,
@@ -628,7 +634,7 @@ handle_curl_setup_finished (void *cls,
       }
       GNUNET_asprintf (&url,
                        "%s/%s",
-                       pd->setup_url,
+                       pd->authorize_url,
                        nonce);
       initiate_with_url (ih,
                          url);
@@ -693,6 +699,22 @@ initiate_task (void *cls)
                  curl_easy_setopt (eh,
                                    CURLOPT_URL,
                                    pd->setup_url));
+  GNUNET_assert (CURLE_OK ==
+                 curl_easy_setopt (eh,
+                                   CURLOPT_POST,
+                                   1));
+  GNUNET_assert (CURLE_OK ==
+                 curl_easy_setopt (eh,
+                                   CURLOPT_POSTFIELDS,
+                                   ""));
+  GNUNET_assert (CURLE_OK ==
+                 curl_easy_setopt (eh,
+                                   CURLOPT_FOLLOWLOCATION,
+                                   1L));
+  GNUNET_assert (CURLE_OK ==
+                 curl_easy_setopt (eh,
+                                   CURLOPT_MAXREDIRS,
+                                   5L));
   GNUNET_asprintf (&hdr,
                    "%s: Bearer %s",
                    MHD_HTTP_HEADER_AUTHORIZATION,
@@ -773,6 +795,11 @@ oauth2_initiate_cancel (struct TALER_KYCLOGIC_InitiateHandle *ih)
 static void
 oauth2_proof_cancel (struct TALER_KYCLOGIC_ProofHandle *ph)
 {
+  if (NULL != ph->ec)
+  {
+    TALER_JSON_external_conversion_stop (ph->ec);
+    ph->ec = NULL;
+  }
   if (NULL != ph->task)
   {
     GNUNET_SCHEDULER_cancel (ph->task);
@@ -889,53 +916,84 @@ handle_proof_error (struct TALER_KYCLOGIC_ProofHandle *ph,
 
 
 /**
- * Convert user data returned by the provider into
- * standardized attribute data.
+ * Type of a callback that receives a JSON @a result.
  *
- * @param pd our provider configuration
- * @param data user-data given by the provider
- * @return converted KYC attribute data object
+ * @param cls closure with a `struct TALER_KYCLOGIC_ProofHandle *`
+ * @param status_type how did the process die
+ * @param code termination status code from the process
+ * @param attr result some JSON result, NULL if we failed to get an JSON output
  */
-static json_t *
-data2attributes (const struct TALER_KYCLOGIC_ProviderDetails *pd,
-                 const json_t *data)
+static void
+converted_proof_cb (void *cls,
+                    enum GNUNET_OS_ProcessStatusType status_type,
+                    unsigned long code,
+                    const json_t *attr)
 {
-  json_t *ret;
-  void *attr_data;
-  size_t attr_size;
-  int rv;
-  json_error_t err;
+  struct TALER_KYCLOGIC_ProofHandle *ph = cls;
 
-  if (NULL == pd->attribute_template)
-    return json_object ();
-  if (0 !=
-      (rv = TALER_TEMPLATING_fill (pd->attribute_template,
-                                   data,
-                                   &attr_data,
-                                   &attr_size)))
+  ph->ec = NULL;
+  if ( (NULL == attr) ||
+       (0 != code) )
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to convert KYC provider data to attributes: %d\n",
-                rv);
-    json_dumpf (data,
-                stderr,
-                JSON_INDENT (2));
-    return NULL;
+    GNUNET_break_op (0);
+    ph->status = TALER_KYCLOGIC_STATUS_PROVIDER_FAILED;
+    ph->response
+      = TALER_MHD_make_error (
+          TALER_EC_EXCHANGE_KYC_PROOF_BACKEND_INVALID_RESPONSE,
+          "Unexpected response from KYC gateway: proof success must contain data and status");
+    ph->http_status
+      = MHD_HTTP_BAD_GATEWAY;
+    ph->task = GNUNET_SCHEDULER_add_now (&return_proof_response,
+                                         ph);
+    return;
   }
-  ret = json_loadb (attr_data,
-                    attr_size,
-                    JSON_REJECT_DUPLICATES,
-                    &err);
-  GNUNET_free (attr_data);
-  if (NULL == ret)
+
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to parse converted KYC attributes as JSON: %s (at offset %d)\n",
-                err.text,
-                err.position);
-    return NULL;
+    const char *id;
+    struct GNUNET_JSON_Specification ispec[] = {
+      GNUNET_JSON_spec_string ("id",
+                               &id),
+      GNUNET_JSON_spec_end ()
+    };
+    enum GNUNET_GenericReturnValue res;
+    const char *emsg;
+    unsigned int line;
+
+    res = GNUNET_JSON_parse (attr,
+                             ispec,
+                             &emsg,
+                             &line);
+    if (GNUNET_OK != res)
+    {
+      GNUNET_break_op (0);
+      json_dumpf (attr,
+                  stderr,
+                  JSON_INDENT (2));
+      ph->status = TALER_KYCLOGIC_STATUS_PROVIDER_FAILED;
+      ph->response
+        = TALER_MHD_make_error (
+            TALER_EC_EXCHANGE_KYC_PROOF_BACKEND_INVALID_RESPONSE,
+            "Unexpected response from KYC gateway: data must contain id");
+      ph->http_status
+        = MHD_HTTP_BAD_GATEWAY;
+      return;
+    }
+    ph->provider_user_id = GNUNET_strdup (id);
   }
-  return ret;
+  ph->status = TALER_KYCLOGIC_STATUS_SUCCESS;
+  ph->response = MHD_create_response_from_buffer (0,
+                                                  "",
+                                                  MHD_RESPMEM_PERSISTENT);
+  GNUNET_assert (NULL != ph->response);
+  GNUNET_break (MHD_YES ==
+                MHD_add_response_header (
+                  ph->response,
+                  MHD_HTTP_HEADER_LOCATION,
+                  ph->pd->post_kyc_redirect_url));
+  ph->http_status = MHD_HTTP_SEE_OTHER;
+  ph->attributes = json_incref ((json_t *) attr);
+  ph->task = GNUNET_SCHEDULER_add_now (&return_proof_response,
+                                       ph);
 }
 
 
@@ -950,88 +1008,35 @@ static void
 parse_proof_success_reply (struct TALER_KYCLOGIC_ProofHandle *ph,
                            const json_t *j)
 {
-  const char *state;
-  const json_t *data;
-  struct GNUNET_JSON_Specification spec[] = {
-    GNUNET_JSON_spec_string ("status",
-                             &state),
-    GNUNET_JSON_spec_object_const ("data",
-                                   &data),
-    GNUNET_JSON_spec_end ()
-  };
-  enum GNUNET_GenericReturnValue res;
-  const char *emsg;
-  unsigned int line;
+  const struct TALER_KYCLOGIC_ProviderDetails *pd = ph->pd;
 
-  res = GNUNET_JSON_parse (j,
-                           spec,
-                           &emsg,
-                           &line);
-  if (GNUNET_OK != res)
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Calling converter `%s' with JSON\n",
+              pd->conversion_binary);
+  json_dumpf (j,
+              stderr,
+              JSON_INDENT (2));
+  ph->ec = TALER_JSON_external_conversion_start (
+    j,
+    &converted_proof_cb,
+    ph,
+    pd->conversion_binary,
+    pd->conversion_binary,
+    NULL);
+  if (NULL == ph->ec)
   {
-    GNUNET_break_op (0);
-    json_dumpf (j,
-                stderr,
-                JSON_INDENT (2));
-    ph->status = TALER_KYCLOGIC_STATUS_PROVIDER_FAILED;
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to start KYCAID conversion helper `%s'\n",
+                pd->conversion_binary);
+    ph->status = TALER_KYCLOGIC_STATUS_INTERNAL_ERROR;
     ph->response
       = TALER_MHD_make_error (
-          TALER_EC_EXCHANGE_KYC_PROOF_BACKEND_INVALID_RESPONSE,
-          "Unexpected response from KYC gateway: proof success must contain data and status");
+          TALER_EC_EXCHANGE_GENERIC_KYC_CONVERTER_FAILED,
+          "Failed to launch KYC conversion helper");
     ph->http_status
-      = MHD_HTTP_BAD_GATEWAY;
+      = MHD_HTTP_INTERNAL_SERVER_ERROR;
     return;
   }
-  if (0 != strcasecmp (state,
-                       "success"))
-  {
-    GNUNET_break_op (0);
-    handle_proof_error (ph,
-                        j);
-    return;
-  }
-  {
-    const char *id;
-    struct GNUNET_JSON_Specification ispec[] = {
-      GNUNET_JSON_spec_string ("id",
-                               &id),
-      GNUNET_JSON_spec_end ()
-    };
-
-    res = GNUNET_JSON_parse (data,
-                             ispec,
-                             &emsg,
-                             &line);
-    if (GNUNET_OK != res)
-    {
-      GNUNET_break_op (0);
-      json_dumpf (data,
-                  stderr,
-                  JSON_INDENT (2));
-      ph->status = TALER_KYCLOGIC_STATUS_PROVIDER_FAILED;
-      ph->response
-        = TALER_MHD_make_error (
-            TALER_EC_EXCHANGE_KYC_PROOF_BACKEND_INVALID_RESPONSE,
-            "Unexpected response from KYC gateway: data must contain id");
-      ph->http_status
-        = MHD_HTTP_BAD_GATEWAY;
-      return;
-    }
-    ph->status = TALER_KYCLOGIC_STATUS_SUCCESS;
-    ph->response = MHD_create_response_from_buffer (0,
-                                                    "",
-                                                    MHD_RESPMEM_PERSISTENT);
-    GNUNET_assert (NULL != ph->response);
-    GNUNET_break (MHD_YES ==
-                  MHD_add_response_header (
-                    ph->response,
-                    MHD_HTTP_HEADER_LOCATION,
-                    ph->pd->post_kyc_redirect_url));
-    ph->http_status = MHD_HTTP_SEE_OTHER;
-    ph->provider_user_id = GNUNET_strdup (id);
-  }
-  ph->attributes = data2attributes (ph->pd,
-                                    data);
 }
 
 
@@ -1055,10 +1060,19 @@ handle_curl_proof_finished (void *cls,
   ph->job = NULL;
   switch (response_code)
   {
+  case 0:
+    ph->status = TALER_KYCLOGIC_STATUS_PROVIDER_FAILED;
+    ph->response
+      = TALER_MHD_make_error (
+          TALER_EC_EXCHANGE_KYC_PROOF_BACKEND_INVALID_RESPONSE,
+          "No response from KYC gateway");
+    ph->http_status
+      = MHD_HTTP_BAD_GATEWAY;
+    break;
   case MHD_HTTP_OK:
     parse_proof_success_reply (ph,
                                j);
-    break;
+    return;
   default:
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "OAuth2.0 info URL returned HTTP status %u\n",
@@ -1299,11 +1313,17 @@ oauth2_proof (void *cls,
                                          ph);
     return ph;
   }
-
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Requesting OAuth 2.0 data via HTTP POST `%s'\n",
+              pd->token_url);
   GNUNET_assert (CURLE_OK ==
                  curl_easy_setopt (ph->eh,
                                    CURLOPT_URL,
                                    pd->token_url));
+  GNUNET_assert (CURLE_OK ==
+                 curl_easy_setopt (ph->eh,
+                                   CURLOPT_VERBOSE,
+                                   1));
   GNUNET_assert (CURLE_OK ==
                  curl_easy_setopt (ph->eh,
                                    CURLOPT_POST,
@@ -1321,10 +1341,9 @@ oauth2_proof (void *cls,
       char *redirect_uri;
 
       GNUNET_asprintf (&redirect_uri,
-                       "%skyc-proof/%s?state=%s",
+                       "%skyc-proof/%s",
                        ps->exchange_base_url,
-                       pd->section,
-                       hps);
+                       pd->section);
       redirect_uri_encoded = TALER_urlencode (redirect_uri);
       GNUNET_free (redirect_uri);
     }
@@ -1342,9 +1361,10 @@ oauth2_proof (void *cls,
                                            0);
     GNUNET_assert (NULL != authorization_code);
     GNUNET_asprintf (&ph->post_body,
-                     "client_id=%s&redirect_uri=%s&client_secret=%s&code=%s&grant_type=authorization_code",
+                     "client_id=%s&redirect_uri=%s&state=%s&client_secret=%s&code=%s&grant_type=authorization_code",
                      client_id,
                      redirect_uri_encoded,
+                     hps,
                      client_secret,
                      authorization_code);
     curl_free (authorization_code);
