@@ -1022,6 +1022,12 @@ start_conversion (const struct TALER_KYCLOGIC_ProviderDetails *pd,
                   TALER_JSON_JsonCallback cb,
                   void *cb_cls)
 {
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Calling converter `%s' with JSON\n",
+              pd->conversion_binary);
+  json_dumpf (attr,
+              stderr,
+              JSON_INDENT (2));
   return TALER_JSON_external_conversion_start (
     attr,
     cb,
@@ -1251,6 +1257,20 @@ handle_proof_finished (void *cls,
                                    j,
                                    &proof_post_conversion_cb,
                                    ph);
+        if (NULL == ph->ec)
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Failed to start Persona conversion helper\n");
+          proof_reply_error (
+            ph,
+            ph->inquiry_id,
+            MHD_HTTP_BAD_GATEWAY,
+            "persona-logic-failure",
+            GNUNET_JSON_PACK (
+              TALER_JSON_pack_ec (
+                TALER_EC_EXCHANGE_GENERIC_KYC_CONVERTER_FAILED)));
+          break;
+        }
       }
       return; /* continued in proof_post_conversion_cb */
     }
@@ -1289,7 +1309,7 @@ handle_proof_finished (void *cls,
     proof_reply_error (
       ph,
       ph->inquiry_id,
-      MHD_HTTP_INTERNAL_SERVER_ERROR,
+      MHD_HTTP_BAD_GATEWAY,
       "persona-exchange-unauthorized",
       GNUNET_JSON_PACK (
         GNUNET_JSON_pack_uint64 ("persona_http_status",
@@ -1309,7 +1329,7 @@ handle_proof_finished (void *cls,
     proof_reply_error (
       ph,
       ph->inquiry_id,
-      MHD_HTTP_INTERNAL_SERVER_ERROR,
+      MHD_HTTP_SERVICE_UNAVAILABLE,
       "persona-exchange-unpaid",
       GNUNET_JSON_PACK (
         GNUNET_JSON_pack_uint64 ("persona_http_status",
@@ -1408,8 +1428,6 @@ handle_proof_finished (void *cls,
                                  response_code),
         TALER_JSON_pack_ec (
           TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_UNEXPECTED_REPLY),
-        GNUNET_JSON_pack_string ("detail",
-                                 "data-relationships-account-data-id"),
         GNUNET_JSON_pack_allow_null (
           GNUNET_JSON_pack_object_incref ("data",
                                           (json_t *)
@@ -1765,6 +1783,15 @@ handle_webhook_finished (void *cls,
                                    j,
                                    &webhook_post_conversion_cb,
                                    wh);
+        if (NULL == wh->ec)
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Failed to start Persona conversion helper\n");
+          webhook_reply_error (wh,
+                               inquiry_id,
+                               MHD_HTTP_INTERNAL_SERVER_ERROR);
+          break;
+        }
       }
       return; /* continued in webhook_post_conversion_cb */
     }
