@@ -59,7 +59,7 @@
 #define EXCHANGE_SERIALIZATION_FORMAT_VERSION 0
 
 /**
- * How far off do we allow key liftimes to be?
+ * How far off do we allow key lifetimes to be?
  */
 #define LIFETIME_TOLERANCE GNUNET_TIME_UNIT_HOURS
 
@@ -224,7 +224,7 @@ append_signature (struct SignatureContext *sig_ctx,
     if (0 == sig_ctx->elements_size)
       new_size = 1024;
     else
-      new_size = sig_ctx->elements_size;
+      new_size = sig_ctx->elements_size * 2;
     GNUNET_array_grow (sig_ctx->elements,
                        sig_ctx->elements_size,
                        new_size);
@@ -280,17 +280,22 @@ parse_fees (const struct TALER_MasterPublicKeyP *master_pub,
             unsigned int *fees_len)
 {
   struct TALER_EXCHANGE_WireFeesByMethod *fbm;
-  unsigned int fbml = json_object_size (fees);
+  size_t fbml = json_object_size (fees);
   unsigned int i = 0;
   const char *key;
   const json_t *fee_array;
 
+  if (UINT_MAX < fbml)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
   fbm = GNUNET_new_array (fbml,
                           struct TALER_EXCHANGE_WireFeesByMethod);
-  *fees_len = fbml;
+  *fees_len = (unsigned int) fbml;
   json_object_foreach ((json_t *) fees, key, fee_array) {
     struct TALER_EXCHANGE_WireFeesByMethod *fe = &fbm[i++];
-    unsigned int idx;
+    size_t idx;
     json_t *fee;
 
     fe->method = GNUNET_strdup (key);
@@ -458,7 +463,7 @@ parse_json_signkey (struct TALER_EXCHANGE_SigningPublicKey *sign_key,
 static enum GNUNET_GenericReturnValue
 parse_json_denomkey_partially (
   struct TALER_EXCHANGE_DenomPublicKey *denom_key,
-  enum TALER_DenominationCipher cipher,
+  enum GNUNET_CRYPTO_BlindSignatureAlgorithm cipher,
   bool check_sigs,
   const json_t *denom_key_obj,
   struct TALER_MasterPublicKeyP *master_key,
@@ -517,11 +522,11 @@ parse_json_denomkey_partially (
             &denom_key->master_sig));
   return GNUNET_OK;
 EXITIF_exit:
+  GNUNET_JSON_parse_free (spec);
   /* invalidate denom_key, just to be sure */
   memset (denom_key,
           0,
           sizeof (*denom_key));
-  GNUNET_JSON_parse_free (spec);
   return GNUNET_SYSERR;
 }
 
@@ -544,13 +549,13 @@ parse_json_auditor (struct TALER_EXCHANGE_AuditorInformation *auditor,
 {
   const json_t *keys;
   json_t *key;
-  unsigned int off;
-  unsigned int pos;
+  size_t off;
+  size_t pos;
   const char *auditor_url;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_fixed_auto ("auditor_pub",
                                  &auditor->auditor_pub),
-    GNUNET_JSON_spec_string ("auditor_url",
+    TALER_JSON_spec_web_url ("auditor_url",
                              &auditor_url),
     GNUNET_JSON_spec_array_const ("denomination_keys",
                                   &keys),
@@ -637,7 +642,12 @@ parse_json_auditor (struct TALER_EXCHANGE_AuditorInformation *auditor,
     auditor->denom_keys[pos].auditor_sig = auditor_sig;
     pos++;
   }
-  auditor->num_denom_keys = pos;
+  if (pos > UINT_MAX)
+  {
+    GNUNET_break (0);
+    return GNUNET_SYSERR;
+  }
+  auditor->num_denom_keys = (unsigned int) pos;
   return GNUNET_OK;
 }
 
@@ -718,7 +728,7 @@ parse_global_fee (struct TALER_EXCHANGE_GlobalFee *gf,
  * @param denom1 first denomination key
  * @param denom2 second denomination key
  * @return 0 if the two keys are equal (not necessarily
- *  the same object), 1 otherwise.
+ *  the same object), non-zero otherwise.
  */
 static unsigned int
 denoms_cmp (const struct TALER_EXCHANGE_DenomPublicKey *denom1,
@@ -791,14 +801,10 @@ decode_keys_json (const json_t *resp_obj,
 #endif
   /* check the version first */
   {
-    const char *ver;
-    unsigned int age;
-    unsigned int revision;
-    unsigned int current;
-    char dummy;
+    struct TALER_JSON_ProtocolVersion pv;
     struct GNUNET_JSON_Specification spec[] = {
-      GNUNET_JSON_spec_string ("version",
-                               &ver),
+      TALER_JSON_spec_version ("version",
+                               &pv),
       GNUNET_JSON_spec_end ()
     };
 
@@ -810,33 +816,23 @@ decode_keys_json (const json_t *resp_obj,
       GNUNET_break_op (0);
       return GNUNET_SYSERR;
     }
-    if (3 != sscanf (ver,
-                     "%u:%u:%u%c",
-                     &current,
-                     &revision,
-                     &age,
-                     &dummy))
-    {
-      GNUNET_break_op (0);
-      return GNUNET_SYSERR;
-    }
     *vc = TALER_EXCHANGE_VC_MATCH;
-    if (EXCHANGE_PROTOCOL_CURRENT < current)
+    if (EXCHANGE_PROTOCOL_CURRENT < pv.current)
     {
       *vc |= TALER_EXCHANGE_VC_NEWER;
-      if (EXCHANGE_PROTOCOL_CURRENT < current - age)
+      if (EXCHANGE_PROTOCOL_CURRENT < pv.current - pv.age)
         *vc |= TALER_EXCHANGE_VC_INCOMPATIBLE;
     }
-    if (EXCHANGE_PROTOCOL_CURRENT > current)
+    if (EXCHANGE_PROTOCOL_CURRENT > pv.current)
     {
       *vc |= TALER_EXCHANGE_VC_OLDER;
-      if (EXCHANGE_PROTOCOL_CURRENT - EXCHANGE_PROTOCOL_AGE > current)
+      if (EXCHANGE_PROTOCOL_CURRENT - EXCHANGE_PROTOCOL_AGE > pv.current)
         *vc |= TALER_EXCHANGE_VC_INCOMPATIBLE;
     }
-    key_data->version = GNUNET_strdup (ver);
   }
 
   {
+    const char *ver;
     const char *currency;
     const char *asset_type;
     struct GNUNET_JSON_Specification mspec[] = {
@@ -864,9 +860,6 @@ decode_keys_json (const json_t *resp_obj,
       GNUNET_JSON_spec_string (
         "currency",
         &currency),
-      GNUNET_JSON_spec_uint32 (
-        "currency_fraction_digits",
-        &key_data->currency_fraction_digits),
       GNUNET_JSON_spec_string (
         "asset_type",
         &asset_type),
@@ -901,6 +894,8 @@ decode_keys_json (const json_t *resp_obj,
           "extensions_sig",
           &key_data->extensions_sig),
         &no_signature),
+      GNUNET_JSON_spec_string ("version",
+                               &ver),
       GNUNET_JSON_spec_mark_optional (
         GNUNET_JSON_spec_array_const (
           "wallet_balance_limit_without_kyc",
@@ -925,6 +920,10 @@ decode_keys_json (const json_t *resp_obj,
     }
     {
       struct GNUNET_JSON_Specification sspec[] = {
+        TALER_JSON_spec_currency_specification (
+          "currency_specification",
+          currency,
+          &key_data->cspec),
         TALER_JSON_spec_amount (
           "stefan_abs",
           currency,
@@ -933,32 +932,41 @@ decode_keys_json (const json_t *resp_obj,
           "stefan_log",
           currency,
           &key_data->stefan_log),
-        TALER_JSON_spec_amount (
+        GNUNET_JSON_spec_double (
           "stefan_lin",
-          currency,
           &key_data->stefan_lin),
         GNUNET_JSON_spec_end ()
       };
 
-      EXITIF (GNUNET_OK !=
-              GNUNET_JSON_parse (resp_obj,
-                                 sspec,
-                                 NULL, NULL));
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (resp_obj,
+                             sspec,
+                             &emsg,
+                             &eline))
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                    "Parsing /keys failed for `%s' (%u)\n",
+                    emsg,
+                    eline);
+        EXITIF (1);
+      }
     }
 
     key_data->currency = GNUNET_strdup (currency);
+    key_data->version = GNUNET_strdup (ver);
     key_data->asset_type = GNUNET_strdup (asset_type);
     if (! no_extensions)
       key_data->extensions = json_incref ((json_t *) manifests);
   }
 
   /* parse the global fees */
+  EXITIF (json_array_size (global_fees) > UINT_MAX);
   key_data->num_global_fees
-    = json_array_size (global_fees);
+    = (unsigned int) json_array_size (global_fees);
   if (0 != key_data->num_global_fees)
   {
     json_t *global_fee;
-    unsigned int index;
+    size_t index;
 
     key_data->global_fees
       = GNUNET_new_array (key_data->num_global_fees,
@@ -974,12 +982,13 @@ decode_keys_json (const json_t *resp_obj,
   }
 
   /* parse the signing keys */
+  EXITIF (json_array_size (sign_keys_array) > UINT_MAX);
   key_data->num_sign_keys
-    = json_array_size (sign_keys_array);
+    = (unsigned int) json_array_size (sign_keys_array);
   if (0 != key_data->num_sign_keys)
   {
     json_t *sign_key_obj;
-    unsigned int index;
+    size_t index;
 
     key_data->sign_keys
       = GNUNET_new_array (key_data->num_sign_keys,
@@ -996,7 +1005,9 @@ decode_keys_json (const json_t *resp_obj,
   /* Parse balance limits */
   if (NULL != wblwk)
   {
-    key_data->wblwk_length = json_array_size (wblwk);
+    EXITIF (json_array_size (wblwk) > UINT_MAX);
+    key_data->wblwk_length
+      = (unsigned int) json_array_size (wblwk);
     key_data->wallet_balance_limit_without_kyc
       = GNUNET_new_array (key_data->wblwk_length,
                           struct TALER_Amount);
@@ -1026,6 +1037,7 @@ decode_keys_json (const json_t *resp_obj,
                                &key_data->fees_len);
   EXITIF (NULL == key_data->fees);
   /* parse accounts */
+  EXITIF (json_array_size (accounts) > UINT_MAX);
   GNUNET_array_grow (key_data->accounts,
                      key_data->accounts_len,
                      json_array_size (accounts));
@@ -1037,7 +1049,7 @@ decode_keys_json (const json_t *resp_obj,
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Parsed %u wire accounts from JSON\n",
-              (unsigned int) json_array_size (accounts));
+              key_data->accounts_len);
 
 
   /* Parse the supported extension(s): age-restriction. */
@@ -1112,7 +1124,6 @@ decode_keys_json (const json_t *resp_obj,
            denomination.  Required to make the validity check inside
            parse_json_denomkey_partially pass */
         struct TALER_EXCHANGE_DenomPublicKey dk = {
-          .key.cipher = group.cipher,
           .value = group.value,
           .fees = group.fees,
           .key.age_mask = group.age_mask
@@ -1154,6 +1165,9 @@ decode_keys_json (const json_t *resp_obj,
           GNUNET_array_grow (key_data->denom_keys,
                              key_data->denom_keys_size,
                              key_data->denom_keys_size * 2 + 2);
+        GNUNET_assert (key_data->denom_keys_size >
+                       key_data->num_denom_keys);
+        GNUNET_assert (key_data->num_denom_keys < UINT_MAX);
         key_data->denom_keys[key_data->num_denom_keys++] = dk;
 
         /* Update "last_denom_issue_date" */
@@ -1232,7 +1246,10 @@ decode_keys_json (const json_t *resp_obj,
         GNUNET_array_grow (key_data->auditors,
                            key_data->auditors_size,
                            key_data->auditors_size * 2 + 2);
+      GNUNET_assert (key_data->auditors_size >
+                     key_data->num_auditors);
       GNUNET_assert (NULL != ai.auditor_url);
+      GNUNET_assert (key_data->num_auditors < UINT_MAX);
       key_data->auditors[key_data->num_auditors++] = ai;
     };
   }
@@ -1346,10 +1363,11 @@ keys_completed_cb (void *cls,
               GNUNET_TIME_timestamp2s (gkh->expire));
   if (GNUNET_TIME_absolute_is_past (gkh->expire.abs_time))
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Exchange failed to give expiration time, assuming in %s\n",
-                GNUNET_TIME_relative2s (DEFAULT_EXPIRATION,
-                                        true));
+    if (MHD_HTTP_OK == response_code)
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "Exchange failed to give expiration time, assuming in %s\n",
+                  GNUNET_TIME_relative2s (DEFAULT_EXPIRATION,
+                                          true));
     gkh->expire
       = GNUNET_TIME_absolute_to_timestamp (
           GNUNET_TIME_relative_to_absolute (DEFAULT_EXPIRATION));
@@ -1389,8 +1407,8 @@ keys_completed_cb (void *cls,
                      kd_old->num_denom_keys
                      * sizeof (struct TALER_EXCHANGE_DenomPublicKey));
       for (unsigned int i = 0; i<kd_old->num_denom_keys; i++)
-        TALER_denom_pub_deep_copy (&kd->denom_keys[i].key,
-                                   &kd_old->denom_keys[i].key);
+        TALER_denom_pub_copy (&kd->denom_keys[i].key,
+                              &kd_old->denom_keys[i].key);
       kd->num_auditors = kd_old->num_auditors;
       kd->auditors = GNUNET_new_array (kd->num_auditors,
                                        struct TALER_EXCHANGE_AuditorInformation);
@@ -1689,7 +1707,7 @@ TALER_EXCHANGE_get_keys (
   GNUNET_break (CURLE_OK ==
                 curl_easy_setopt (eh,
                                   CURLOPT_VERBOSE,
-                                  0));
+                                  1));
   GNUNET_break (CURLE_OK ==
                 curl_easy_setopt (eh,
                                   CURLOPT_TIMEOUT,
@@ -1799,8 +1817,8 @@ TALER_EXCHANGE_copy_denomination_key (
 
   copy = GNUNET_new (struct TALER_EXCHANGE_DenomPublicKey);
   *copy = *key;
-  TALER_denom_pub_deep_copy (&copy->key,
-                             &key->key);
+  TALER_denom_pub_copy (&copy->key,
+                        &key->key);
   return copy;
 }
 
@@ -1872,6 +1890,8 @@ TALER_EXCHANGE_keys_decref (struct TALER_EXCHANGE_Keys *keys)
   free_fees (keys->fees,
              keys->fees_len);
   json_decref (keys->extensions);
+  GNUNET_free (keys->cspec.name);
+  json_decref (keys->cspec.map_alt_unit_names);
   GNUNET_free (keys->wallet_balance_limit_without_kyc);
   GNUNET_free (keys->version);
   GNUNET_free (keys->currency);
@@ -1895,7 +1915,7 @@ TALER_EXCHANGE_keys_from_json (const json_t *j)
                              &version),
     GNUNET_JSON_spec_object_const ("keys",
                                    &jkeys),
-    GNUNET_JSON_spec_string ("exchange_url",
+    TALER_JSON_spec_web_url ("exchange_url",
                              &url),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_timestamp ("expire",
@@ -1978,10 +1998,10 @@ add_grp (void *cls,
   (void) key;
   switch (gd->meta.cipher)
   {
-  case TALER_DENOMINATION_RSA:
+  case GNUNET_CRYPTO_BSA_RSA:
     cipher = age_restricted ? "RSA+age_restricted" : "RSA";
     break;
-  case TALER_DENOMINATION_CS:
+  case GNUNET_CRYPTO_BSA_CS:
     cipher = age_restricted ? "CS+age_restricted" : "CS";
     break;
   default:
@@ -2055,7 +2075,7 @@ ar_to_json (unsigned int ar_len,
               "type",
               "regex"),
             GNUNET_JSON_pack_string (
-              "regex",
+              "payto_regex",
               ar->details.regex.posix_egrep),
             GNUNET_JSON_pack_string (
               "human_hint",
@@ -2125,7 +2145,7 @@ TALER_EXCHANGE_keys_to_json (const struct TALER_EXCHANGE_Keys *kd)
     {
       const struct TALER_EXCHANGE_DenomPublicKey *dk = &kd->denom_keys[i];
       struct TALER_DenominationGroup meta = {
-        .cipher = dk->key.cipher,
+        .cipher = dk->key.bsign_pub_key->cipher,
         .value = dk->value,
         .fees = dk->fees,
         .age_mask = dk->key.age_mask
@@ -2159,18 +2179,18 @@ TALER_EXCHANGE_keys_to_json (const struct TALER_EXCHANGE_Keys *kd)
       }
       switch (meta.cipher)
       {
-      case TALER_DENOMINATION_RSA:
+      case GNUNET_CRYPTO_BSA_RSA:
         key_spec =
           GNUNET_JSON_pack_rsa_public_key (
             "rsa_pub",
-            dk->key.details.rsa_public_key);
+            dk->key.bsign_pub_key->details.rsa_public_key);
         break;
-      case TALER_DENOMINATION_CS:
+      case GNUNET_CRYPTO_BSA_CS:
         key_spec =
           GNUNET_JSON_pack_data_varsize (
             "cs_pub",
-            &dk->key.details.cs_public_key,
-            sizeof (dk->key.details.cs_public_key));
+            &dk->key.bsign_pub_key->details.cs_public_key,
+            sizeof (dk->key.bsign_pub_key->details.cs_public_key));
         break;
       default:
         GNUNET_assert (false);
@@ -2381,14 +2401,15 @@ TALER_EXCHANGE_keys_to_json (const struct TALER_EXCHANGE_Keys *kd)
                              kd->version),
     GNUNET_JSON_pack_string ("currency",
                              kd->currency),
-    GNUNET_JSON_pack_uint64 ("currency_fraction_digits",
-                             kd->currency_fraction_digits),
+    GNUNET_JSON_pack_object_steal ("currency_specification",
+                                   TALER_CONFIG_currency_specs_to_json (
+                                     &kd->cspec)),
     TALER_JSON_pack_amount ("stefan_abs",
                             &kd->stefan_abs),
     TALER_JSON_pack_amount ("stefan_log",
                             &kd->stefan_log),
-    TALER_JSON_pack_amount ("stefan_lin",
-                            &kd->stefan_lin),
+    GNUNET_JSON_pack_double ("stefan_lin",
+                             kd->stefan_lin),
     GNUNET_JSON_pack_string ("asset_type",
                              kd->asset_type),
     GNUNET_JSON_pack_data_auto ("master_public_key",
