@@ -174,13 +174,12 @@ qconv_amount_tuple (void *cls,
                                           oid_v,
                                           oid_f);
 
-      sz = sizeof(uint32_t); /* number of elements in tuple */
-      sz += sizeof(d);
+      sz = sizeof(d);
       out = GNUNET_malloc (sz);
       scratch[0] = out;
-      *(uint32_t *) out = htonl (2);
-      out += sizeof(uint32_t);
-      *(struct TALER_PQ_AmountP*) out = d;
+      GNUNET_memcpy (out,
+                     &d,
+                     sizeof(d));
     }
   }
 
@@ -235,6 +234,7 @@ qconv_denom_pub (void *cls,
                  unsigned int scratch_length)
 {
   const struct TALER_DenominationPublicKey *denom_pub = data;
+  const struct GNUNET_CRYPTO_BlindSignPublicKey *bsp = denom_pub->bsign_pub_key;
   size_t tlen;
   size_t len;
   uint32_t be[2];
@@ -246,17 +246,17 @@ qconv_denom_pub (void *cls,
   GNUNET_assert (1 == param_length);
   GNUNET_assert (scratch_length > 0);
   GNUNET_break (NULL == cls);
-  be[0] = htonl ((uint32_t) denom_pub->cipher);
+  be[0] = htonl ((uint32_t) bsp->cipher);
   be[1] = htonl (denom_pub->age_mask.bits);
-  switch (denom_pub->cipher)
+  switch (bsp->cipher)
   {
-  case TALER_DENOMINATION_RSA:
+  case GNUNET_CRYPTO_BSA_RSA:
     tlen = GNUNET_CRYPTO_rsa_public_key_encode (
-      denom_pub->details.rsa_public_key,
+      bsp->details.rsa_public_key,
       &tbuf);
     break;
-  case TALER_DENOMINATION_CS:
-    tlen = sizeof (denom_pub->details.cs_public_key);
+  case GNUNET_CRYPTO_BSA_CS:
+    tlen = sizeof (bsp->details.cs_public_key);
     break;
   default:
     GNUNET_assert (0);
@@ -266,17 +266,17 @@ qconv_denom_pub (void *cls,
   GNUNET_memcpy (buf,
                  be,
                  sizeof (be));
-  switch (denom_pub->cipher)
+  switch (bsp->cipher)
   {
-  case TALER_DENOMINATION_RSA:
+  case GNUNET_CRYPTO_BSA_RSA:
     GNUNET_memcpy (&buf[sizeof (be)],
                    tbuf,
                    tlen);
     GNUNET_free (tbuf);
     break;
-  case TALER_DENOMINATION_CS:
+  case GNUNET_CRYPTO_BSA_CS:
     GNUNET_memcpy (&buf[sizeof (be)],
-                   &denom_pub->details.cs_public_key,
+                   &bsp->details.cs_public_key,
                    tlen);
     break;
   default:
@@ -331,6 +331,7 @@ qconv_denom_sig (void *cls,
                  unsigned int scratch_length)
 {
   const struct TALER_DenominationSignature *denom_sig = data;
+  const struct GNUNET_CRYPTO_UnblindedSignature *ubs = denom_sig->unblinded_sig;
   size_t tlen;
   size_t len;
   uint32_t be[2];
@@ -342,17 +343,17 @@ qconv_denom_sig (void *cls,
   GNUNET_assert (1 == param_length);
   GNUNET_assert (scratch_length > 0);
   GNUNET_break (NULL == cls);
-  be[0] = htonl ((uint32_t) denom_sig->cipher);
+  be[0] = htonl ((uint32_t) ubs->cipher);
   be[1] = htonl (0x00); /* magic marker: unblinded */
-  switch (denom_sig->cipher)
+  switch (ubs->cipher)
   {
-  case TALER_DENOMINATION_RSA:
+  case GNUNET_CRYPTO_BSA_RSA:
     tlen = GNUNET_CRYPTO_rsa_signature_encode (
-      denom_sig->details.rsa_signature,
+      ubs->details.rsa_signature,
       &tbuf);
     break;
-  case TALER_DENOMINATION_CS:
-    tlen = sizeof (denom_sig->details.cs_signature);
+  case GNUNET_CRYPTO_BSA_CS:
+    tlen = sizeof (ubs->details.cs_signature);
     break;
   default:
     GNUNET_assert (0);
@@ -362,17 +363,17 @@ qconv_denom_sig (void *cls,
   GNUNET_memcpy (buf,
                  &be,
                  sizeof (be));
-  switch (denom_sig->cipher)
+  switch (ubs->cipher)
   {
-  case TALER_DENOMINATION_RSA:
+  case GNUNET_CRYPTO_BSA_RSA:
     GNUNET_memcpy (&buf[sizeof (be)],
                    tbuf,
                    tlen);
     GNUNET_free (tbuf);
     break;
-  case TALER_DENOMINATION_CS:
+  case GNUNET_CRYPTO_BSA_CS:
     GNUNET_memcpy (&buf[sizeof (be)],
-                   &denom_sig->details.cs_signature,
+                   &ubs->details.cs_signature,
                    tlen);
     break;
   default:
@@ -427,6 +428,7 @@ qconv_blinded_denom_sig (void *cls,
                          unsigned int scratch_length)
 {
   const struct TALER_BlindedDenominationSignature *denom_sig = data;
+  const struct GNUNET_CRYPTO_BlindedSignature *bs = denom_sig->blinded_sig;
   size_t tlen;
   size_t len;
   uint32_t be[2];
@@ -438,17 +440,17 @@ qconv_blinded_denom_sig (void *cls,
   GNUNET_assert (1 == param_length);
   GNUNET_assert (scratch_length > 0);
   GNUNET_break (NULL == cls);
-  be[0] = htonl ((uint32_t) denom_sig->cipher);
+  be[0] = htonl ((uint32_t) bs->cipher);
   be[1] = htonl (0x01); /* magic marker: blinded */
-  switch (denom_sig->cipher)
+  switch (bs->cipher)
   {
-  case TALER_DENOMINATION_RSA:
+  case GNUNET_CRYPTO_BSA_RSA:
     tlen = GNUNET_CRYPTO_rsa_signature_encode (
-      denom_sig->details.blinded_rsa_signature,
+      bs->details.blinded_rsa_signature,
       &tbuf);
     break;
-  case TALER_DENOMINATION_CS:
-    tlen = sizeof (denom_sig->details.blinded_cs_answer);
+  case GNUNET_CRYPTO_BSA_CS:
+    tlen = sizeof (bs->details.blinded_cs_answer);
     break;
   default:
     GNUNET_assert (0);
@@ -458,17 +460,17 @@ qconv_blinded_denom_sig (void *cls,
   GNUNET_memcpy (buf,
                  &be,
                  sizeof (be));
-  switch (denom_sig->cipher)
+  switch (bs->cipher)
   {
-  case TALER_DENOMINATION_RSA:
+  case GNUNET_CRYPTO_BSA_RSA:
     GNUNET_memcpy (&buf[sizeof (be)],
                    tbuf,
                    tlen);
     GNUNET_free (tbuf);
     break;
-  case TALER_DENOMINATION_CS:
+  case GNUNET_CRYPTO_BSA_CS:
     GNUNET_memcpy (&buf[sizeof (be)],
-                   &denom_sig->details.blinded_cs_answer,
+                   &bs->details.blinded_cs_answer,
                    tlen);
     break;
   default:
@@ -523,6 +525,7 @@ qconv_blinded_planchet (void *cls,
                         unsigned int scratch_length)
 {
   const struct TALER_BlindedPlanchet *bp = data;
+  const struct GNUNET_CRYPTO_BlindedMessage *bm = bp->blinded_message;
   size_t tlen;
   size_t len;
   uint32_t be[2];
@@ -533,15 +536,15 @@ qconv_blinded_planchet (void *cls,
   GNUNET_assert (1 == param_length);
   GNUNET_assert (scratch_length > 0);
   GNUNET_break (NULL == cls);
-  be[0] = htonl ((uint32_t) bp->cipher);
+  be[0] = htonl ((uint32_t) bm->cipher);
   be[1] = htonl (0x0100); /* magic marker: blinded */
-  switch (bp->cipher)
+  switch (bm->cipher)
   {
-  case TALER_DENOMINATION_RSA:
-    tlen = bp->details.rsa_blinded_planchet.blinded_msg_size;
+  case GNUNET_CRYPTO_BSA_RSA:
+    tlen = bm->details.rsa_blinded_message.blinded_msg_size;
     break;
-  case TALER_DENOMINATION_CS:
-    tlen = sizeof (bp->details.cs_blinded_planchet);
+  case GNUNET_CRYPTO_BSA_CS:
+    tlen = sizeof (bm->details.cs_blinded_message);
     break;
   default:
     GNUNET_assert (0);
@@ -551,16 +554,16 @@ qconv_blinded_planchet (void *cls,
   GNUNET_memcpy (buf,
                  &be,
                  sizeof (be));
-  switch (bp->cipher)
+  switch (bm->cipher)
   {
-  case TALER_DENOMINATION_RSA:
+  case GNUNET_CRYPTO_BSA_RSA:
     GNUNET_memcpy (&buf[sizeof (be)],
-                   bp->details.rsa_blinded_planchet.blinded_msg,
+                   bm->details.rsa_blinded_message.blinded_msg,
                    tlen);
     break;
-  case TALER_DENOMINATION_CS:
+  case GNUNET_CRYPTO_BSA_CS:
     GNUNET_memcpy (&buf[sizeof (be)],
-                   &bp->details.cs_blinded_planchet,
+                   &bm->details.cs_blinded_message,
                    tlen);
     break;
   default:
@@ -614,6 +617,8 @@ qconv_exchange_withdraw_values (void *cls,
                                 unsigned int scratch_length)
 {
   const struct TALER_ExchangeWithdrawValues *alg_values = data;
+  const struct GNUNET_CRYPTO_BlindingInputValues *bi =
+    alg_values->blinding_inputs;
   size_t tlen;
   size_t len;
   uint32_t be[2];
@@ -624,15 +629,15 @@ qconv_exchange_withdraw_values (void *cls,
   GNUNET_assert (1 == param_length);
   GNUNET_assert (scratch_length > 0);
   GNUNET_break (NULL == cls);
-  be[0] = htonl ((uint32_t) alg_values->cipher);
+  be[0] = htonl ((uint32_t) bi->cipher);
   be[1] = htonl (0x010000); /* magic marker: EWV */
-  switch (alg_values->cipher)
+  switch (bi->cipher)
   {
-  case TALER_DENOMINATION_RSA:
+  case GNUNET_CRYPTO_BSA_RSA:
     tlen = 0;
     break;
-  case TALER_DENOMINATION_CS:
-    tlen = sizeof (struct TALER_DenominationCSPublicRPairP);
+  case GNUNET_CRYPTO_BSA_CS:
+    tlen = sizeof (struct GNUNET_CRYPTO_CSPublicRPairP);
     break;
   default:
     GNUNET_assert (0);
@@ -642,13 +647,13 @@ qconv_exchange_withdraw_values (void *cls,
   GNUNET_memcpy (buf,
                  &be,
                  sizeof (be));
-  switch (alg_values->cipher)
+  switch (bi->cipher)
   {
-  case TALER_DENOMINATION_RSA:
+  case GNUNET_CRYPTO_BSA_RSA:
     break;
-  case TALER_DENOMINATION_CS:
+  case GNUNET_CRYPTO_BSA_CS:
     GNUNET_memcpy (&buf[sizeof (be)],
-                   &alg_values->details.cs_values,
+                   &bi->details.cs_values,
                    tlen);
     break;
   default:
@@ -848,7 +853,7 @@ qconv_array (
       noerror = false; \
       goto DONE; \
     } \
-  } while(0)
+  } while (0)
 
   /* Calculate sizes and check bounds */
   {
@@ -876,6 +881,46 @@ qconv_array (
     {
       switch (meta->typ)
       {
+      case TALER_PQ_array_of_amount_currency:
+        {
+          const struct TALER_Amount *amounts = data;
+          Oid oid_v;
+          Oid oid_f;
+          Oid oid_c;
+
+          buffer_lengths  = GNUNET_new_array (num, size_t);
+          /* hoist out of loop? */
+          GNUNET_assert (GNUNET_OK ==
+                         GNUNET_PQ_get_oid_by_name (meta->db,
+                                                    "int8",
+                                                    &oid_v));
+          GNUNET_assert (GNUNET_OK ==
+                         GNUNET_PQ_get_oid_by_name (meta->db,
+                                                    "int4",
+                                                    &oid_f));
+          GNUNET_assert (GNUNET_OK ==
+                         GNUNET_PQ_get_oid_by_name (meta->db,
+                                                    "varchar",
+                                                    &oid_c));
+          for (size_t i = 0; i<num; i++)
+          {
+            struct TALER_PQ_AmountCurrencyP am;
+            size_t len;
+
+            len = TALER_PQ_make_taler_pq_amount_currency_ (
+              &amounts[i],
+              oid_v,
+              oid_f,
+              oid_c,
+              &am);
+            buffer_lengths[i] = len;
+            y = total_size;
+            total_size += len;
+            RETURN_UNLESS (total_size >= y);
+          }
+          sizes = buffer_lengths;
+          break;
+        }
       case TALER_PQ_array_of_blinded_denom_sig:
         {
           const struct TALER_BlindedDenominationSignature *denom_sigs = data;
@@ -886,16 +931,19 @@ qconv_array (
 
           for (size_t i = 0; i<num; i++)
           {
-            switch (denom_sigs[i].cipher)
+            const struct GNUNET_CRYPTO_BlindedSignature *bs =
+              denom_sigs[i].blinded_sig;
+
+            switch (bs->cipher)
             {
-            case TALER_DENOMINATION_RSA:
+            case GNUNET_CRYPTO_BSA_RSA:
               len = GNUNET_CRYPTO_rsa_signature_encode (
-                denom_sigs[i].details.blinded_rsa_signature,
+                bs->details.blinded_rsa_signature,
                 &buffers[i]);
               RETURN_UNLESS (len != 0);
               break;
-            case TALER_DENOMINATION_CS:
-              len = sizeof (denom_sigs[i].details.blinded_cs_answer);
+            case GNUNET_CRYPTO_BSA_CS:
+              len = sizeof (bs->details.blinded_cs_answer);
               break;
             default:
               GNUNET_assert (0);
@@ -955,6 +1003,7 @@ qconv_array (
           Oid oid_v;
           Oid oid_f;
 
+          /* hoist out of loop? */
           GNUNET_assert (GNUNET_OK ==
                          GNUNET_PQ_get_oid_by_name (meta->db,
                                                     "int8",
@@ -970,21 +1019,56 @@ qconv_array (
                   oid_v,
                   oid_f);
 
-            *(uint32_t *) out = htonl (2); /* number of elements in tuple */
-            out += sizeof(uint32_t);
-            sz -= sizeof(uint32_t);
             GNUNET_memcpy (out,
                            &am,
                            sizeof(am));
           }
           break;
         }
+      case TALER_PQ_array_of_amount_currency:
+        {
+          const struct TALER_Amount *amounts = data;
+          Oid oid_v;
+          Oid oid_f;
+          Oid oid_c;
+
+          /* hoist out of loop? */
+          GNUNET_assert (GNUNET_OK ==
+                         GNUNET_PQ_get_oid_by_name (meta->db,
+                                                    "int8",
+                                                    &oid_v));
+          GNUNET_assert (GNUNET_OK ==
+                         GNUNET_PQ_get_oid_by_name (meta->db,
+                                                    "int4",
+                                                    &oid_f));
+          GNUNET_assert (GNUNET_OK ==
+                         GNUNET_PQ_get_oid_by_name (meta->db,
+                                                    "varchar",
+                                                    &oid_c));
+          {
+            struct TALER_PQ_AmountCurrencyP am;
+            size_t len;
+
+            len = TALER_PQ_make_taler_pq_amount_currency_ (
+              &amounts[i],
+              oid_v,
+              oid_f,
+              oid_c,
+              &am);
+            GNUNET_memcpy (out,
+                           &am,
+                           len);
+          }
+          break;
+        }
       case TALER_PQ_array_of_blinded_denom_sig:
         {
           const struct TALER_BlindedDenominationSignature *denom_sigs = data;
+          const struct GNUNET_CRYPTO_BlindedSignature *bs =
+            denom_sigs[i].blinded_sig;
           uint32_t be[2];
 
-          be[0] = htonl ((uint32_t) denom_sigs[i].cipher);
+          be[0] = htonl ((uint32_t) bs->cipher);
           be[1] = htonl (0x01);     /* magic margker: blinded */
           GNUNET_memcpy (out,
                          &be,
@@ -992,18 +1076,18 @@ qconv_array (
           out += sizeof(be);
           sz -= sizeof(be);
 
-          switch (denom_sigs[i].cipher)
+          switch (bs->cipher)
           {
-          case TALER_DENOMINATION_RSA:
+          case GNUNET_CRYPTO_BSA_RSA:
             /* For RSA, 'same_sized' must have been false */
             GNUNET_assert (NULL != buffers);
             GNUNET_memcpy (out,
                            buffers[i],
                            sz);
             break;
-          case TALER_DENOMINATION_CS:
+          case GNUNET_CRYPTO_BSA_CS:
             GNUNET_memcpy (out,
-                           &denom_sigs[i].details.blinded_cs_answer,
+                           &bs->details.blinded_cs_answer,
                            sz);
             break;
           default:
@@ -1030,6 +1114,15 @@ qconv_array (
                          sizeof(struct TALER_DenominationHashP));
           break;
         }
+      case TALER_PQ_array_of_hash_code:
+        {
+          const struct GNUNET_HashCode *hashes = data;
+
+          GNUNET_memcpy (out,
+                         &hashes[i],
+                         sizeof(struct GNUNET_HashCode));
+          break;
+        }
       default:
         {
           GNUNET_assert (0);
@@ -1083,6 +1176,7 @@ query_param_array_generic (
   struct GNUNET_PQ_Context *db)
 {
   struct qconv_array_cls *meta = GNUNET_new (struct qconv_array_cls);
+
   meta->typ = typ;
   meta->oid = oid;
   meta->sizes = sizes;
@@ -1090,16 +1184,18 @@ query_param_array_generic (
   meta->continuous = continuous;
   meta->db = db;
 
-  struct GNUNET_PQ_QueryParam res = {
-    .conv = qconv_array,
-    .conv_cls = meta,
-    .conv_cls_cleanup = qconv_array_cls_cleanup,
-    .data = elements,
-    .size = num,
-    .num_params = 1,
-  };
+  {
+    struct GNUNET_PQ_QueryParam res = {
+      .conv = qconv_array,
+      .conv_cls = meta,
+      .conv_cls_cleanup = qconv_array_cls_cleanup,
+      .data = elements,
+      .size = num,
+      .num_params = 1,
+    };
 
-  return res;
+    return res;
+  }
 }
 
 
@@ -1110,8 +1206,11 @@ TALER_PQ_query_param_array_blinded_denom_sig (
   struct GNUNET_PQ_Context *db)
 {
   Oid oid;
+
   GNUNET_assert (GNUNET_OK ==
-                 GNUNET_PQ_get_oid_by_name (db, "bytea", &oid));
+                 GNUNET_PQ_get_oid_by_name (db,
+                                            "bytea",
+                                            &oid));
   return query_param_array_generic (num,
                                     true,
                                     denom_sigs,
@@ -1130,8 +1229,11 @@ TALER_PQ_query_param_array_blinded_coin_hash (
   struct GNUNET_PQ_Context *db)
 {
   Oid oid;
+
   GNUNET_assert (GNUNET_OK ==
-                 GNUNET_PQ_get_oid_by_name (db, "bytea", &oid));
+                 GNUNET_PQ_get_oid_by_name (db,
+                                            "bytea",
+                                            &oid));
   return query_param_array_generic (num,
                                     true,
                                     coin_hs,
@@ -1150,8 +1252,11 @@ TALER_PQ_query_param_array_denom_hash (
   struct GNUNET_PQ_Context *db)
 {
   Oid oid;
+
   GNUNET_assert (GNUNET_OK ==
-                 GNUNET_PQ_get_oid_by_name (db, "bytea", &oid));
+                 GNUNET_PQ_get_oid_by_name (db,
+                                            "bytea",
+                                            &oid));
   return query_param_array_generic (num,
                                     true,
                                     denom_hs,
@@ -1163,6 +1268,26 @@ TALER_PQ_query_param_array_denom_hash (
 }
 
 
+struct GNUNET_PQ_QueryParam
+TALER_PQ_query_param_array_hash_code (
+  size_t num,
+  const struct GNUNET_HashCode *hashes,
+  struct GNUNET_PQ_Context *db)
+{
+  Oid oid;
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_PQ_get_oid_by_name (db, "gnunet_hashcode", &oid));
+  return query_param_array_generic (num,
+                                    true,
+                                    hashes,
+                                    NULL,
+                                    sizeof(struct GNUNET_HashCode),
+                                    TALER_PQ_array_of_hash_code,
+                                    oid,
+                                    NULL);
+}
+
+
 struct GNUNET_PQ_QueryParam
 TALER_PQ_query_param_array_amount (
   size_t num,
@@ -1170,18 +1295,245 @@ TALER_PQ_query_param_array_amount (
   struct GNUNET_PQ_Context *db)
 {
   Oid oid;
+
   GNUNET_assert (GNUNET_OK ==
-                 GNUNET_PQ_get_oid_by_name (db, "taler_amount", &oid));
+                 GNUNET_PQ_get_oid_by_name (db,
+                                            "taler_amount",
+                                            &oid));
   return query_param_array_generic (
     num,
     true,
     amounts,
     NULL,
-    sizeof(uint32_t) + sizeof(struct TALER_PQ_AmountP),
+    sizeof(struct TALER_PQ_AmountP),
     TALER_PQ_array_of_amount,
     oid,
     db);
 }
 
 
+struct GNUNET_PQ_QueryParam
+TALER_PQ_query_param_array_amount_with_currency (
+  size_t num,
+  const struct TALER_Amount *amounts,
+  struct GNUNET_PQ_Context *db)
+{
+  Oid oid;
+
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_PQ_get_oid_by_name (db,
+                                            "taler_amount_currency",
+                                            &oid));
+  return query_param_array_generic (
+    num,
+    true,
+    amounts,
+    NULL,
+    0, /* currency is technically variable length */
+    TALER_PQ_array_of_amount_currency,
+    oid,
+    db);
+}
+
+
+/**
+ * Function called to convert input argument into SQL parameters.
+ *
+ * @param cls closure
+ * @param data pointer to input argument
+ * @param data_len number of bytes in @a data (if applicable)
+ * @param[out] param_values SQL data to set
+ * @param[out] param_lengths SQL length data to set
+ * @param[out] param_formats SQL format data to set
+ * @param param_length number of entries available in the @a param_values, @a param_lengths and @a param_formats arrays
+ * @param[out] scratch buffer for dynamic allocations (to be done via #GNUNET_malloc()
+ * @param scratch_length number of entries left in @a scratch
+ * @return -1 on error, number of offsets used in @a scratch otherwise
+ */
+static int
+qconv_blind_sign_pub (void *cls,
+                      const void *data,
+                      size_t data_len,
+                      void *param_values[],
+                      int param_lengths[],
+                      int param_formats[],
+                      unsigned int param_length,
+                      void *scratch[],
+                      unsigned int scratch_length)
+{
+  const struct GNUNET_CRYPTO_BlindSignPublicKey *public_key = data;
+  size_t tlen;
+  size_t len;
+  uint32_t be;
+  char *buf;
+  void *tbuf;
+
+  (void) cls;
+  (void) data_len;
+  GNUNET_assert (1 == param_length);
+  GNUNET_assert (scratch_length > 0);
+  GNUNET_break (NULL == cls);
+  be = htonl ((uint32_t) public_key->cipher);
+  switch (public_key->cipher)
+  {
+  case GNUNET_CRYPTO_BSA_RSA:
+    tlen = GNUNET_CRYPTO_rsa_public_key_encode (
+      public_key->details.rsa_public_key,
+      &tbuf);
+    break;
+  case GNUNET_CRYPTO_BSA_CS:
+    tlen = sizeof (public_key->details.cs_public_key);
+    break;
+  default:
+    GNUNET_assert (0);
+  }
+  len = tlen + sizeof (be);
+  buf = GNUNET_malloc (len);
+  GNUNET_memcpy (buf,
+                 &be,
+                 sizeof (be));
+  switch (public_key->cipher)
+  {
+  case GNUNET_CRYPTO_BSA_RSA:
+    GNUNET_memcpy (&buf[sizeof (be)],
+                   tbuf,
+                   tlen);
+    GNUNET_free (tbuf);
+    break;
+  case GNUNET_CRYPTO_BSA_CS:
+    GNUNET_memcpy (&buf[sizeof (be)],
+                   &public_key->details.cs_public_key,
+                   tlen);
+    break;
+  default:
+    GNUNET_assert (0);
+  }
+
+  scratch[0] = buf;
+  param_values[0] = (void *) buf;
+  param_lengths[0] = len;
+  param_formats[0] = 1;
+  return 1;
+}
+
+
+/**
+ * Generate query parameter for a blind sign public key of variable size.
+ *
+ * @param public_key pointer to the query parameter to pass
+ */
+struct GNUNET_PQ_QueryParam
+TALER_PQ_query_param_blind_sign_pub (
+  const struct GNUNET_CRYPTO_BlindSignPublicKey *public_key)
+{
+  struct GNUNET_PQ_QueryParam res = {
+    .conv = &qconv_blind_sign_pub,
+    .data = public_key,
+    .num_params = 1
+  };
+
+  return res;
+}
+
+
+/**
+ * Function called to convert input argument into SQL parameters.
+ *
+ * @param cls closure
+ * @param data pointer to input argument
+ * @param data_len number of bytes in @a data (if applicable)
+ * @param[out] param_values SQL data to set
+ * @param[out] param_lengths SQL length data to set
+ * @param[out] param_formats SQL format data to set
+ * @param param_length number of entries available in the @a param_values, @a param_lengths and @a param_formats arrays
+ * @param[out] scratch buffer for dynamic allocations (to be done via #GNUNET_malloc()
+ * @param scratch_length number of entries left in @a scratch
+ * @return -1 on error, number of offsets used in @a scratch otherwise
+ */
+static int
+qconv_blind_sign_priv (void *cls,
+                       const void *data,
+                       size_t data_len,
+                       void *param_values[],
+                       int param_lengths[],
+                       int param_formats[],
+                       unsigned int param_length,
+                       void *scratch[],
+                       unsigned int scratch_length)
+{
+  const struct GNUNET_CRYPTO_BlindSignPrivateKey *private_key = data;
+  size_t tlen;
+  size_t len;
+  uint32_t be;
+  char *buf;
+  void *tbuf;
+
+  (void) cls;
+  (void) data_len;
+  GNUNET_assert (1 == param_length);
+  GNUNET_assert (scratch_length > 0);
+  GNUNET_break (NULL == cls);
+  be = htonl ((uint32_t) private_key->cipher);
+  switch (private_key->cipher)
+  {
+  case GNUNET_CRYPTO_BSA_RSA:
+    tlen = GNUNET_CRYPTO_rsa_private_key_encode (
+      private_key->details.rsa_private_key,
+      &tbuf);
+    break;
+  case GNUNET_CRYPTO_BSA_CS:
+    tlen = sizeof (private_key->details.cs_private_key);
+    break;
+  default:
+    GNUNET_assert (0);
+  }
+  len = tlen + sizeof (be);
+  buf = GNUNET_malloc (len);
+  GNUNET_memcpy (buf,
+                 &be,
+                 sizeof (be));
+  switch (private_key->cipher)
+  {
+  case GNUNET_CRYPTO_BSA_RSA:
+    GNUNET_memcpy (&buf[sizeof (be)],
+                   tbuf,
+                   tlen);
+    GNUNET_free (tbuf);
+    break;
+  case GNUNET_CRYPTO_BSA_CS:
+    GNUNET_memcpy (&buf[sizeof (be)],
+                   &private_key->details.cs_private_key,
+                   tlen);
+    break;
+  default:
+    GNUNET_assert (0);
+  }
+
+  scratch[0] = buf;
+  param_values[0] = (void *) buf;
+  param_lengths[0] = len;
+  param_formats[0] = 1;
+  return 1;
+}
+
+
+/**
+ * Generate query parameter for a blind sign private key of variable size.
+ *
+ * @param private_key pointer to the query parameter to pass
+ */
+struct GNUNET_PQ_QueryParam
+TALER_PQ_query_param_blind_sign_priv (
+  const struct GNUNET_CRYPTO_BlindSignPrivateKey *private_key)
+{
+  struct GNUNET_PQ_QueryParam res = {
+    .conv = &qconv_blind_sign_priv,
+    .data = private_key,
+    .num_params = 1
+  };
+
+  return res;
+}
+
+
 /* end of pq/pq_query_helper.c */
