@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2022 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -81,7 +81,7 @@ extract_amount_currency_tuple (void *cls,
   {
     struct TALER_PQ_AmountCurrencyP ap;
     const char *in;
-    int size;
+    size_t size;
 
     size = PQgetlength (result,
                         row,
@@ -90,11 +90,11 @@ extract_amount_currency_tuple (void *cls,
          (size <= sizeof (ap) - TALER_CURRENCY_LEN) )
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Incorrect size of binary field `%s' (got %d, expected (%d-%d))\n",
+                  "Incorrect size of binary field `%s' (got %zu, expected (%zu-%zu))\n",
                   fname,
                   size,
-                  (int) (sizeof (ap) - TALER_CURRENCY_LEN),
-                  (int) sizeof (ap));
+                  sizeof (ap) - TALER_CURRENCY_LEN,
+                  sizeof (ap));
       return GNUNET_SYSERR;
     }
 
@@ -222,44 +222,57 @@ extract_amount_tuple (void *cls,
 
   /* Parse the tuple */
   {
-    const char *in;
-    uint32_t num;
     struct TALER_PQ_AmountP ap;
-    int size;
-    const static int expected_size = sizeof(uint32_t) /* length */
-                                     + sizeof(ap);
+    const char *in;
+    size_t size;
 
     size = PQgetlength (result,
                         row,
                         col);
-
-    if (expected_size != size)
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Incorrect size of binary field `%s' (got %d, expected %d)\n",
-                  fname,
-                  size,
-                  expected_size);
-      return GNUNET_SYSERR;
-    }
-
     in = PQgetvalue (result,
                      row,
                      col);
+    if (sizeof(struct TALER_PQ_AmountNullP) == size)
+    {
+      struct TALER_PQ_AmountNullP apn;
 
-    num = ntohl (*(uint32_t *) in);
-    if (2 != num)
+      memcpy (&apn,
+              in,
+              size);
+      if ( (2 == ntohl (apn.cnt)) &&
+           (-1 == (int32_t) ntohl (apn.sz_v)) &&
+           (-1 == (int32_t) ntohl (apn.sz_f)) )
+      {
+        /* is NULL! */
+        return GNUNET_NO;
+      }
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Incorrect size of binary field `%s' and not NULL (got %zu, expected %zu)\n",
+                  fname,
+                  size,
+                  sizeof(ap));
+      return GNUNET_SYSERR;
+    }
+    if (sizeof(ap) != size)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Incorrect size of binary field `%s' (got %zu, expected %zu)\n",
+                  fname,
+                  size,
+                  sizeof(ap));
+      return GNUNET_SYSERR;
+    }
+
+    memcpy (&ap,
+            in,
+            size);
+    if (2 != ntohl (ap.cnt))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Incorrect number of elements in tuple-field `%s'\n",
                   fname);
       return GNUNET_SYSERR;
     }
-    in += sizeof(uint32_t);
-    memcpy (&ap,
-            in,
-            sizeof (ap));
-
     /* TODO[oec]: OID-checks? */
 
     r_amount->value = GNUNET_ntohll (ap.v);
@@ -433,6 +446,7 @@ extract_denom_pub (void *cls,
                    void *dst)
 {
   struct TALER_DenominationPublicKey *pk = dst;
+  struct GNUNET_CRYPTO_BlindSignPublicKey *bpk;
   size_t len;
   const char *res;
   int fnum;
@@ -470,33 +484,47 @@ extract_denom_pub (void *cls,
                  sizeof (be));
   res += sizeof (be);
   len -= sizeof (be);
-  pk->cipher = ntohl (be[0]);
+  bpk = GNUNET_new (struct GNUNET_CRYPTO_BlindSignPublicKey);
+  bpk->cipher = ntohl (be[0]);
+  bpk->rc = 1;
   pk->age_mask.bits = ntohl (be[1]);
-  switch (pk->cipher)
+  switch (bpk->cipher)
   {
-  case TALER_DENOMINATION_RSA:
-    pk->details.rsa_public_key
+  case GNUNET_CRYPTO_BSA_INVALID:
+    break;
+  case GNUNET_CRYPTO_BSA_RSA:
+    bpk->details.rsa_public_key
       = GNUNET_CRYPTO_rsa_public_key_decode (res,
                                              len);
-    if (NULL == pk->details.rsa_public_key)
+    if (NULL == bpk->details.rsa_public_key)
     {
       GNUNET_break (0);
+      GNUNET_free (bpk);
       return GNUNET_SYSERR;
     }
+    pk->bsign_pub_key = bpk;
+    GNUNET_CRYPTO_hash (res,
+                        len,
+                        &bpk->pub_key_hash);
     return GNUNET_OK;
-  case TALER_DENOMINATION_CS:
-    if (sizeof (pk->details.cs_public_key) != len)
+  case GNUNET_CRYPTO_BSA_CS:
+    if (sizeof (bpk->details.cs_public_key) != len)
     {
       GNUNET_break (0);
+      GNUNET_free (bpk);
       return GNUNET_SYSERR;
     }
-    GNUNET_memcpy (&pk->details.cs_public_key,
+    GNUNET_memcpy (&bpk->details.cs_public_key,
                    res,
                    len);
+    pk->bsign_pub_key = bpk;
+    GNUNET_CRYPTO_hash (res,
+                        len,
+                        &bpk->pub_key_hash);
     return GNUNET_OK;
-  default:
-    GNUNET_break (0);
   }
+  GNUNET_break (0);
+  GNUNET_free (bpk);
   return GNUNET_SYSERR;
 }
 
@@ -556,6 +584,7 @@ extract_denom_sig (void *cls,
                    void *dst)
 {
   struct TALER_DenominationSignature *sig = dst;
+  struct GNUNET_CRYPTO_UnblindedSignature *ubs;
   size_t len;
   const char *res;
   int fnum;
@@ -598,32 +627,40 @@ extract_denom_sig (void *cls,
   }
   res += sizeof (be);
   len -= sizeof (be);
-  sig->cipher = ntohl (be[0]);
-  switch (sig->cipher)
+  ubs = GNUNET_new (struct GNUNET_CRYPTO_UnblindedSignature);
+  ubs->rc = 1;
+  ubs->cipher = ntohl (be[0]);
+  switch (ubs->cipher)
   {
-  case TALER_DENOMINATION_RSA:
-    sig->details.rsa_signature
+  case GNUNET_CRYPTO_BSA_INVALID:
+    break;
+  case GNUNET_CRYPTO_BSA_RSA:
+    ubs->details.rsa_signature
       = GNUNET_CRYPTO_rsa_signature_decode (res,
                                             len);
-    if (NULL == sig->details.rsa_signature)
+    if (NULL == ubs->details.rsa_signature)
     {
       GNUNET_break (0);
+      GNUNET_free (ubs);
       return GNUNET_SYSERR;
     }
+    sig->unblinded_sig = ubs;
     return GNUNET_OK;
-  case TALER_DENOMINATION_CS:
-    if (sizeof (sig->details.cs_signature) != len)
+  case GNUNET_CRYPTO_BSA_CS:
+    if (sizeof (ubs->details.cs_signature) != len)
     {
       GNUNET_break (0);
+      GNUNET_free (ubs);
       return GNUNET_SYSERR;
     }
-    GNUNET_memcpy (&sig->details.cs_signature,
+    GNUNET_memcpy (&ubs->details.cs_signature,
                    res,
                    len);
+    sig->unblinded_sig = ubs;
     return GNUNET_OK;
-  default:
-    GNUNET_break (0);
   }
+  GNUNET_break (0);
+  GNUNET_free (ubs);
   return GNUNET_SYSERR;
 }
 
@@ -683,6 +720,7 @@ extract_blinded_denom_sig (void *cls,
                            void *dst)
 {
   struct TALER_BlindedDenominationSignature *sig = dst;
+  struct GNUNET_CRYPTO_BlindedSignature *bs;
   size_t len;
   const char *res;
   int fnum;
@@ -725,32 +763,40 @@ extract_blinded_denom_sig (void *cls,
   }
   res += sizeof (be);
   len -= sizeof (be);
-  sig->cipher = ntohl (be[0]);
-  switch (sig->cipher)
+  bs = GNUNET_new (struct GNUNET_CRYPTO_BlindedSignature);
+  bs->rc = 1;
+  bs->cipher = ntohl (be[0]);
+  switch (bs->cipher)
   {
-  case TALER_DENOMINATION_RSA:
-    sig->details.blinded_rsa_signature
+  case GNUNET_CRYPTO_BSA_INVALID:
+    break;
+  case GNUNET_CRYPTO_BSA_RSA:
+    bs->details.blinded_rsa_signature
       = GNUNET_CRYPTO_rsa_signature_decode (res,
                                             len);
-    if (NULL == sig->details.blinded_rsa_signature)
+    if (NULL == bs->details.blinded_rsa_signature)
     {
       GNUNET_break (0);
+      GNUNET_free (bs);
       return GNUNET_SYSERR;
     }
+    sig->blinded_sig = bs;
     return GNUNET_OK;
-  case TALER_DENOMINATION_CS:
-    if (sizeof (sig->details.blinded_cs_answer) != len)
+  case GNUNET_CRYPTO_BSA_CS:
+    if (sizeof (bs->details.blinded_cs_answer) != len)
     {
       GNUNET_break (0);
+      GNUNET_free (bs);
       return GNUNET_SYSERR;
     }
-    GNUNET_memcpy (&sig->details.blinded_cs_answer,
+    GNUNET_memcpy (&bs->details.blinded_cs_answer,
                    res,
                    len);
+    sig->blinded_sig = bs;
     return GNUNET_OK;
-  default:
-    GNUNET_break (0);
   }
+  GNUNET_break (0);
+  GNUNET_free (bs);
   return GNUNET_SYSERR;
 }
 
@@ -811,6 +857,7 @@ extract_blinded_planchet (void *cls,
                           void *dst)
 {
   struct TALER_BlindedPlanchet *bp = dst;
+  struct GNUNET_CRYPTO_BlindedMessage *bm;
   size_t len;
   const char *res;
   int fnum;
@@ -853,29 +900,36 @@ extract_blinded_planchet (void *cls,
   }
   res += sizeof (be);
   len -= sizeof (be);
-  bp->cipher = ntohl (be[0]);
-  switch (bp->cipher)
+  bm = GNUNET_new (struct GNUNET_CRYPTO_BlindedMessage);
+  bm->rc = 1;
+  bm->cipher = ntohl (be[0]);
+  switch (bm->cipher)
   {
-  case TALER_DENOMINATION_RSA:
-    bp->details.rsa_blinded_planchet.blinded_msg_size
+  case GNUNET_CRYPTO_BSA_INVALID:
+    break;
+  case GNUNET_CRYPTO_BSA_RSA:
+    bm->details.rsa_blinded_message.blinded_msg_size
       = len;
-    bp->details.rsa_blinded_planchet.blinded_msg
+    bm->details.rsa_blinded_message.blinded_msg
       = GNUNET_memdup (res,
                        len);
+    bp->blinded_message = bm;
     return GNUNET_OK;
-  case TALER_DENOMINATION_CS:
-    if (sizeof (bp->details.cs_blinded_planchet) != len)
+  case GNUNET_CRYPTO_BSA_CS:
+    if (sizeof (bm->details.cs_blinded_message) != len)
     {
       GNUNET_break (0);
+      GNUNET_free (bm);
       return GNUNET_SYSERR;
     }
-    GNUNET_memcpy (&bp->details.cs_blinded_planchet,
+    GNUNET_memcpy (&bm->details.cs_blinded_message,
                    res,
                    len);
+    bp->blinded_message = bm;
     return GNUNET_OK;
-  default:
-    GNUNET_break (0);
   }
+  GNUNET_break (0);
+  GNUNET_free (bm);
   return GNUNET_SYSERR;
 }
 
@@ -936,6 +990,7 @@ extract_exchange_withdraw_values (void *cls,
                                   void *dst)
 {
   struct TALER_ExchangeWithdrawValues *alg_values = dst;
+  struct GNUNET_CRYPTO_BlindingInputValues *bi;
   size_t len;
   const char *res;
   int fnum;
@@ -978,29 +1033,37 @@ extract_exchange_withdraw_values (void *cls,
   }
   res += sizeof (be);
   len -= sizeof (be);
-  alg_values->cipher = ntohl (be[0]);
-  switch (alg_values->cipher)
+  bi = GNUNET_new (struct GNUNET_CRYPTO_BlindingInputValues);
+  bi->rc = 1;
+  bi->cipher = ntohl (be[0]);
+  switch (bi->cipher)
   {
-  case TALER_DENOMINATION_RSA:
+  case GNUNET_CRYPTO_BSA_INVALID:
+    break;
+  case GNUNET_CRYPTO_BSA_RSA:
     if (0 != len)
     {
       GNUNET_break (0);
+      GNUNET_free (bi);
       return GNUNET_SYSERR;
     }
+    alg_values->blinding_inputs = bi;
     return GNUNET_OK;
-  case TALER_DENOMINATION_CS:
-    if (sizeof (struct TALER_DenominationCSPublicRPairP) != len)
+  case GNUNET_CRYPTO_BSA_CS:
+    if (sizeof (bi->details.cs_values) != len)
     {
       GNUNET_break (0);
+      GNUNET_free (bi);
       return GNUNET_SYSERR;
     }
-    GNUNET_memcpy (&alg_values->details.cs_values,
+    GNUNET_memcpy (&bi->details.cs_values,
                    res,
                    len);
+    alg_values->blinding_inputs = bi;
     return GNUNET_OK;
-  default:
-    GNUNET_break (0);
   }
+  GNUNET_break (0);
+  GNUNET_free (bi);
   return GNUNET_SYSERR;
 }
 
@@ -1026,29 +1089,44 @@ TALER_PQ_result_spec_exchange_withdraw_values (
  */
 struct ArrayResultCls
 {
-  /* Oid of the expected type, must match the oid in the header of the PQResult struct */
+  /**
+   * Oid of the expected type, must match the oid in the header of the PQResult struct
+   */
   Oid oid;
 
-  /* Target type */
+  /**
+   * Target type
+   */
   enum TALER_PQ_ArrayType typ;
 
-  /* If not 0, defines the expected size of each entry */
+  /**
+   * If not 0, defines the expected size of each entry
+   */
   size_t same_size;
 
-  /* Out-pointer to write the number of elements in the array */
+  /**
+   * Out-pointer to write the number of elements in the array
+   */
   size_t *num;
 
-  /* Out-pointer. If @a typ is TALER_PQ_array_of_byte and @a same_size is 0,
-   * allocate and put the array of @a num sizes here. NULL otherwise */
+  /**
+   * Out-pointer. If @a typ is TALER_PQ_array_of_byte and @a same_size is 0,
+   * allocate and put the array of @a num sizes here. NULL otherwise
+   */
   size_t **sizes;
 
-  /* DB_connection, needed for OID-lookup for composite types */
+  /**
+   * DB_connection, needed for OID-lookup for composite types
+   */
   const struct GNUNET_PQ_Context *db;
 
-  /* Currency information for amount composites */
+  /**
+   * Currency information for amount composites
+   */
   char currency[TALER_CURRENCY_LEN];
 };
 
+
 /**
  * Extract data from a Postgres database @a result as array of a specific type
  * from row @a row.  The type information and optionally additional
@@ -1090,7 +1168,7 @@ extract_array_generic (
       GNUNET_break (! (cond)); \
       goto FAIL; \
     } \
-  } while(0)
+  } while (0)
 
   col_num = PQfnumber (result, fname);
   FAIL_IF (0 > col_num);
@@ -1133,27 +1211,31 @@ extract_array_generic (
         if (NULL != dst_size)
           *dst_size = sizeof(struct TALER_Amount) * (header.dim);
 
-        amounts = GNUNET_new_array (header.dim, struct TALER_Amount);
+        amounts = GNUNET_new_array (header.dim,
+                                    struct TALER_Amount);
         *((void **) dst) = amounts;
 
         for (uint32_t i = 0; i < header.dim; i++)
         {
           struct TALER_PQ_AmountP ap;
           struct TALER_Amount *amount = &amounts[i];
-          size_t sz = ntohl (*(uint32_t *) in);
-          in += sizeof(uint32_t);
+          uint32_t val;
+          size_t sz;
+
+          GNUNET_memcpy (&val,
+                         in,
+                         sizeof(val));
+          sz =  ntohl (val);
+          in += sizeof(val);
 
           /* total size for this array-entry */
-          FAIL_IF ((sizeof(uint32_t)
-                    + sizeof(struct TALER_PQ_AmountP))
-                   > sz);
+          FAIL_IF (sizeof(ap) != sz);
 
-          /* number of elements in composite type*/
-          sz = ntohl (*(uint32_t *) in);
-          in += sizeof(uint32_t);
-          FAIL_IF (2 != sz);
+          GNUNET_memcpy (&ap,
+                         in,
+                         sz);
+          FAIL_IF (2 != ntohl (ap.cnt));
 
-          ap = *(struct TALER_PQ_AmountP *) in;
           amount->value = GNUNET_ntohll (ap.v);
           amount->fraction = ntohl (ap.f);
           GNUNET_memcpy (amount->currency,
@@ -1167,11 +1249,18 @@ extract_array_generic (
     case TALER_PQ_array_of_denom_hash:
       if (NULL != dst_size)
         *dst_size = sizeof(struct TALER_DenominationHashP) * (header.dim);
-      out = GNUNET_new_array (header.dim, struct TALER_DenominationHashP);
+      out = GNUNET_new_array (header.dim,
+                              struct TALER_DenominationHashP);
       *((void **) dst) = out;
       for (uint32_t i = 0; i < header.dim; i++)
       {
-        size_t sz =  ntohl (*(uint32_t *) in);
+        uint32_t val;
+        size_t sz;
+
+        GNUNET_memcpy (&val,
+                       in,
+                       sizeof(val));
+        sz =  ntohl (val);
         FAIL_IF (sz != sizeof(struct TALER_DenominationHashP));
         in += sizeof(uint32_t);
         *(struct TALER_DenominationHashP *) out =
@@ -1181,14 +1270,45 @@ extract_array_generic (
       }
       return GNUNET_OK;
 
-    case TALER_PQ_array_of_blinded_coin_hash:
+    case TALER_PQ_array_of_hash_code:
       if (NULL != dst_size)
-        *dst_size = sizeof(struct TALER_BlindedCoinHashP) * (header.dim);
-      out = GNUNET_new_array (header.dim, struct TALER_BlindedCoinHashP);
+        *dst_size = sizeof(struct GNUNET_HashCode) * (header.dim);
+      out = GNUNET_new_array (header.dim,
+                              struct GNUNET_HashCode);
       *((void **) dst) = out;
       for (uint32_t i = 0; i < header.dim; i++)
       {
-        size_t sz =  ntohl (*(uint32_t *) in);
+        uint32_t val;
+        size_t sz;
+
+        GNUNET_memcpy (&val,
+                       in,
+                       sizeof(val));
+        sz =  ntohl (val);
+        FAIL_IF (sz != sizeof(struct GNUNET_HashCode));
+        in += sizeof(uint32_t);
+        *(struct GNUNET_HashCode *) out =
+          *(struct GNUNET_HashCode *) in;
+        in += sz;
+        out += sz;
+      }
+      return GNUNET_OK;
+
+    case TALER_PQ_array_of_blinded_coin_hash:
+      if (NULL != dst_size)
+        *dst_size = sizeof(struct TALER_BlindedCoinHashP) * (header.dim);
+      out = GNUNET_new_array (header.dim,
+                              struct TALER_BlindedCoinHashP);
+      *((void **) dst) = out;
+      for (uint32_t i = 0; i < header.dim; i++)
+      {
+        uint32_t val;
+        size_t sz;
+
+        GNUNET_memcpy (&val,
+                       in,
+                       sizeof(val));
+        sz =  ntohl (val);
         FAIL_IF (sz != sizeof(struct TALER_BlindedCoinHashP));
         in += sizeof(uint32_t);
         *(struct TALER_BlindedCoinHashP *) out =
@@ -1216,11 +1336,18 @@ extract_array_generic (
         for (uint32_t i = 0; i < header.dim; i++)
         {
           struct TALER_BlindedDenominationSignature *denom_sig = &denom_sigs[i];
+          struct GNUNET_CRYPTO_BlindedSignature *bs;
           uint32_t be[2];
-          size_t sz =  ntohl (*(uint32_t *) in);
-          in += sizeof(uint32_t);
+          uint32_t val;
+          size_t sz;
 
+          GNUNET_memcpy (&val,
+                         in,
+                         sizeof(val));
+          sz = ntohl (val);
           FAIL_IF (sizeof(be) > sz);
+
+          in += sizeof(val);
           GNUNET_memcpy (&be,
                          in,
                          sizeof(be));
@@ -1228,28 +1355,36 @@ extract_array_generic (
 
           in += sizeof(be);
           sz -= sizeof(be);
-
-          denom_sig->cipher = ntohl (be[0]);
-          switch (denom_sig->cipher)
+          bs = GNUNET_new (struct GNUNET_CRYPTO_BlindedSignature);
+          bs->cipher = ntohl (be[0]);
+          bs->rc = 1;
+          switch (bs->cipher)
           {
-          case TALER_DENOMINATION_RSA:
-            denom_sig->details.blinded_rsa_signature =
-              GNUNET_CRYPTO_rsa_signature_decode (in,
-                                                  sz);
-            FAIL_IF (NULL == denom_sig->details.blinded_rsa_signature);
+          case GNUNET_CRYPTO_BSA_RSA:
+            bs->details.blinded_rsa_signature
+              = GNUNET_CRYPTO_rsa_signature_decode (in,
+                                                    sz);
+            if (NULL == bs->details.blinded_rsa_signature)
+            {
+              GNUNET_free (bs);
+              FAIL_IF (true);
+            }
             break;
-
-          case TALER_DENOMINATION_CS:
-            FAIL_IF (sizeof(denom_sig->details.blinded_cs_answer) != sz);
-            GNUNET_memcpy (&denom_sig->details.blinded_cs_answer,
+          case GNUNET_CRYPTO_BSA_CS:
+            if (sizeof(bs->details.blinded_cs_answer) != sz)
+            {
+              GNUNET_free (bs);
+              FAIL_IF (true);
+            }
+            GNUNET_memcpy (&bs->details.blinded_cs_answer,
                            in,
                            sz);
             break;
-
           default:
+            GNUNET_free (bs);
             FAIL_IF (true);
           }
-
+          denom_sig->blinded_sig = bs;
           in += sz;
         }
         return GNUNET_OK;
@@ -1258,12 +1393,10 @@ extract_array_generic (
       FAIL_IF (true);
     }
   }
-
 FAIL:
   GNUNET_free (*(void **) dst);
   return GNUNET_SYSERR;
-  #undef FAIL_IF
-
+#undef FAIL_IF
 }
 
 
@@ -1274,7 +1407,6 @@ static void
 array_cleanup (void *cls,
                void *rd)
 {
-
   struct ArrayResultCls *info = cls;
   void **dst = rd;
 
@@ -1282,6 +1414,15 @@ array_cleanup (void *cls,
       (NULL != info->sizes))
     GNUNET_free (*(info->sizes));
 
+  /* Clean up signatures, if applicable */
+  if (TALER_PQ_array_of_blinded_denom_sig == info->typ)
+  {
+    struct TALER_BlindedDenominationSignature *denom_sigs = *dst;
+    GNUNET_assert (NULL != info->num);
+    for (size_t i = 0; i < *info->num; i++)
+      GNUNET_free (denom_sigs[i].blinded_sig);
+  }
+
   GNUNET_free (cls);
   GNUNET_free (*dst);
   *dst = NULL;
@@ -1408,4 +1549,34 @@ TALER_PQ_result_spec_array_amount (
 }
 
 
+struct GNUNET_PQ_ResultSpec
+TALER_PQ_result_spec_array_hash_code (
+  struct GNUNET_PQ_Context *db,
+  const char *name,
+  size_t *num,
+  struct GNUNET_HashCode **hashes)
+{
+  struct ArrayResultCls *info = GNUNET_new (struct ArrayResultCls);
+
+  info->num = num;
+  info->typ = TALER_PQ_array_of_hash_code;
+  info->db = db;
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_PQ_get_oid_by_name (db,
+                                            "gnunet_hashcode",
+                                            &info->oid));
+
+  struct GNUNET_PQ_ResultSpec res = {
+    .conv = extract_array_generic,
+    .cleaner = array_cleanup,
+    .dst = (void *) hashes,
+    .fname = name,
+    .cls = info,
+  };
+  return res;
+
+
+}
+
+
 /* end of pq_result_helper.c */
