@@ -5,6 +5,9 @@ set -eu
 # even bother testing for it in configure.ac.
 # However, in that case, skip the test suite.
 
-make -f Makefile.orig mustach || exit 77
-make -f Makefile.orig test
-make -f Makefile.orig clean || true
+export CFLAGS="-g"
+
+make -f mustach-original-Makefile mustach || exit 77
+make -f mustach-original-Makefile clean || true
+make -f mustach-original-Makefile basic-tests
+make -f mustach-original-Makefile clean || true
