@@ -1,7 +1,7 @@
 #!/bin/bash
 #
 # This file is part of TALER
-# Copyright (C) 2023 Taler Systems SA
+# Copyright (C) 2023, 2024 Taler Systems SA
 #
 # TALER is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -48,7 +48,7 @@ function exit_fail() {
 # Cleanup to run whenever we exit
 function cleanup()
 {
-    echo "Taler unified setup terminating!" >&2
+    echo "Taler unified setup terminating at $STAGE!" >&2
 
     for n in $(jobs -p)
     do
@@ -59,6 +59,8 @@ function cleanup()
     exit "$EXIT_STATUS"
 }
 
+STAGE="boot"
+
 # Install cleanup handler (except for kill -9)
 trap cleanup EXIT
 
@@ -67,12 +69,17 @@ START_AUDITOR=0
 START_BACKUP=0
 START_EXCHANGE=0
 START_FAKEBANK=0
+START_DONAU=0
+START_CHALLENGER=0
 START_AGGREGATOR=0
 START_MERCHANT=0
 START_NEXUS=0
-START_SANDBOX=0
+START_BANK=0
 START_TRANSFER=0
 START_WIREWATCH=0
+START_DEPOSITCHECK=0
+START_MERCHANT_EXCHANGE=0
+START_MERCHANT_WIREWATCH=0
 USE_ACCOUNT="exchange-account-1"
 USE_VALGRIND=""
 WIRE_DOMAIN="x-taler-bank"
@@ -81,13 +88,13 @@ LOGLEVEL="DEBUG"
 DEFAULT_SLEEP="0.2"
 
 # Parse command-line options
-while getopts ':abc:d:efghL:mnr:stu:vwW' OPTION; do
+while getopts ':abc:d:DeEfghkL:mMnr:stu:vwWz' OPTION; do
     case "$OPTION" in
         a)
             START_AUDITOR="1"
             ;;
         b)
-            START_BACKUP="1"
+            START_BANK="1"
             ;;
         c)
             CONF_ORIG="$OPTARG"
@@ -95,47 +102,65 @@ while getopts ':abc:d:efghL:mnr:stu:vwW' OPTION; do
         d)
             WIRE_DOMAIN="$OPTARG"
             ;;
+        D)
+            START_DONAU="1"
+            ;;
         e)
             START_EXCHANGE="1"
             ;;
+        E)
+            START_MERCHANT_EXCHANGE="1"
+            ;;
         f)
             START_FAKEBANK="1"
             ;;
         h)
             echo 'Supported options:'
             echo '  -a           -- start auditor'
-            echo '  -b           -- start backup/sync'
+            echo '  -b           -- start bank'
             # shellcheck disable=SC2016
             echo '  -c $CONF     -- set configuration'
             # shellcheck disable=SC2016
             echo '  -d $METHOD   -- use wire method (default: x-taler-bank)'
+            echo '  -D           -- start donau'
             echo '  -e           -- start exchange'
+            echo '  -E           -- start taler-merchant-exchange'
             echo '  -f           -- start fakebank'
-            echo '  -g           -- start aggregator'
+            echo '  -g           -- start taler-exchange-aggregator'
             echo '  -h           -- print this help'
+            echo '  -k           -- start challenger (KYC service)'
             # shellcheck disable=SC2016
             echo '  -L $LOGLEVEL -- set log level'
-            echo '  -m           -- start merchant'
+            echo '  -m           -- start taler-merchant'
+            echo '  -M           -- start taler-merchant-depositcheck'
             echo '  -n           -- start nexus'
             # shellcheck disable=SC2016
             echo '  -r $MEX      -- which exchange to use at the merchant (optional)'
-            echo '  -s           -- start sandbox'
-            echo '  -t           -- start transfer'
+            echo '  -s           -- start backup/sync'
+            echo '  -t           -- start taler-exchange-transfer'
             # shellcheck disable=SC2016
             echo '  -u $SECTION  -- exchange account to use'
             echo '  -v           -- use valgrind'
-            echo '  -w           -- start wirewatch'
+            echo '  -w           -- start taler-exchange-wirewatch'
+            echo '  -W           -- wait for signal'
+            echo '  -z           -- start taler-merchant-wirewatch'
             exit 0
             ;;
         g)
             START_AGGREGATOR="1"
             ;;
+        k)
+            START_CHALLENGER="1"
+            ;;
         L)
             LOGLEVEL="$OPTARG"
             ;;
         m)
             START_MERCHANT="1"
             ;;
+        M)
+            START_DEPOSITCHECK="1"
+            ;;
         n)
             START_NEXUS="1"
             ;;
@@ -143,7 +168,7 @@ while getopts ':abc:d:efghL:mnr:stu:vwW' OPTION; do
             USE_MERCHANT_EXCHANGE="$OPTARG"
             ;;
         s)
-            START_SANDBOX="1"
+            START_BACKUP="1"
             ;;
         t)
             START_TRANSFER="1"
@@ -161,20 +186,31 @@ while getopts ':abc:d:efghL:mnr:stu:vwW' OPTION; do
         W)
             WAIT_FOR_SIGNAL="1"
             ;;
+        z)
+            START_MERCHANT_WIREWATCH="1"
+            ;;
         ?)
         exit_fail "Unrecognized command line option"
         ;;
     esac
 done
 
+STAGE="init"
+
 echo "Starting with configuration file at: $CONF_ORIG"
 CONF="$CONF_ORIG.edited"
 cp "${CONF_ORIG}" "${CONF}"
 
+STAGE="checks"
+
 echo -n "Testing for jq"
 jq -h > /dev/null || exit_skip " jq required"
 echo " FOUND"
 
+echo -n "Testing for wget"
+wget --help > /dev/null || exit_skip " wget required"
+echo " FOUND"
+
 if [ "1" = "$START_EXCHANGE" ]
 then
     echo -n "Testing for Taler exchange"
@@ -182,6 +218,13 @@ then
     echo " FOUND"
 fi
 
+if [ "1" = "$START_DONAU" ]
+then
+    echo -n "Testing for Donau"
+    donau-httpd -h > /dev/null || exit_skip " donau-httpd required"
+    echo " FOUND"
+fi
+
 if [ "1" = "$START_MERCHANT" ]
 then
     echo -n "Testing for Taler merchant"
@@ -189,6 +232,13 @@ then
     echo " FOUND"
 fi
 
+if [ "1" = "$START_CHALLENGER" ]
+then
+    echo -n "Testing for Taler challenger"
+    challenger-httpd -h > /dev/null || exit_skip " challenger-httpd required"
+    echo " FOUND"
+fi
+
 if [ "1" = "$START_BACKUP" ]
 then
     echo -n "Testing for sync-httpd"
@@ -203,39 +253,68 @@ then
     echo " FOUND"
 fi
 
+if [ "1" = "$START_BANK" ]
+then
+    echo -n "Testing for libeufin-bank"
+    libeufin-bank --help >/dev/null </dev/null || exit_skip " MISSING"
+    echo " FOUND"
+fi
+
+STAGE="config"
+
+
 EXCHANGE_URL=$(taler-config -c "$CONF" -s "EXCHANGE" -o "BASE_URL")
 CURRENCY=$(taler-config -c "$CONF" -s "TALER" -o "CURRENCY")
 
 echo "Setting up for $CURRENCY at $EXCHANGE_URL"
 
-register_sandbox_account() {
-    export LIBEUFIN_SANDBOX_USERNAME="$1"
-    export LIBEUFIN_SANDBOX_PASSWORD="$2"
-    # FIXME-MS: delete should be removed after we make 'register' idempotent!
-    libeufin-cli sandbox \
-      demobank \
-      delete \
-      --bank-account "$1" &> /dev/null || true
-
-    MAYBE_IBAN="${4:-}"
-    if test -n "$MAYBE_IBAN"; then
-      libeufin-cli sandbox \
-        demobank \
-        register --name "$3" --iban "$MAYBE_IBAN"
+register_bank_account() {
+    wget \
+        --http-user="$AUSER" \
+        --http-password="$APASS" \
+        --method=DELETE \
+        -o /dev/null \
+        -O /dev/null \
+        -a wget-delete-account.log \
+        "http://localhost:${BANK_PORT}/accounts/$1" \
+        || true # deletion may fail, that's OK!
+    if [ "$1" = "exchange" ] || [ "$1" = "Exchange" ]
+    then
+        IS_EXCHANGE="true"
     else
-      libeufin-cli sandbox \
-        demobank \
-        register --name "$3"
+        IS_EXCHANGE="false"
     fi
-
-    unset LIBEUFIN_SANDBOX_USERNAME
-    unset LIBEUFIN_SANDBOX_PASSWORD
+    MAYBE_IBAN="${4:-}"
+    if [ -n "$MAYBE_IBAN" ]
+    then
+        # shellcheck disable=SC2001
+        ENAME=$(echo "$3" | sed -e "s/ /+/g")
+        # Note: this assumes that $3 has no spaces. Should probably escape in the future..
+        PAYTO="payto://iban/SANDBOXX/${MAYBE_IBAN}?receiver-name=$ENAME"
+        BODY='{"username":"'"$1"'","password":"'"$2"'","is_taler_exchange":'"$IS_EXCHANGE"',"name":"'"$3"'","payto_uri":"'"$PAYTO"'"}'
+    else
+        BODY='{"username":"'"$1"'","password":"'"$2"'","is_taler_exchange":'"$IS_EXCHANGE"',"name":"'"$3"'"}'
+    fi
+    wget \
+        --http-user="$AUSER" \
+        --http-password="$APASS" \
+        --method=POST \
+        --header='Content-type: application/json' \
+        --body-data="${BODY}" \
+        -o /dev/null \
+        -O /dev/null \
+        -a wget-register-account.log \
+        "http://localhost:${BANK_PORT}/accounts"
 }
 
 register_fakebank_account() {
-    # FIXME: may need to pass 'is_taler_exchange' here
-    # in the future to get exchange facade from libeufin!
-    BODY='{"username":"'"$1"'","password":"'"$2"'","name":"'"$1"'"}'
+    if [ "$1" = "exchange" ] || [ "$1" = "Exchange" ]
+    then
+        IS_EXCHANGE="true"
+    else
+        IS_EXCHANGE="false"
+    fi
+    BODY='{"username":"'"$1"'","password":"'"$2"'","name":"'"$1"'","is_taler_exchange":'"$IS_EXCHANGE"'}'
     wget \
         --post-data="$BODY" \
         --header='Content-type: application/json' \
@@ -243,168 +322,114 @@ register_fakebank_account() {
         --waitretry=1 \
         --timeout=30 \
         "http://localhost:$BANK_PORT/accounts" \
+        -a wget-register-account.log \
         -o /dev/null \
         -O /dev/null \
         >/dev/null
 }
 
 
-if [[ "1" = "$START_NEXUS" || "1" = "$START_FAKEBANK" ]]
+if [[ "1" = "$START_BANK" ]]
 then
-    BANK_PORT=$(taler-config -c "$CONF" -s "BANK" -o "HTTP_PORT")
-    if [ "1" = "$START_NEXUS" ]
-    then
-        NEXUS_PORT="$BANK_PORT"
-        SANDBOX_PORT="1$BANK_PORT"
-    else
-        NEXUS_PORT="0"
-        SANDBOX_PORT="1$BANK_PORT"
-    fi
-else
-    if [ "1" = "$START_SANDBOX" ]
-    then
-        BANK_PORT=$(taler-config -c "$CONF" -s "BANK" -o "HTTP_PORT")
-        SANDBOX_PORT="$BANK_PORT"
-    fi
+    BANK_PORT=$(taler-config -c "$CONF" -s "libeufin-bank" -o "PORT")
+    BANK_URL="http://localhost:${BANK_PORT}/"
 fi
 
-if [ "1" = "$START_SANDBOX" ]
+if [[ "1" = "$START_FAKEBANK" ]]
 then
-    #
-    LIBEUFIN_SANDBOX_DB_CONNECTION=$(taler-config -c "$CONF" -s "libeufin-sandbox" -o "DB_CONNECTION")
-    if [ -n "${PGHOST:+}" ]
-    then
-        EHOST=$(echo "$PGHOST" | sed -e "s/\//\\\\\//g")
-        LIBEUFIN_SANDBOX_DB_CONNECTION=$(echo "$LIBEUFIN_SANDBOX_DB_CONNECTION" | sed -e "s/\/var\/run\/postgresql/$EHOST/")
-        taler-config -c "$CONF" -s "libeufin-sandbox" -o "DB_CONNECTION" -V "$LIBEUFIN_SANDBOX_DB_CONNECTION"
-    fi
-    export LIBEUFIN_SANDBOX_DB_CONNECTION
-    # Create the default demobank.
-    echo -n "Configuring sandbox at ${LIBEUFIN_SANDBOX_DB_CONNECTION} "
+    BANK_PORT=$(taler-config -c "$CONF" -s "BANK" -o "HTTP_PORT")
+    BANK_URL="http://localhost:${BANK_PORT}/"
+fi
 
-    libeufin-sandbox reset-tables \
-        &> libeufin-sandbox-reset.log
-    libeufin-sandbox config \
-                     --currency "$CURRENCY" \
-                     --users-debt-limit 99999999 \
-                     --bank-debt-limit 99999999 \
-       default &> libeufin-sandbox-config.log
+STAGE="bank"
+
+if [ "1" = "$START_BANK" ]
+then
+    echo -n "Setting up bank database ... "
+    libeufin-bank dbinit \
+        -r \
+        -c "$CONF" \
+        &> libeufin-bank-reset.log
     echo "DONE"
-    echo "sandbox uses DB at $LIBEUFIN_SANDBOX_DB_CONNECTION"
-    echo -n "Launching sandbox ... "
-    export LIBEUFIN_SANDBOX_ADMIN_PASSWORD="secret"
-    libeufin-sandbox serve \
-      --port "$SANDBOX_PORT" \
-      > libeufin-sandbox-stdout.log \
-      2> libeufin-sandbox-stderr.log &
-    echo $! > libeufin-sandbox.pid
+    echo -n "Launching bank ... "
+    libeufin-bank serve \
+      -c "$CONF" \
+      > libeufin-bank-stdout.log \
+      2> libeufin-bank-stderr.log &
+    echo $! > libeufin-bank.pid
     echo "DONE"
-    export LIBEUFIN_SANDBOX_URL="http://localhost:$SANDBOX_PORT/"
+    echo -n "Waiting for Bank ..."
     OK="0"
-    echo -n "Waiting for Sandbox ..."
     for n in $(seq 1 100); do
         echo -n "."
         sleep "$DEFAULT_SLEEP"
         wget --timeout=1 \
              --tries=3 \
              --waitretry=0 \
+             -a wget-bank-check.log \
              -o /dev/null \
              -O /dev/null \
-             "$LIBEUFIN_SANDBOX_URL" || continue
+             "${BANK_URL}config" || continue
         OK="1"
         break
     done
     if [ "1" != "$OK" ]
     then
-        exit_skip "Failed to launch services (sandbox)"
+        exit_skip "Failed to launch services (bank)"
     fi
     echo "OK"
-    echo -n "Register Sandbox users ..."
-    # The specified IBAN and name must match the ones hard-coded into
-    # the C helper for the add-incoming call.  Without this value,
-    # Sandbox  won't find the target account to debit along a /add-incoming
-    # call.
-    register_sandbox_account fortytwo x "User42" FR7630006000011234567890189
-    register_sandbox_account fortythree x "Forty Three"
-    register_sandbox_account exchange x "Exchange Company"
-    register_sandbox_account tor x "Tor Project"
-    register_sandbox_account gnunet x "GNUnet"
-    register_sandbox_account tutorial x "Tutorial"
-    register_sandbox_account survey x "Survey"
-    echo " DONE"
-
-    echo -n "Fixing up exchange's PAYTO_URI in the config ..."
-    export LIBEUFIN_SANDBOX_USERNAME="exchange"
-    export LIBEUFIN_SANDBOX_PASSWORD="x"
-    EXCHANGE_PAYTO=$(libeufin-cli sandbox demobank info --bank-account exchange | jq --raw-output '.paytoUri')
-    taler-config -c "$CONF" -s "$USE_ACCOUNT" -o "PAYTO_URI" -V "$EXCHANGE_PAYTO"
+    echo -n "Set admin password..."
+    AUSER="admin"
+    APASS="secret"
+    libeufin-bank \
+      passwd \
+      -c "$CONF" \
+      "$AUSER" "$APASS" \
+      &> libeufin-bank-passwd.log
+    libeufin-bank \
+      edit-account \
+      -c "$CONF" \
+      --debit_threshold="$CURRENCY:1000000" \
+      "$AUSER" \
+      &> libeufin-bank-debit-threshold.log
     echo " OK"
-
-    echo -n "Setting this exchange as the bank's default ..."
-    libeufin-sandbox default-exchange "$EXCHANGE_URL" "$EXCHANGE_PAYTO"
-    echo " OK"
-
-    # Prepare EBICS: create Ebics host and Exchange subscriber.
-    # Shortly becoming admin to setup Ebics.
-    export LIBEUFIN_SANDBOX_USERNAME="admin"
-    export LIBEUFIN_SANDBOX_PASSWORD="secret"
-    echo -n "Create EBICS host at Sandbox.."
-    # FIXME-MS: || true should be removed after we make 'create' idempotent!
-    libeufin-cli sandbox \
-       --sandbox-url "$LIBEUFIN_SANDBOX_URL" \
-       ebicshost create --host-id talerebics &> libeufin-sandbox-ebicshost-create.log || true
-    echo "OK"
-    echo -n "Create exchange EBICS subscriber at Sandbox.."
-    # FIXME-MS: || true should be removed after we make 'new-ebicssubscriber' idempotent!
-    libeufin-cli sandbox \
-       demobank new-ebicssubscriber --host-id talerebics \
-       --user-id exchangeebics --partner-id talerpartner \
-       --bank-account exchange &> libeufin-sandbox-ebicsscubscriber.log || true
-    # that's a username _and_ a bank account name
-    echo "OK"
-    unset LIBEUFIN_SANDBOX_USERNAME
-    unset LIBEUFIN_SANDBOX_PASSWORD
 fi
 
 if [ "1" = "$START_NEXUS" ]
 then
-    echo "Setting up Nexus ..."
+    echo "Nexus currently not supported ..."
+fi
 
-    # Prepare Nexus, which is the side actually talking
-    # to the exchange.
-    LIBEUFIN_NEXUS_DB_CONNECTION=$(taler-config -c "$CONF" -s "libeufin-nexus" -o "DB_CONNECTION")
+if [ "1" = "$START_FAKEBANK" ]
+then
+    echo -n "Setting up fakebank ..."
+    $USE_VALGRIND taler-fakebank-run \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" \
+                  -n 4 \
+                  2> taler-fakebank-run.log &
+    echo " OK"
+fi
 
-    if [ -n "${PGHOST:+}" ]
-    then
-        EHOST=$(echo "$PGHOST" | sed -e "s/\//\\\\\//g")
-        LIBEUFIN_NEXUS_DB_CONNECTION=$(echo "$LIBEUFIN_NEXUS_DB_CONNECTION" | sed -e "s/\/var\/run\/postgresql/$EHOST/")
-        taler-config -c "$CONF" -s "libeufin-nexus" -o "DB_CONNECTION" -V "$LIBEUFIN_NEXUS_DB_CONNECTION"
-    fi
-    export LIBEUFIN_NEXUS_DB_CONNECTION
-    libeufin-nexus reset-tables \
-        &> libeufin-nexus-reset.log
-
-    # For convenience, username and password are
-    # identical to those used at the Sandbox.
-    echo -n "Create exchange Nexus user ..."
-    libeufin-nexus superuser exchange --password x
-    echo "OK"
-    libeufin-nexus serve --port "$NEXUS_PORT" \
-      2> libeufin-nexus-stderr.log \
-      > libeufin-nexus-stdout.log &
-    echo $! > libeufin-nexus.pid
-    export LIBEUFIN_NEXUS_URL="http://localhost:$NEXUS_PORT"
-    echo -n "Waiting for Nexus ..."
+if [[ "1" = "$START_NEXUS" || "1" = "$START_FAKEBANK" ]]
+then
+    echo -n "Waiting for the bank"
+    # Wait for bank to be available (usually the slowest)
     OK="0"
-    for n in $(seq 1 100); do
+    for n in $(seq 1 300)
+    do
         echo -n "."
         sleep "$DEFAULT_SLEEP"
-        wget --timeout=1 \
-             --tries=3 \
+        # bank
+        wget --tries=1 \
              --waitretry=0 \
+             --timeout=1 \
+             --user admin \
+             --password secret \
+             -a wget-bank-check.log \
              -o /dev/null \
              -O /dev/null \
-             "$LIBEUFIN_NEXUS_URL" || continue
+             "http://localhost:${BANK_PORT}/" || continue
         OK="1"
         break
     done
@@ -413,71 +438,42 @@ then
         exit_skip "Failed to launch services (bank)"
     fi
     echo " OK"
-
-    export LIBEUFIN_NEXUS_USERNAME=exchange
-    export LIBEUFIN_NEXUS_PASSWORD=x
-    echo -n "Creating a EBICS connection at Nexus ..."
-    # FIXME-MS: '||true' should be removed after we make 'new-ebics-connection' idempotent!
-    libeufin-cli connections new-ebics-connection \
-      --ebics-url "http://localhost:$SANDBOX_PORT/ebicsweb" \
-      --host-id talerebics \
-      --partner-id talerpartner \
-      --ebics-user-id exchangeebics \
-      talerconn
-    echo "OK"
-
-    echo -n "Setup EBICS keying ..."
-    libeufin-cli connections connect talerconn
-    echo "OK"
-    echo -n "Download bank account name from Sandbox ..."
-    libeufin-cli connections download-bank-accounts talerconn
-    echo "OK"
-    echo -n "Importing bank account info into Nexus ..."
-    libeufin-cli connections import-bank-account \
-      --offered-account-id exchange \
-      --nexus-bank-account-id exchange-nexus \
-      talerconn
-    echo "OK"
-    echo -n "Setup payments submission task..."
-    # Tries every second.
-    libeufin-cli accounts task-schedule \
-      --task-type submit \
-      --task-name exchange-payments \
-      --task-cronspec "* * *" \
-      exchange-nexus
-    echo "OK"
-    # Tries every second.  Ask C52
-    echo -n "Setup history fetch task..."
-    libeufin-cli accounts task-schedule \
-      --task-type fetch \
-      --task-name exchange-history \
-      --task-cronspec "* * *" \
-      --task-param-level report \
-      --task-param-range-type latest \
-      exchange-nexus
-    echo "OK"
-    # create Taler facade.
-    echo -n "Create the Taler facade at Nexus..."
-    libeufin-cli facades \
-      new-taler-wire-gateway-facade \
-      --currency TESTKUDOS --facade-name test-facade \
-      talerconn exchange-nexus
-    echo "OK"
-    # Facade schema: http://localhost:$NEXUS_PORT/facades/test-facade/taler-wire-gateway/
-    # FIXME: set the above URL automatically in the configuration?
 fi
 
+STAGE="accounts"
+
 if [ "1" = "$START_FAKEBANK" ]
 then
-    echo -n "Setting up fakebank ..."
-    $USE_VALGRIND taler-fakebank-run\
-                  -c "$CONF" \
-                  -L "$LOGLEVEL" \
-                  -n 4 \
-                  2> taler-fakebank-run.log &
-    echo " OK"
+    echo -n "Register Fakebank users ..."
+    register_fakebank_account fortytwo x
+    register_fakebank_account fortythree x
+    register_fakebank_account exchange x
+    register_fakebank_account tor x
+    register_fakebank_account gnunet x
+    register_fakebank_account tutorial x
+    register_fakebank_account survey x
+    echo " DONE"
 fi
 
+if [ "1" = "$START_BANK" ]
+then
+    echo -n "Register bank users ..."
+    # The specified IBAN and name must match the ones hard-coded into
+    # the C helper for the add-incoming call.  Without this value,
+    # libeufin-bank  won't find the target account to debit along a /add-incoming
+    # call.
+    register_bank_account fortytwo x "User42" FR7630006000011234567890189
+    register_bank_account fortythree x "Forty Three"
+    register_bank_account exchange x "Exchange Company" DE989651
+    register_bank_account tor x "Tor Project"
+    register_bank_account gnunet x "GNUnet"
+    register_bank_account tutorial x "Tutorial"
+    register_bank_account survey x "Survey"
+    echo " DONE"
+fi
+
+STAGE="exchange"
+
 if [ "1" = "$START_EXCHANGE" ]
 then
     echo -n "Starting exchange ..."
@@ -505,38 +501,85 @@ then
         taler-config -c "$CONF" -s exchange -o MASTER_PUBLIC_KEY -V "$MASTER_PUB"
     fi
     taler-exchange-dbinit -c "$CONF" --reset
-    $USE_VALGRIND taler-exchange-secmod-eddsa -c "$CONF" -L "$LOGLEVEL" 2> taler-exchange-secmod-eddsa.log &
-    $USE_VALGRIND taler-exchange-secmod-rsa -c "$CONF" -L "$LOGLEVEL" 2> taler-exchange-secmod-rsa.log &
-    $USE_VALGRIND taler-exchange-secmod-cs -c "$CONF" -L "$LOGLEVEL" 2> taler-exchange-secmod-cs.log &
-    $USE_VALGRIND taler-exchange-httpd -c "$CONF" -L "$LOGLEVEL" 2> taler-exchange-httpd.log &
+    $USE_VALGRIND taler-exchange-secmod-eddsa \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" \
+                  2> taler-exchange-secmod-eddsa.log &
+    $USE_VALGRIND taler-exchange-secmod-rsa \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" \
+                  2> taler-exchange-secmod-rsa.log &
+    $USE_VALGRIND taler-exchange-secmod-cs \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" \
+                  2> taler-exchange-secmod-cs.log &
+    $USE_VALGRIND taler-exchange-httpd \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" 2> taler-exchange-httpd.log &
     echo " DONE"
 fi
 
+STAGE="donau"
+
+if [ "1" = "$START_DONAU" ]
+then
+    echo -n "Starting Donau ..."
+    DONAU_PORT=$(donau-config -c "$CONF" -s DONAU -o PORT)
+    SERVE=$(donau-config -c "$CONF" -s DONAU -o SERVE)
+    if [ "${SERVE}" = "unix" ]
+    then
+        DONAU_URL=$(donau-config -c "$CONF" -s DONAU -o BASE_URL)
+    else
+        DONAU_URL="http://localhost:${DONAU_PORT}/"
+    fi
+    donau-dbinit -c "$CONF" --reset
+    $USE_VALGRIND taler-exchange-secmod-eddsa -c "$CONF" -L "$LOGLEVEL" -s donau 2> donau-secmod-eddsa.log &
+    $USE_VALGRIND taler-exchange-secmod-rsa -c "$CONF" -L "$LOGLEVEL" -s donau 2> donau-secmod-rsa.log &
+    $USE_VALGRIND taler-exchange-secmod-cs -c "$CONF" -L "$LOGLEVEL" -s donau 2> donau-secmod-cs.log &
+    $USE_VALGRIND donau-httpd -c "$CONF" -L "$LOGLEVEL" 2> donau-httpd.log &
+    echo " DONE"
+fi
+
+STAGE="wirewatch"
+
 if [ "1" = "$START_WIREWATCH" ]
 then
     echo -n "Starting wirewatch ..."
     $USE_VALGRIND taler-exchange-wirewatch \
                   --account="$USE_ACCOUNT" \
                   -c "$CONF" \
-                  --longpoll-timeout="1 s" \
+                  -L "$LOGLEVEL" \
+                  --longpoll-timeout="60 s" \
                   2> taler-exchange-wirewatch.log &
     echo " DONE"
 fi
 
+STAGE="aggregator"
+
 if [ "1" = "$START_AGGREGATOR" ]
 then
     echo -n "Starting aggregator ..."
-    $USE_VALGRIND taler-exchange-aggregator -c "$CONF" 2> taler-exchange-aggregator.log &
+    $USE_VALGRIND taler-exchange-aggregator \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" \
+                  2> taler-exchange-aggregator.log &
     echo " DONE"
 fi
 
+STAGE="transfer"
+
 if [ "1" = "$START_TRANSFER" ]
 then
     echo -n "Starting transfer ..."
-    $USE_VALGRIND taler-exchange-transfer -c "$CONF" 2> taler-exchange-transfer.log &
+    $USE_VALGRIND taler-exchange-transfer \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" \
+                  2> taler-exchange-transfer.log &
     echo " DONE"
 fi
 
+STAGE="merchant"
+
 if [ "1" = "$START_MERCHANT" ]
 then
     echo -n "Starting merchant ..."
@@ -558,22 +601,111 @@ then
         MERCHANT_PORT="$(taler-config -c "$CONF" -s MERCHANT -o PORT)"
         MERCHANT_URL="http://localhost:${MERCHANT_PORT}/"
     fi
-    taler-merchant-dbinit -c "$CONF" -L "$LOGLEVEL" --reset &> taler-merchant-dbinit.log
-    $USE_VALGRIND taler-merchant-httpd -c "$CONF" -L "$LOGLEVEL" 2> taler-merchant-httpd.log &
-    $USE_VALGRIND taler-merchant-webhook -c "$CONF" -L "$LOGLEVEL" 2> taler-merchant-webhook.log &
+    taler-merchant-dbinit \
+        -c "$CONF" \
+        --reset &> taler-merchant-dbinit.log
+    $USE_VALGRIND taler-merchant-httpd \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" 2> taler-merchant-httpd.log &
+    $USE_VALGRIND taler-merchant-webhook \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" 2> taler-merchant-webhook.log &
     echo " DONE"
+    if [ "1" = "$START_MERCHANT_WIREWATCH" ]
+    then
+       echo -n "Starting taler-merchant-wirewatch ..."
+       $USE_VALGRIND taler-merchant-wirewatch \
+                     -c "$CONF" \
+                     -L "$LOGLEVEL" \
+                     --persist \
+                     2> taler-merchant-wirewatch.log &
+       echo " DONE"
+    fi
+    if [ "1" = "$START_MERCHANT_EXCHANGE" ]
+    then
+        echo -n "Starting taler-merchant-exchange ..."
+        $USE_VALGRIND taler-merchant-exchange \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" 2> taler-merchant-exchange.log &
+        echo " DONE"
+    fi
+    if [ "1" = "$START_DEPOSITCHECK" ]
+    then
+        echo -n "Starting taler-merchant-depositcheck ..."
+        $USE_VALGRIND taler-merchant-depositcheck \
+                      -c "$CONF" \
+                      -L "$LOGLEVEL" 2> taler-merchant-depositcheck.log &
+        echo " DONE"
+    fi
 fi
 
+STAGE="sync"
+
 if [ "1" = "$START_BACKUP" ]
 then
     echo -n "Starting sync ..."
     SYNC_PORT=$(taler-config -c "$CONF" -s SYNC -o PORT)
-    SYNC_URL="http://localhost:${SYNC_PORT}/"
+    SERVE=$(taler-config -c "$CONF" -s SYNC -o SERVE)
+    if [ "${SERVE}" = "unix" ]
+    then
+        SYNC_URL=$(taler-config -c "$CONF" -s SYNC -o BASE_URL)
+    else
+        SYNC_URL="http://localhost:${SYNC_PORT}/"
+    fi
     sync-dbinit -c "$CONF" --reset
-    $USE_VALGRIND sync-httpd -c "$CONF" -L "$LOGLEVEL" 2> sync-httpd.log &
+    $USE_VALGRIND sync-httpd \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" \
+                  2> sync-httpd.log &
     echo " DONE"
 fi
 
+STAGE="challenger"
+
+if [ "1" = "$START_CHALLENGER" ]
+then
+    echo -n "Starting challenger ..."
+    CHALLENGER_PORT=$(challenger-config -c "$CONF" -s CHALLENGER -o PORT)
+    SERVE=$(taler-config -c "$CONF" -s CHALLENGER -o SERVE)
+    if [ "${SERVE}" = "unix" ]
+    then
+        CHALLENGER_URL=$(taler-config -c "$CONF" -s CHALLENGER -o BASE_URL)
+    else
+        CHALLENGER_URL="http://localhost:${CHALLENGER_PORT}/"
+    fi
+    challenger-dbinit \
+        -c "$CONF" \
+        --reset
+    $USE_VALGRIND challenger-httpd \
+                  -c "$CONF" \
+                  -L "$LOGLEVEL" \
+                  2> challenger-httpd.log &
+    echo " DONE"
+    for SECTION in $(taler-config -c "$CONF" -S | grep kyc-provider)
+    do
+        LOGIC=$(taler-config -c "$CONF" -s "$SECTION" -o "LOGIC")
+        if [ "${LOGIC}" = "oauth2" ]
+        then
+            INFO=$(taler-config -c "$CONF" -s "$SECTION" -o "KYC_OAUTH2_INFO_URL")
+            if [ "${CHALLENGER_URL}info" = "$INFO" ]
+            then
+                echo -n "Enabling Challenger client for $SECTION"
+                CLIENT_SECRET=$(taler-config -c "$CONF" -s "$SECTION" -o "KYC_OAUTH2_CLIENT_SECRET")
+                RFC_8959_PREFIX="secret-token:"
+                if ! echo "${CLIENT_SECRET}" | grep ^${RFC_8959_PREFIX} > /dev/null
+                then
+                    exit_fail "Client secret does not begin with '${RFC_8959_PREFIX}'"
+                fi
+                REDIRECT_URI="${EXCHANGE_URL}kyc-proof/kyc-provider-example-challeger"
+                CLIENT_ID=$(challenger-admin --add="${CLIENT_SECRET}" --quiet "${REDIRECT_URI}")
+                taler-config -c "$CONF" -s "$SECTION" -o KYC_OAUTH2_CLIENT_ID -V "$CLIENT_ID"
+                echo " DONE"
+            fi
+        fi
+    done
+fi
+
+STAGE="auditor"
 
 if [ "1" = "$START_AUDITOR" ]
 then
@@ -589,59 +721,26 @@ then
     fi
     AUDITOR_PUB=$(gnunet-ecc -p "${AUDITOR_PRIV_FILE}")
     MAPUB=${MASTER_PUB:-$(taler-config -c "$CONF" -s exchange -o MASTER_PUBLIC_KEY)}
-    taler-auditor-dbinit -c "$CONF" --reset
-    taler-auditor-exchange -c "$CONF" -m "$MAPUB" -u "$EXCHANGE_URL"
-    $USE_VALGRIND taler-auditor-httpd -L "$LOGLEVEL" -c "$CONF" 2> taler-auditor-httpd.log &
-    echo " DONE"
-fi
-
-if [[ "1" = "$START_NEXUS" || "1" = "$START_FAKEBANK" ]]
-then
-    echo -n "Waiting for the bank"
-    # Wait for bank to be available (usually the slowest)
-    OK="0"
-    for n in $(seq 1 300)
-    do
-        echo -n "."
-        sleep "$DEFAULT_SLEEP"
-        # bank
-        wget --tries=1 \
-             --waitretry=0 \
-             --timeout=1 \
-             --user admin \
-             --password secret \
-             "http://localhost:${BANK_PORT}/" \
-             -o /dev/null \
-             -O /dev/null >/dev/null || continue
-        OK="1"
-        break
-    done
-    if [ "1" != "$OK" ]
-    then
-        exit_skip "Failed to launch services (bank)"
-    fi
-    echo " OK"
-fi
-
-if [ "1" = "$START_FAKEBANK" ]
-then
-    echo -n "Register Fakebank users ..."
-    register_fakebank_account fortytwo x
-    register_fakebank_account fortythree x
-    register_fakebank_account exchange x
-    register_fakebank_account tor x
-    register_fakebank_account gnunet x
-    register_fakebank_account tutorial x
-    register_fakebank_account survey x
+    taler-auditor-dbinit \
+        -c "$CONF" \
+        --reset
+    $USE_VALGRIND taler-auditor-httpd \
+                  -L "$LOGLEVEL" \
+                  -c "$CONF" 2> taler-auditor-httpd.log &
+#    $USE_VALGRIND taler-helper-auditor-deposits \
+#                  -L "$LOGLEVEL" \
+#                  -c "$CONF" 2> taler-helper-auditor.log &
     echo " DONE"
 fi
 
+STAGE="wait"
 
 echo -n "Waiting for Taler services ..."
 # Wait for all other taler services to be available
 E_DONE=0
 M_DONE=0
 S_DONE=0
+K_DONE=0
 A_DONE=0
 for n in $(seq 1 20)
 do
@@ -680,6 +779,17 @@ do
             -O /dev/null >/dev/null || continue
         S_DONE=1
     fi
+    if [ "0" = "$K_DONE" ] && [ "1" = "$START_CHALLENGER" ]
+    then
+        echo -n "K"
+        wget \
+            --tries=1 \
+            --timeout=1 \
+            "${CHALLENGER_URL}config" \
+            -o /dev/null \
+            -O /dev/null >/dev/null || continue
+        K_DONE=1
+    fi
     if [ "0" = "$A_DONE" ] && [ "1" = "$START_AUDITOR" ]
     then
         echo -n "A"
@@ -734,7 +844,7 @@ then
       download \
       sign \
       wire-fee now "$WIRE_DOMAIN" "$CURRENCY:0.01" "$CURRENCY:0.01" \
-      global-fee now "$CURRENCY:0.01" "$CURRENCY:0.01" "$CURRENCY:0.01" 1h 1year 5 \
+      global-fee now "$CURRENCY:0.01" "$CURRENCY:0.01" "$CURRENCY:0.0" 1h 1year 5 \
       upload &> taler-exchange-offline.log
     echo "OK"
     ENABLED=$(taler-config -c "$CONF" -s "$USE_ACCOUNT" -o "ENABLE_CREDIT")
@@ -767,18 +877,19 @@ then
         sleep "$DEFAULT_SLEEP"
         wget \
             --tries=1 \
-            --timeout=1 \
+            --timeout=5 \
             "${EXCHANGE_URL}keys" \
+            -a wget-keys-check.log \
             -o /dev/null \
             -O "$LAST_RESPONSE" \
-             >/dev/null || continue
+            >/dev/null || continue
         OK="1"
         break
     done
     if [ "1" != "$OK" ]
     then
         cat "$LAST_RESPONSE"
-        exit_fail " Failed to setup keys"
+        exit_fail " Failed to fetch ${EXCHANGE_URL}keys"
     fi
     rm "$LAST_RESPONSE"
     echo " OK"
@@ -794,6 +905,8 @@ then
     echo " OK"
 fi
 
+STAGE="ready"
+
 # Signal caller that we are ready.
 echo "<<READY>>"
 
@@ -809,6 +922,8 @@ else
     read
 fi
 
+STAGE="exiting"
+
 echo "Taler unified setup terminating!" >&2
 EXIT_STATUS=0
 exit "$EXIT_STATUS"
