@@ -201,8 +201,6 @@ run (void *cls,
   };
 
   struct TALER_TESTING_Command wallet_kyc[] = {
-    TALER_TESTING_cmd_oauth ("start-oauth-service",
-                             6666),
     TALER_TESTING_cmd_wallet_kyc_get ("wallet-kyc-fail",
                                       NULL,
                                       "EUR:1000000",
@@ -294,6 +292,10 @@ run (void *cls,
       "p2p_withdraw-coin-1",
       "EUR:1.01",
       NULL),
+    TALER_TESTING_cmd_coin_history ("coin-history-purse-with-deposit",
+                                    "p2p_withdraw-coin-1#0",
+                                    "EUR:3.99",
+                                    MHD_HTTP_OK),
     TALER_TESTING_cmd_purse_poll (
       "push-poll-purse-before-merge",
       MHD_HTTP_OK,
@@ -388,6 +390,10 @@ run (void *cls,
       "p2p_withdraw-coin-1",
       "EUR:1.01",
       NULL),
+    TALER_TESTING_cmd_coin_history ("coin-history-purse-pull-deposit",
+                                    "p2p_withdraw-coin-1#0",
+                                    "EUR:2.98",
+                                    MHD_HTTP_OK),
     TALER_TESTING_cmd_purse_poll_finish (
       "pull-deposit-purse-poll-finish",
       GNUNET_TIME_relative_multiply (
@@ -395,14 +401,14 @@ run (void *cls,
         5),
       "pull-poll-purse-before-deposit"),
     TALER_TESTING_cmd_status (
-      "pull-check-post-merge-reserve-balance-get",
+      "pull-check-post-merge-reserve-balance-get-2",
       "p2p_create-reserve-3",
-      "EUR:1.02",
+      "EUR:1.03",
       MHD_HTTP_OK),
     TALER_TESTING_cmd_reserve_history (
-      "push-check-post-merge-reserve-balance-post",
+      "push-check-post-merge-reserve-balance-post-2",
       "p2p_create-reserve-3",
-      "EUR:1.02",
+      "EUR:1.03",
       MHD_HTTP_OK),
     TALER_TESTING_cmd_end ()
   };
