@@ -307,6 +307,13 @@ static struct GNUNET_TIME_Timestamp now_tmp;
  */
 static char *keydir;
 
+/**
+ * Name of the configuration section prefix to use.  Usually either "taler-exchange" or
+ * "donau". The actual configuration section will then be
+ * "$SECTION-secmod-rsa".
+ */
+static char *section;
+
 /**
  * How much should coin creation (@e duration_withdraw) duration overlap
  * with the next denomination?  Basically, the starting time of two
@@ -409,15 +416,13 @@ generate_response (struct DenominationKey *dk)
  * Do the actual signing work.
  *
  * @param h_rsa key to sign with
- * @param blinded_msg message to sign
- * @param blinded_msg_size number of bytes in @a blinded_msg
+ * @param bm blinded message to sign
  * @param[out] rsa_signaturep set to the RSA signature
  * @return #TALER_EC_NONE on success
  */
 static enum TALER_ErrorCode
 do_sign (const struct TALER_RsaPubHashP *h_rsa,
-         const void *blinded_msg,
-         size_t blinded_msg_size,
+         const struct GNUNET_CRYPTO_RsaBlindedMessage *bm,
          struct GNUNET_CRYPTO_RsaSignature **rsa_signaturep)
 {
   struct DenominationKey *dk;
@@ -447,15 +452,14 @@ do_sign (const struct TALER_RsaPubHashP *h_rsa,
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Received request to sign over %u bytes with key %s\n",
-              (unsigned int) blinded_msg_size,
+              (unsigned int) bm->blinded_msg_size,
               GNUNET_h2s (&h_rsa->hash));
   GNUNET_assert (dk->rc < UINT_MAX);
   dk->rc++;
   GNUNET_assert (0 == pthread_mutex_unlock (&keys_lock));
   rsa_signature
     = GNUNET_CRYPTO_rsa_sign_blinded (dk->denom_priv,
-                                      blinded_msg,
-                                      blinded_msg_size);
+                                      bm);
   GNUNET_assert (0 == pthread_mutex_lock (&keys_lock));
   GNUNET_assert (dk->rc > 0);
   dk->rc--;
@@ -548,14 +552,15 @@ static enum GNUNET_GenericReturnValue
 handle_sign_request (struct TES_Client *client,
                      const struct TALER_CRYPTO_SignRequest *sr)
 {
-  const void *blinded_msg = &sr[1];
-  size_t blinded_msg_size = ntohs (sr->header.size) - sizeof (*sr);
+  struct GNUNET_CRYPTO_RsaBlindedMessage bm = {
+    .blinded_msg = (void *) &sr[1],
+    .blinded_msg_size = ntohs (sr->header.size) - sizeof (*sr)
+  };
   struct GNUNET_CRYPTO_RsaSignature *rsa_signature;
   enum TALER_ErrorCode ec;
 
   ec = do_sign (&sr->h_rsa,
-                blinded_msg,
-                blinded_msg_size,
+                &bm,
                 &rsa_signature);
   if (TALER_EC_NONE != ec)
   {
@@ -660,12 +665,13 @@ worker (void *cls)
     {
       struct BatchJob *bj = w->job;
       const struct TALER_CRYPTO_SignRequest *sr = bj->sr;
-      const void *blinded_msg = &sr[1];
-      size_t blinded_msg_size = ntohs (sr->header.size) - sizeof (*sr);
+      struct GNUNET_CRYPTO_RsaBlindedMessage bm = {
+        .blinded_msg = (void *) &sr[1],
+        .blinded_msg_size = ntohs (sr->header.size) - sizeof (*sr)
+      };
 
       bj->ec = do_sign (&sr->h_rsa,
-                        blinded_msg,
-                        blinded_msg_size,
+                        &bm,
                         &bj->rsa_signature);
       sem_up (&bj->sem);
       w->job = NULL;
@@ -880,8 +886,8 @@ setup_key (struct DenominationKey *dk,
   }
   buf_size = GNUNET_CRYPTO_rsa_private_key_encode (priv,
                                                    &buf);
-  TALER_rsa_pub_hash (pub,
-                      &dk->h_rsa);
+  GNUNET_CRYPTO_rsa_public_key_hash (pub,
+                                     &dk->h_rsa.hash);
   GNUNET_asprintf (&dk->filename,
                    "%s/%s/%llu",
                    keydir,
@@ -1255,6 +1261,7 @@ create_key (struct Denomination *denom,
   struct GNUNET_TIME_Timestamp anchor;
 
   anchor = now;
+  // FIXME: round down to multiple of 'anchor_round' value from configuration
   if (NULL != denom->keys_tail)
   {
     struct GNUNET_TIME_Absolute abs;
@@ -1545,8 +1552,8 @@ parse_key (struct Denomination *denom,
     dk->denom = denom;
     dk->anchor = anchor;
     dk->filename = GNUNET_strdup (filename);
-    TALER_rsa_pub_hash (pub,
-                        &dk->h_rsa);
+    GNUNET_CRYPTO_rsa_public_key_hash (pub,
+                                       &dk->h_rsa.hash);
     dk->denom_pub = pub;
     generate_response (dk);
     if (GNUNET_OK !=
@@ -1723,7 +1730,11 @@ parse_denomination_cfg (const struct GNUNET_CONFIGURATION_Handle *cfg,
                         struct Denomination *denom)
 {
   unsigned long long rsa_keysize;
+  char *secname;
 
+  GNUNET_asprintf (&secname,
+                   "%s-secmod-rsa",
+                   section);
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_time (cfg,
                                            ct,
@@ -1733,6 +1744,7 @@ parse_denomination_cfg (const struct GNUNET_CONFIGURATION_Handle *cfg,
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                ct,
                                "DURATION_WITHDRAW");
+    GNUNET_free (secname);
     return GNUNET_SYSERR;
   }
   if (GNUNET_TIME_relative_cmp (overlap_duration,
@@ -1740,9 +1752,10 @@ parse_denomination_cfg (const struct GNUNET_CONFIGURATION_Handle *cfg,
                                 denom->duration_withdraw))
   {
     GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
-                               "taler-exchange-secmod-rsa",
+                               section,
                                "OVERLAP_DURATION",
                                "Value given must be smaller than value for DURATION_WITHDRAW!");
+    GNUNET_free (secname);
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
@@ -1754,6 +1767,7 @@ parse_denomination_cfg (const struct GNUNET_CONFIGURATION_Handle *cfg,
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                ct,
                                "RSA_KEYSIZE");
+    GNUNET_free (secname);
     return GNUNET_SYSERR;
   }
   if ( (rsa_keysize > 4 * 2048) ||
@@ -1763,8 +1777,10 @@ parse_denomination_cfg (const struct GNUNET_CONFIGURATION_Handle *cfg,
                                ct,
                                "RSA_KEYSIZE",
                                "Given RSA keysize outside of permitted range [1024,8192]\n");
+    GNUNET_free (secname);
     return GNUNET_SYSERR;
   }
+  GNUNET_free (secname);
   denom->rsa_keysize = (unsigned int) rsa_keysize;
   denom->section = GNUNET_strdup (ct);
   return GNUNET_OK;
@@ -1879,28 +1895,36 @@ load_denominations (void *cls,
 static enum GNUNET_GenericReturnValue
 load_durations (const struct GNUNET_CONFIGURATION_Handle *cfg)
 {
+  char *secname;
+
+  GNUNET_asprintf (&secname,
+                   "%s-secmod-rsa",
+                   section);
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_time (cfg,
-                                           "taler-exchange-secmod-rsa",
+                                           secname,
                                            "OVERLAP_DURATION",
                                            &overlap_duration))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               "taler-exchange-secmod-rsa",
+                               secname,
                                "OVERLAP_DURATION");
+    GNUNET_free (secname);
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_time (cfg,
-                                           "taler-exchange-secmod-rsa",
+                                           secname,
                                            "LOOKAHEAD_SIGN",
                                            &lookahead_sign))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               "taler-exchange-secmod-rsa",
+                               secname,
                                "LOOKAHEAD_SIGN");
+    GNUNET_free (secname);
     return GNUNET_SYSERR;
   }
+  GNUNET_free (secname);
   return GNUNET_OK;
 }
 
@@ -1944,6 +1968,7 @@ run (void *cls,
     .updater = rsa_update_client_keys,
     .init = rsa_client_init
   };
+  char *secname;
 
   (void) cls;
   (void) args;
@@ -1958,27 +1983,40 @@ run (void *cls,
     /* get current time again, we may be timetraveling! */
     now = GNUNET_TIME_timestamp_get ();
   }
+  GNUNET_asprintf (&secname,
+                   "%s-secmod-rsa",
+                   section);
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_filename (cfg,
-                                               "taler-exchange-secmod-rsa",
+                                               secname,
                                                "KEY_DIR",
                                                &keydir))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               "taler-exchange-secmod-rsa",
+                               secname,
                                "KEY_DIR");
+    GNUNET_free (secname);
     global_ret = EXIT_NOTCONFIGURED;
     return;
   }
+  GNUNET_free (secname);
   if (GNUNET_OK !=
       load_durations (cfg))
   {
     global_ret = EXIT_NOTCONFIGURED;
     return;
   }
-  global_ret = TES_listen_start (cfg,
-                                 "taler-exchange-secmod-rsa",
-                                 &cb);
+  {
+    char *secname;
+
+    GNUNET_asprintf (&secname,
+                     "%s-secmod-rsa",
+                     section);
+    global_ret = TES_listen_start (cfg,
+                                   secname,
+                                   &cb);
+    GNUNET_free (secname);
+  }
   if (0 != global_ret)
     return;
   sem_init (&worker_sem,
@@ -2052,6 +2090,11 @@ main (int argc,
       char **argv)
 {
   struct GNUNET_GETOPT_CommandLineOption options[] = {
+    GNUNET_GETOPT_option_string ('s',
+                                 "section",
+                                 "SECTION",
+                                 "name of the configuration section prefix to use, default is 'taler'",
+                                 &section),
     GNUNET_GETOPT_option_timetravel ('T',
                                      "timetravel"),
     GNUNET_GETOPT_option_timestamp ('t',
@@ -2070,7 +2113,7 @@ main (int argc,
 
   /* Restrict permissions for the key files that we create. */
   (void) umask (S_IWGRP | S_IROTH | S_IWOTH | S_IXOTH);
-
+  section = GNUNET_strdup ("taler-exchange");
   /* force linker to link against libtalerutil; if we do
    not do this, the linker may "optimize" libtalerutil
    away and skip #TALER_OS_init(), which we do need */
